﻿/*
	Name:			XenoN Core Dark Issue Header
    Version:        0.8.9.0 Dark Issue
    Update:			2009-10-17
	Copyright:		Copyright © 2007-2009 XenoN Core by PsichiX. All rights reserved.
	Author:			PsichiX
	Website:		http://www.xenon.psichix.com
	Description:	XenoN Core 4D World Engine
*/

/*
	== EN ==
	This file is part of Xenon Core Engine.
	You may distribute it further, but you can not modify it.
	Please do not use in modified form.
	The principles of XenoN Core License available in the file LICENSE_CORE_EN.TXT or visit: http://www.xenon.psichix.com/download.php.

	== PL ==
	Ten plik jest czescia XenoN Core Engine.
	Mozesz go rozpowszechniac dalej, jednak nie mozesz go modyfikowac.
	Nalezy uzytkowac w nie zmodyfikowanej formie.
	Nalezy przestrzegac zasad Licencji XenoN Core dostepnej w pliku LICENSE_CORE_PL.TXT oraz na stronie: http://www.xenon.psichix.com/download.php.
*/

/*
	MODULES:
		Photon		- Graphics
		Chaos		- Physics
		Psyche		- Artificial Intelligence
		TimeBridge	- Time Control
		Echo		- Music & Sounds
		Ether		- Network
*/

#ifndef _XENON_CORE_H_
#define _XENON_CORE_H_

#include <windows.h>
#include <stdlib.h>
#include <memory.h>
#include <stdarg.h>
#include <malloc.h>
#include <math.h>
#ifdef XE_COMPILE_EXPORT
#include "XenonCoreGL.h"
#endif /* XE_COMPILE_EXPORT */

//! Przestrzen nazw XenoN Core
namespace XeCore
{

//! Przestrzen nazw modulu Photon
namespace Photon{}
//! Przestrzen nazw modulu Chaos
namespace Chaos{}

/*--- Dll Mode ---*/
#ifndef XE_COMPILE_EXPORT
#ifndef XE_NOT_USE_LIBRARY
#pragma comment(lib,"XeCore/XenonCore.lib")
#endif /* XE_NOT_USE_LIBRARY */
#endif /* XE_COMPILE_EXPORT */
#ifdef XE_COMPILE_EXPORT
#define exportdll extern "C" __declspec(dllexport)
#else
#define exportdll extern "C" __declspec(dllimport)
#endif /* XE_COMPILE_EXPORT */

/*--- Misc Definitions ---*/
#define PI 3.1415926535897932384626433832795
#define cstring char*

/*--- XE_RES_* ---*/
#define XE_RES_XENON	1024

#define XE_RES_SPLASH	1
#define XE_RES_INTRO	2

/*--- XE_BASE_* ---*/
#define XE_BASE_BIN		2
#define XE_BASE_OCT		8
#define XE_BASE_DEC		10
#define XE_BASE_HEX		16

/*--- XE_FLAG_* ---*/
#define	XE_FLAG_EMPTY			0x00000000
#define	XE_FLAG_ALL				0xFFFFFFFF
#define	XE_FLAG_SWAP			0x00000001
#define	XE_FLAG_COLORBUFF		0x00000002
#define	XE_FLAG_DEPTHBUFF		0x00000004
#define	XE_FLAG_STENCILBUFF		0x00000008
#define	XE_FLAG_ACCUMBUFF		0x00000010
#define	XE_FLAG_TRANSFORMIDENT	0x00000020
#define XE_FLAG_ITALIC			0x00000001
#define XE_FLAG_UNDERLINE		0x00000002
#define XE_FLAG_STRIKEOUT		0x00000004

/*--- XE_ESTATE ---*/
//! Zmienne stanu silnika
#define XE_NOONE (XE_ESTATE)-1
#define XE_NULL (XE_ESTATE)0
enum XE_ESTATE
{
XE_FALSE,
XE_TRUE,
XE_POINTS,
XE_LINES,
XE_LINELOOP,
XE_LINESTRIP,
XE_TRIANGLES,
XE_TRIANGLESTRIP,
XE_TRIANGLEFAN,
XE_QUADS,
XE_QUADSTRIP,
XE_POLYGON,
XE_RENDER_TARGET,
XE_FRAMEBUFFER,
XE_SHADER,
XE_TEXTURE,
XE_DISPLAYLIST,
XE_RENDERQUEUE,
XE_SUBSTANCE,
XE_RAY,
XE_COVALENCE_FORCE,
XE_FORCE,
XE_SURFACE,
XE_PARTICLE,
XE_GLOBAL_FORCE,
XE_BLACK_HOLE,
XE_WAVE_FORCE,
XE_PSYCHE,
XE_RENDER_TARGET_COUNT,
XE_FRAMEBUFFER_COUNT,
XE_SHADER_COUNT,
XE_TEXTURE_COUNT,
XE_DISPLAYLIST_COUNT,
XE_RENDERQUEUE_COUNT,
XE_SUBSTANCE_COUNT,
XE_RAY_COUNT,
XE_COVALENCE_FORCE_COUNT,
XE_FORCE_COUNT,
XE_SURFACE_COUNT,
XE_PARTICLE_COUNT,
XE_GLOBAL_FORCE_COUNT,
XE_BLACK_HOLE_COUNT,
XE_WAVE_FORCE_COUNT,
XE_PSYCHE_COUNT,
XE_1D,
XE_2D,
XE_3D,
XE_4D,
XE_CUBE,
XE_CUBE_POS_X,
XE_CUBE_NEG_X,
XE_CUBE_POS_Y,
XE_CUBE_NEG_Y,
XE_CUBE_POS_Z,
XE_CUBE_NEG_Z,
XE_FRONT_LEFT,
XE_FRONT_RIGHT,
XE_BACK_LEFT,
XE_BACK_RIGHT,
XE_WINDOW_HANDLE,
XE_DISPLAY_CONTEXT,
XE_RENDER_CONTEXT,
XE_ENGINE_NAME,
XE_ENGINE_CODENAME,
XE_ENGINE_CREATOR,
XE_ENGINE_VERSION,
XE_ENGINE_DESCRIPTION,
XE_ENGINE_COPYRIGHT,
XE_ENGINE_OWNER,
XE_ENGINE_COMPILE_OPTIONS,
XE_DRAW_TO_WINDOW,
XE_RENDER_VENDOR,
XE_RENDER_RENDERER,
XE_RENDER_VERSION,
XE_RENDER_EXTENSIONS,
XE_RENDER_MODE,
XE_RENDER_FORMAT_RGBA,
XE_RENDER_FORMAT_DEPTH,
XE_RENDER_FORMAT_STENCIL,
XE_RENDER_VSYNC_MODE,
XE_RENDER_X,
XE_RENDER_Y,
XE_RENDER_WIDTH,
XE_RENDER_HEIGHT,
XE_RENDER_ANGLE,
XE_RENDER_NEAR,
XE_RENDER_FAR,
XE_RENDER_ASPECT,
XE_RENDER_ORIENTATION,
XE_RENDER_CLOCKWISE,
XE_RENDER_COUNTERCLOCKWISE,
XE_RENDER_TEXT_SETTINGS,
XE_RENDER_TEXT_SIZE_X,
XE_RENDER_TEXT_SIZE_Y,
XE_RENDER_TEXT_SEP_X,
XE_RENDER_TEXT_SEP_Y,
XE_RENDER_TEXT_NEWLINE,
XE_RENDER_MASK,
XE_RENDER_MASK_DEPTH,
XE_RENDER_MASK_RED,
XE_RENDER_MASK_GREEN,
XE_RENDER_MASK_BLUE,
XE_RENDER_MASK_ALPHA,
XE_RENDER_DITHER,
XE_RENDER_TEXTURING_2D,
XE_RENDER_NORMALIZE,
XE_RENDER_FACE_FRONT,
XE_RENDER_FACE_BACK,
XE_RENDER_FACE_FRONT_AND_BACK,
XE_RENDER_LIGHTING,
XE_RENDER_LIGHT_SOURCE,
XE_RENDER_LIGHT_AMBIENT,
XE_RENDER_LIGHT_DIFFUSE,
XE_RENDER_LIGHT_SPECULAR,
XE_RENDER_LIGHT_POSITION,
XE_RENDER_LIGHT_SPOT_DIRECTION,
XE_RENDER_LIGHT_SPOT_EXPONENT,
XE_RENDER_LIGHT_SPOT_CUTOFF,
XE_RENDER_LIGHT_MODEL_AMBIENT,
XE_RENDER_MATERIAL_AMBIENT,
XE_RENDER_MATERIAL_DIFFUSE,
XE_RENDER_MATERIAL_SPECULAR,
XE_RENDER_MATERIAL_EMISSION,
XE_RENDER_MATERIAL_SHININESS,
XE_RENDER_DEPTH_TEST,
XE_RENDER_DEPTH_TYPE,
XE_RENDER_DEPTH_NEVER,
XE_RENDER_DEPTH_LESS,
XE_RENDER_DEPTH_EQUAL,
XE_RENDER_DEPTH_LEQUAL,
XE_RENDER_DEPTH_GREATER,
XE_RENDER_DEPTH_NOTEQUAL,
XE_RENDER_DEPTH_GEQUAL,
XE_RENDER_DEPTH_ALWAYS,
XE_RENDER_ALPHA_TEST,
XE_RENDER_ALPHA_TYPE,
XE_RENDER_ALPHA_REF,
XE_RENDER_ALPHA_NEVER,
XE_RENDER_ALPHA_LESS,
XE_RENDER_ALPHA_EQUAL,
XE_RENDER_ALPHA_LEQUAL,
XE_RENDER_ALPHA_GREATER,
XE_RENDER_ALPHA_NOTEQUAL,
XE_RENDER_ALPHA_GEQUAL,
XE_RENDER_ALPHA_ALWAYS,
XE_RENDER_CULL_FACE,
XE_RENDER_CULL_TYPE,
XE_RENDER_CULL_FRONT,
XE_RENDER_CULL_BACK,
XE_RENDER_CULL_FRONT_AND_BACK,
XE_RENDER_BLEND,
XE_RENDER_BLEND_TYPE,
XE_RENDER_BLEND_SRC,
XE_RENDER_BLEND_DST,
XE_RENDER_BLEND_ZERO,
XE_RENDER_BLEND_ONE,
XE_RENDER_BLEND_SRC_COLOR,
XE_RENDER_BLEND_ONE_MINUS_SRC_COLOR,
XE_RENDER_BLEND_DST_COLOR,
XE_RENDER_BLEND_ONE_MINUS_DST_COLOR,
XE_RENDER_BLEND_SRC_ALPHA,
XE_RENDER_BLEND_ONE_MINUS_SRC_ALPHA,
XE_RENDER_BLEND_DST_ALPHA,
XE_RENDER_BLEND_ONE_MINUS_DST_ALPHA,
XE_RENDER_BLEND_SRC_ALPHA_SATURATE,
XE_RENDER_BLEND_CONSTANT_COLOR,
XE_RENDER_BLEND_ONE_MINUS_CONSTANT_COLOR,
XE_RENDER_BLEND_CONSTANT_ALPHA,
XE_RENDER_BLEND_ONE_MINUS_CONSTANT_ALPHA,
XE_RENDER_FOG,
XE_RENDER_FOG_COLOR,
XE_RENDER_FOG_START,
XE_RENDER_FOG_END,
XE_RENDER_FOG_DENSITY,
XE_RENDER_FOG_MODE,
XE_RENDER_FOG_TYPE,
XE_RENDER_FOG_DISTANCE,
XE_RENDER_FOG_COORD,
XE_RENDER_FOG_LINEAR,
XE_RENDER_FOG_EXP,
XE_RENDER_FOG_EXP2,
XE_RENDER_STENCIL_TEST,
XE_RENDER_STENCIL_CLEAR,
XE_RENDER_STENCIL_TYPE,
XE_RENDER_STENCIL_FUNC,
XE_RENDER_STENCIL_FUNC_NEVER,
XE_RENDER_STENCIL_FUNC_LESS,
XE_RENDER_STENCIL_FUNC_EQUAL,
XE_RENDER_STENCIL_FUNC_LEQUAL,
XE_RENDER_STENCIL_FUNC_GREATER,
XE_RENDER_STENCIL_FUNC_NOTEQUAL,
XE_RENDER_STENCIL_FUNC_GEQUAL,
XE_RENDER_STENCIL_FUNC_ALWAYS,
XE_RENDER_STENCIL_REF,
XE_RENDER_STENCIL_MASK,
XE_RENDER_STENCIL_OPERATION,
XE_RENDER_STENCIL_KEEP,
XE_RENDER_STENCIL_ZERO,
XE_RENDER_STENCIL_REPLACE,
XE_RENDER_STENCIL_INCR,
XE_RENDER_STENCIL_DECR,
XE_RENDER_STENCIL_INVERT,
XE_RENDER_ACCUM,
XE_RENDER_ACCUM_COLOR,
XE_RENDER_ACCUM_ACCUM,
XE_RENDER_ACCUM_LOAD,
XE_RENDER_ACCUM_RETURN,
XE_RENDER_ACCUM_MULT,
XE_RENDER_ACCUM_ADD,
XE_RENDER_SCISSOR,
XE_RENDER_SCISSOR_MODE,
XE_HINT_FOG,
XE_HINT_PERSPECTIVE_CORRECTION,
XE_HINT_POINT_SMOOTH,
XE_HINT_LINE_SMOOTH,
XE_HINT_POLYGON_SMOOTH,
XE_HINT_NICEST,
XE_HINT_FASTEST,
XE_HINT_DONT_CARE,
XE_TEXTURE_PARAM_MIN_FILTER,
XE_TEXTURE_PARAM_MAG_FILTER,
XE_TEXTURE_PARAM_WRAP_S,
XE_TEXTURE_PARAM_WRAP_T,
XE_TEXTURE_PARAM_PRIORITY,
XE_TEXTURE_PARAM_NEAREST,
XE_TEXTURE_PARAM_LINEAR,
XE_TEXTURE_PARAM_NEAREST_MIPMAP_NEAREST,
XE_TEXTURE_PARAM_LINEAR_MIPMAP_NEAREST,
XE_TEXTURE_PARAM_NEAREST_MIPMAP_LINEAR,
XE_TEXTURE_PARAM_LINEAR_MIPMAP_LINEAR,
XE_TEXTURE_PARAM_REPEAT,
XE_TEXTURE_PARAM_CLAMP,
XE_TEXTURE_MODE_ADD,
XE_TEXTURE_MODE_MODULATE,
XE_TEXTURE_MODE_DECAL,
XE_TEXTURE_MODE_BLEND,
XE_TEXTURE_MODE_REPLACE,
XE_TEXTURE_MODE_COMBINE,
XE_TEXTURE_MULTI_MODE,
XE_TEXTURE_UNIT_COUNT,
XE_TEXTURE_WIDTH,
XE_TEXTURE_HEIGHT,
XE_SCENE_COLOR,
XE_COLOR,
XE_MATRIX_PROJECTION,
XE_MATRIX_MODELVIEW,
XE_MATRIX_TEXTURE,
XE_PHYSICAL_GRAVITY_MODE,
XE_PHYSICAL_COVALENCE_FORCE_MODE,
XE_PHYSICAL_COVALENCE_FORCE_FIRST_STREAM,
XE_PHYSICAL_COVALENCE_FORCE_LAST_STREAM,
XE_PHYSICAL_COVALENCE_FORCE_ITERATIVE_STREAM,
XE_PHYSICAL_COVALENCE_FORCE_BREAK_MODE,
XE_PHYSICAL_FORCE_MODE,
XE_PHYSICAL_GLOBAL_FORCE_MODE,
XE_PHYSICAL_BLACK_HOLE_MODE,
XE_PHYSICAL_RANGE_COLLISION_MODE,
XE_PHYSICAL_RANGE_COLLISION_TYPE,
XE_PHYSICAL_RANGE_COLLISION_STANDARD_MOVE,
XE_PHYSICAL_RANGE_COLLISION_TEMPORARY_MOVE,
XE_PHYSICAL_SURFACE_COLLISION_MODE,
XE_PHYSICAL_SURFACE_COLLISION_MODE_2D,
XE_PHYSICAL_SURFACE_COLLISION_TYPE,
XE_PHYSICAL_SURFACE_COLLISION_STANDARD_MOVE,
XE_PHYSICAL_SURFACE_COLLISION_TEMPORARY_MOVE,
XE_PHYSICAL_SURFACE_COLLISION_SAMPLE,
XE_PHYSICAL_INERTION_MODE,
XE_PHYSICAL_INERTION_FRICTION,
XE_PHYSICAL_INERTION_RANGE,
XE_PHYSICAL_INERTION_SURFACE,
XE_PHYSICAL_INERTION_FORCE,
XE_PHYSICAL_INERTION_GLOBAL_FORCE,
XE_PHYSICAL_INERTION_WAVE_FORCE,
XE_PHYSICAL_INERTION_MOVE,
XE_PHYSICAL_INERTION_FACTOR,
XE_PHYSICAL_INERTION_LIMITATION,
XE_PHYSICAL_WAVE_FORCE_MODE,
XE_SUBSTANCE_ID,
XE_SUBSTANCE_OBJECT,
XE_SUBSTANCE_X,
XE_SUBSTANCE_Y,
XE_SUBSTANCE_Z,
XE_SUBSTANCE_RANGE,
XE_SUBSTANCE_MASS,
XE_SUBSTANCE_ENERGY,
XE_SUBSTANCE_SOLID,
XE_SUBSTANCE_INERT_X,
XE_SUBSTANCE_INERT_Y,
XE_SUBSTANCE_INERT_Z,
XE_RAY_ID,
XE_RAY_X,
XE_RAY_Y,
XE_RAY_Z,
XE_RAY_ALPHA,
XE_RAY_BETA,
XE_RAY_ANGLE,
XE_RAY_ENERGY,
XE_RAY_FREQUENCY,
XE_COVALENCE_FORCE_ID,
XE_COVALENCE_FORCE_SUBSTANCE_1,
XE_COVALENCE_FORCE_SUBSTANCE_2,
XE_COVALENCE_FORCE_FORCE,
XE_COVALENCE_FORCE_RESILIENCE,
XE_COVALENCE_FORCE_RANGE,
XE_COVALENCE_FORCE_SOLID,
XE_COVALENCE_FORCE_SOLID_ALWAYS,
XE_COVALENCE_FORCE_SOLID_TEMPORARY,
XE_COVALENCE_FORCE_SOLID_NEVER,
XE_FORCE_ID,
XE_FORCE_X,
XE_FORCE_Y,
XE_FORCE_Z,
XE_FORCE_ALPHA,
XE_FORCE_BETA,
XE_FORCE_FORCE,
XE_FORCE_SPEED,
XE_FORCE_ANCHOR,
XE_SURFACE_ID,
XE_SURFACE_SUBSTANCE_1,
XE_SURFACE_SUBSTANCE_2,
XE_SURFACE_SUBSTANCE_3,
XE_SURFACE_TEX_SUBSTANCE_1,
XE_SURFACE_TEX_SUBSTANCE_2,
XE_SURFACE_TEX_SUBSTANCE_3,
XE_SURFACE_TEXTURE,
XE_SURFACE_TEXTURE_TYPE,
XE_SURFACE_ORDER,
XE_SURFACE_TEX_SUBSTANCE_1_X,
XE_SURFACE_TEX_SUBSTANCE_2_X,
XE_SURFACE_TEX_SUBSTANCE_3_X,
XE_SURFACE_TEX_SUBSTANCE_1_Y,
XE_SURFACE_TEX_SUBSTANCE_2_Y,
XE_SURFACE_TEX_SUBSTANCE_3_Y,
XE_SURFACE_TEX_SUBSTANCE_1_Z,
XE_SURFACE_TEX_SUBSTANCE_2_Z,
XE_SURFACE_TEX_SUBSTANCE_3_Z,
XE_SURFACE_DIFFUSION,
XE_SURFACE_MIRROR,
XE_SURFACE_ORIENTATION,
XE_SURFACE_ORIENTATION_SINGLE,
XE_SURFACE_ORIENTATION_DOUBLE,
XE_PARTICLE_ID,
XE_PARTICLE_SUBSTANCE,
XE_PARTICLE_WIDTH,
XE_PARTICLE_HEIGHT,
XE_PARTICLE_TEXTURE,
XE_PARTICLE_TEXTURE_TYPE,
XE_GLOBAL_FORCE_ID,
XE_GLOBAL_FORCE_X,
XE_GLOBAL_FORCE_Y,
XE_GLOBAL_FORCE_Z,
XE_GLOBAL_FORCE_ALPHA,
XE_GLOBAL_FORCE_BETA,
XE_GLOBAL_FORCE_FORCE,
XE_BLACK_HOLE_ID,
XE_BLACK_HOLE_X,
XE_BLACK_HOLE_Y,
XE_BLACK_HOLE_Z,
XE_BLACK_HOLE_FORCE,
XE_BLACK_HOLE_RANGE,
XE_BLACK_HOLE_ANCHOR,
XE_WAVE_FORCE_ID,
XE_WAVE_FORCE_X,
XE_WAVE_FORCE_Y,
XE_WAVE_FORCE_Z,
XE_WAVE_FORCE_FORCE,
XE_WAVE_FORCE_RANGE,
XE_WAVE_FORCE_LENGTH,
XE_WAVE_FORCE_SPEED,
XE_WAVE_FORCE_ORIENTATION,
XE_WAVE_FORCE_ORIENTATION_IN,
XE_WAVE_FORCE_ORIENTATION_OUT,
XE_WAVE_FORCE_ORIENTATION_INNER,
XE_WAVE_FORCE_ORIENTATION_OUTER,
XE_LOGGER_LOG,
XE_LOGGER_FILE,
XE_AREA_MODE,
XE_AREA_X1,
XE_AREA_Y1,
XE_AREA_Z1,
XE_AREA_X2,
XE_AREA_Y2,
XE_AREA_Z2,
XE_AREA_PROPERTIES,
XE_PHYSICAL_PROCEDURE,
XE_V3,
XE_C3_V3,
XE_C4_N3_V3,
XE_T4_C4_N3_V4,
XE_SHADER_VP_ARB,
XE_SHADER_VP_NV,
XE_SHADER_FP_ARB,
XE_SHADER_VP_ARB_PROFILE,
XE_SHADER_VP_NV_PROFILE,
XE_SHADER_FP_ARB_PROFILE,
XE_FRAMEBUFFER_WIDTH,
XE_FRAMEBUFFER_HEIGHT,
XE_FRAMEBUFFER_TEXTURE,
XE_REGISTER_ON_TIME_AXIS,
XE_PHYSICAL_TIME,
XE_PHYSICAL_TIME_FACTOR
};

/*--- degtorad() ---*/
/*! Zamienia stopnie na radiany.
\param deg Stopnie.
\return Radiany.
*/
inline double degtorad(double deg)
{
return(deg*PI/180);
}

inline float degtorad(float deg)
{
return(deg*(float)PI/180);
}

/*--- radtodeg() ---*/
/*! Zamienia radiany na stopnie.
\param rad Radiany.
\return Stopnie.
*/
inline double radtodeg(double rad)
{
return(rad*180/PI);
}

inline float radtodeg(float rad)
{
return(rad*180/(float)PI);
}

/*--- UintToHex() ---*/
/*! Konwertuje liczbe unsigned int do wartosci szesnastkowej i zapisuje do cstringa.
\param buffsrc Wskaznik na cstring do ktorego zwroci wartosc hexadecymalna.
\param buffsize Rozmiar bufora buffsrc.
\param value Wartosc do przekonwertowania.
\return true jesli operacja powiodla sie.
 */
bool UintToHex(char* buffsrc,int buffsize,unsigned int value)
{
if(!buffsrc||buffsize<9)return(false);
int _pos;
char _chars[8];
unsigned int _sub4;
for(_pos=0;_pos<8;_pos++)
{
_sub4=value;
_sub4<<=(8-_pos-1)*4;
_sub4>>=(8-1)*4;
if(_sub4==0)_chars[_pos]='0';
if(_sub4==1)_chars[_pos]='1';
if(_sub4==2)_chars[_pos]='2';
if(_sub4==3)_chars[_pos]='3';
if(_sub4==4)_chars[_pos]='4';
if(_sub4==5)_chars[_pos]='5';
if(_sub4==6)_chars[_pos]='6';
if(_sub4==7)_chars[_pos]='7';
if(_sub4==8)_chars[_pos]='8';
if(_sub4==9)_chars[_pos]='9';
if(_sub4==10)_chars[_pos]='A';
if(_sub4==11)_chars[_pos]='B';
if(_sub4==12)_chars[_pos]='C';
if(_sub4==13)_chars[_pos]='D';
if(_sub4==14)_chars[_pos]='E';
if(_sub4==15)_chars[_pos]='F';
}
for(_pos=0;_pos<8;_pos++)
buffsrc[8-_pos-1]=_chars[_pos];
buffsrc[8]='\0';
return(true);
}

/*--- HexToUint() ---*/
/*! Konwertuje wartosc szesnastkowa z cstringa do liczby unsigned int.
\param buffsrc Wskaznik na cstring zawierajacy wartosc hexadecymalna.
\param buffsize Rozmiar bufora buffsrc.
\return Przekonwertowana liczbe.
*/
unsigned int HexToUint(char* buffsrc,int buffsize)
{
if(!buffsrc||buffsize<8)return(0);
unsigned int value=0;
int _pos;
unsigned int _sub4;
for(_pos=0;_pos<8;_pos++)
{
char _sub=buffsrc[8-_pos-1];
if(_sub=='0')
{
_sub4=0;
_sub4<<=_pos*4;
value|=_sub4;
}
if(_sub=='1')
{
_sub4=1;
_sub4<<=_pos*4;
value|=_sub4;
}
if(_sub=='2')
{
_sub4=2;
_sub4<<=_pos*4;
value|=_sub4;
}
if(_sub=='3')
{
_sub4=3;
_sub4<<=_pos*4;
value|=_sub4;
}
if(_sub=='4')
{
_sub4=4;
_sub4<<=_pos*4;
value|=_sub4;
}
if(_sub=='5')
{
_sub4=5;
_sub4<<=_pos*4;
value|=_sub4;
}
if(_sub=='6')
{
_sub4=6;
_sub4<<=_pos*4;
value|=_sub4;
}
if(_sub=='7')
{
_sub4=7;
_sub4<<=_pos*4;
value|=_sub4;
}
if(_sub=='8')
{
_sub4=8;
_sub4<<=_pos*4;
value|=_sub4;
}
if(_sub=='9')
{
_sub4=9;
_sub4<<=_pos*4;
value|=_sub4;
}
if(_sub=='A')
{
_sub4=10;
_sub4<<=_pos*4;
value|=_sub4;
}
if(_sub=='B')
{
_sub4=11;
_sub4<<=_pos*4;
value|=_sub4;
}
if(_sub=='C')
{
_sub4=12;
_sub4<<=_pos*4;
value|=_sub4;
}
if(_sub=='D')
{
_sub4=13;
_sub4<<=_pos*4;
value|=_sub4;
}
if(_sub=='E')
{
_sub4=14;
_sub4<<=_pos*4;
value|=_sub4;
}
if(_sub=='F')
{
_sub4=15;
_sub4<<=_pos*4;
value|=_sub4;
}
}
return(value);
}

/*--- XE_SMART ---*/
template<typename T> class XE_SMART;

/*! Klasa inteligentnego wskaznika.
Obiekt tego typu przechowuje wskaznik na dane tworzone dynamicznie i zwalnia zajmowana pamiec gdy zaden inny obiekt z niego nie korzysta.
*/
template<typename T>
class XE_SMART
{
private:
bool Empty;
unsigned int* Counter;
T* Data;
public:
/*! Ustala czy pamiec obiektu jest statyczna (jesli true to podczas niszczenia wskaznika nie zostaje zwalniana pamiec obiektu) */
bool Static;
/*! Konstruktor domyslny. */
XE_SMART();
/*! Konstruktor powiazujacy.
Powiazuje ze soba dany obiekt wskazywany przez wskaznik.
\param _pointer Wskaznik na obiekt.
*/
XE_SMART(T* _pointer);
/*! Konstruktor klonujacy.
Klonuje dany obiekt i powiazuje go ze soba.
\param _clone Obiekt.
*/
XE_SMART(const T _clone);
/*! Konstruktor powiazujacy.
Powiazuje ze soba obiekt innego inteligentnego wskaznika.
\param _smart Referencja na wskaznik inteligentny.
*/
XE_SMART(const XE_SMART<T>& _smart);
/*! Konstruktor powiazujacy.
Powiazuje ze soba obiekt innego inteligentnego wskaznika, poprzez wskaznik na niego.
\param _smartp Wskaznik na wskaznik inteligentny.
*/
XE_SMART(XE_SMART<T>* _smartp);
/*! Destruktor. */
~XE_SMART();
/*! Powiazuje ze soba dany obiekt wskazywany przez wskaznik.
\param _pointer Wskaznik na obiekt.
\return true jesli operacja powiodla sie.
*/
bool RefPointer(T* _pointer);
/*! Klonuje dany obiekt i powiazuje go ze soba.
\param _clone Obiekt.
\return true jesli operacja powiodla sie.
*/
bool RefClone(const T _clone);
/*! Powiazuje ze soba obiekt innego inteligentnego wskaznika.
\param _smart Referencja na wskaznik inteligentny.
\return true jesli operacja powiodla sie.
*/
bool RefSmart(const XE_SMART<T>& _smart);
/*! Powiazuje ze soba obiekt innego inteligentnego wskaznika, poprzez wskaznik na niego.
\param _smartp Wskaznik na wskaznik inteligentny.
\return true jesli operacja powiodla sie.
*/
bool RefSmartP(XE_SMART<T>* _smartp);
/*! Odwiazuje siebie od obiektu. */
void Unref();
/*! Abstrakcyjne powiazanie (nie powiazuje z zadnym fizycznym obiektem, jednak zwieksza licznik powiazan).
\return true jesli operacja powiodla sie.
*/
bool AbstractRef();
/*! Abstrakcyjne odwiazanie (nie odwiazuje od siebie fizycznego obiektu, jednak zmniejsza licznik powiazan). */
void AbstractUnref();
/*! Zwraca wskaznik na obiekt powiazany. */
T* Pointer();
/*! Zwraca klona obiektu powiazanego. */
T Clone();
/*! Sprawdza czy inteligentny wskaznik jest pusty (czyli nie jest powiazany z zadnym obiektem).
\return true jesli wskaznik jest pusty.
*/
bool IsEmpty();
/*! Zwraca liczbe powiazan.
\return Liczba powiazan inteligentnego wskaznika.
*/
unsigned int Count();
/*! Operator powiazujacy.
Powiazuje ze soba dany obiekt wskazywany przez wskaznik.
\param _pointer Wskaznik na obiekt.
*/
void operator= (T* _pointer);
/*! Operator klonujacy.
Klonuje dany obiekt i powiazuje go ze soba.
\param _clone Obiekt.
*/
void operator= (const T _clone);
/*! Operator powiazujacy.
Powiazuje ze soba obiekt innego inteligentnego wskaznika.
\param _smart Referencja na wskaznik inteligentny.
*/
void operator= (const XE_SMART<T>& _smart);
/*! Operator powiazujacy.
Powiazuje ze soba obiekt innego inteligentnego wskaznika, poprzez wskaznik na niego.
\param _smartp Wskaznik na wskaznik inteligentny.
*/
void operator= (XE_SMART<T>* _smartp);
};

template<typename T>
XE_SMART<T>::XE_SMART()
{
Empty=true;
Static=false;
Counter=NULL;
Data=NULL;
}

template<typename T>
XE_SMART<T>::XE_SMART(T* _pointer)
{
Empty=true;
Static=false;
Counter=NULL;
Data=NULL;
RefPointer(_pointer);
}

template<typename T>
XE_SMART<T>::XE_SMART(const T _clone)
{
Empty=true;
Static=false;
Counter=NULL;
Data=NULL;
RefClone(_clone);
}

template<typename T>
XE_SMART<T>::XE_SMART(const XE_SMART<T>& _smart)
{
Empty=true;
Static=false;
Counter=NULL;
Data=NULL;
RefSmart(_smart);
}

template<typename T>
XE_SMART<T>::XE_SMART(XE_SMART<T>* _smartp)
{
Empty=true;
Static=false;
Counter=NULL;
Data=NULL;
RefSmartP(_smartp);
}

template<typename T>
XE_SMART<T>::~XE_SMART()
{
Unref();
}

template<typename T>
bool XE_SMART<T>::RefPointer(T* _pointer)
{
if(Empty&&Counter==NULL)
{
Empty=false;
#ifdef XE_ALLOC_C
Counter=(unsigned int*)malloc(sizeof(unsigned int));
#else
Counter=new unsigned int[1];
#endif /* XE_ALLOC_C */
*Counter=0;
Data=_pointer;
*Counter+=1;
return(true);
}
if(!Empty&&Counter!=NULL)
if(_pointer!=Data)
{
Unref();
Empty=false;
#ifdef XE_ALLOC_C
Counter=(unsigned int*)malloc(sizeof(unsigned int));
#else
Counter=new unsigned int[1];
#endif /* XE_ALLOC_C */
*Counter=0;
Data=_pointer;
*Counter+=1;
return(true);
}
return(false);
}

template<typename T>
bool XE_SMART<T>::RefClone(const T _clone)
{
if(Empty&&Counter==NULL)
{
Empty=false;
#ifdef XE_ALLOC_C
Counter=(unsigned int*)malloc(sizeof(unsigned int));
#else
Counter=new unsigned int[1];
#endif /* XE_ALLOC_C */
*Counter=0;
#ifdef XE_ALLOC_C
Data=(T*)malloc(sizeof(T));
#else
Data=new T[1];
#endif /* XE_ALLOC_C */
*Data=_clone;
*Counter+=1;
return(true);
}
if(!Empty&&Counter!=NULL)
{
Unref();
Empty=false;
#ifdef XE_ALLOC_C
Counter=(unsigned int*)malloc(sizeof(unsigned int));
#else
Counter=new unsigned int[1];
#endif /* XE_ALLOC_C */
*Counter=0;
#ifdef XE_ALLOC_C
Data=(T*)malloc(sizeof(T));
#else
Data=new T[1];
#endif /* XE_ALLOC_C */
*Data=_clone;
*Counter+=1;
return(true);
}
return(false);
}

template<typename T>
bool XE_SMART<T>::RefSmart(const XE_SMART<T>& _smart)
{
if(Empty&&Counter==NULL)
{
Empty=false;
Counter=_smart.Counter;
Data=_smart.Data;
*Counter+=1;
return(true);
}
if(!Empty&&Counter!=NULL)
if(_smart.Data!=Data)
{
Unref();
Empty=false;
Counter=_smart.Counter;
Data=_smart.Data;
*Counter+=1;
return(true);
}
return(false);
}

template<typename T>
bool XE_SMART<T>::RefSmartP(XE_SMART<T>* _smartp)
{
if(Empty&&Counter==NULL)
{
Empty=false;
Counter=_smartp->Counter;
Data=_smartp->Data;
*Counter+=1;
return(true);
}
if(!Empty&&Counter!=NULL)
if(_smartp->Data!=Data)
{
Unref();
Empty=false;
Counter=_smartp->Counter;
Data=_smartp->Data;
*Counter+=1;
return(true);
}
return(false);
}

template<typename T>
void XE_SMART<T>::Unref()
{
if(!Empty&&Counter!=NULL)
{
*Counter-=1;
if(*Counter==0)
{
#ifdef XE_ALLOC_C
free(Counter);
#else
delete[] Counter;
#endif /* XE_ALLOC_C */
Counter=NULL;
if(!Static)
{
#ifdef XE_ALLOC_C
free(Data);
#else
delete[] Data;
#endif /* XE_ALLOC_C */
Data=NULL;
}
}
}
}

template<typename T>
bool XE_SMART<T>::AbstractRef()
{
if(!Empty&&Counter!=NULL)
{
*Counter+=1;
return(true);
}
return(false);
}

template<typename T>
void XE_SMART<T>::AbstractUnref()
{
if(!Empty&&Counter!=NULL)
*Counter-=1;
}

template<typename T>
T* XE_SMART<T>::Pointer()
{
if(!Empty&&Counter!=NULL)
return(Data);
return(NULL);
}

template<typename T>
T XE_SMART<T>::Clone()
{
if(!Empty&&Counter!=NULL)
return(*Data);
T _temp;
return(_temp);
}

template<typename T>
bool XE_SMART<T>::IsEmpty()
{
return(Empty);
}

template<typename T>
unsigned int XE_SMART<T>::Count()
{
if(!Empty&&Counter!=NULL)
return(*Counter);
return(0);
}

template<typename T>
void XE_SMART<T>::operator= (T* _pointer)
{
RefPointer(_pointer);
}

template<typename T>
void XE_SMART<T>::operator= (const T _clone)
{
RefClone(_clone);
}

template<typename T>
void XE_SMART<T>::operator= (const XE_SMART<T>& _smart)
{
RefSmart(_smart);
}

template<typename T>
void XE_SMART<T>::operator= (XE_SMART<T>* _smartp)
{
RefSmartP(_smartp);
}

/*--- XE_ELEMENTS & XE_ELEMENT_BASE & XE_ELEMENT_CLONE & XE_ELEMENT_POINTER ---*/
template<typename T> class XE_ELEMENT_BASE;
template<typename T> class XE_ELEMENT_CLONE;
template<typename T> class XE_ELEMENT_POINTER;
template<typename T> class XE_ELEMENTS;

#define XE_FOREACH(type,container,as)	for(XE_ELEMENT_POINTER< type > as=container.FirstPointer();!as.IsEmpty();as.Next())if(!as.IsEmpty())

/*-- XE_ELEMENT_BASE --*/
template<typename T>
class XE_ELEMENT_BASE
{
friend class XE_ELEMENT_CLONE<T>;
friend class XE_ELEMENT_POINTER<T>;
friend class XE_ELEMENTS<T>;
private:
T data;
bool empty;
bool original;
XE_ELEMENT_BASE<T> *parent;
XE_ELEMENT_BASE<T> *prev;
XE_ELEMENT_BASE<T> *next;
XE_ELEMENTS<T> *container;
public:
XE_ELEMENT_BASE();
~XE_ELEMENT_BASE();
};

template<typename T>
XE_ELEMENT_BASE<T>::XE_ELEMENT_BASE()
{
empty=true;
original=true;
parent=NULL;
prev=NULL;
next=NULL;
container=NULL;
}

template<typename T>
XE_ELEMENT_BASE<T>::~XE_ELEMENT_BASE()
{
}

/*-- XE_ELEMENT_CLONE --*/
/*! Klasa klona elementu kontenera elementow.
Przechowuje klon elementu z kontenera elementow.
*/
template<typename T>
class XE_ELEMENT_CLONE
{
friend class XE_ELEMENT_BASE<T>;
friend class XE_ELEMENT_POINTER<T>;
friend class XE_ELEMENTS<T>;
private:
T data;
bool empty;
bool original;
XE_ELEMENT_BASE<T> *parent;
public:
/*! Konstruktor domyslny. */
XE_ELEMENT_CLONE();
/*! Zwraca posredni klon elementu (przechowywany w kontenerze).
\return Klon obiektu.
*/
T iClone();
/*! Zwraca bezposredni klon elementu (przechowywany w elemencie).
\return Klon obiektu.
*/
T dClone();
/*! Zwraca posredni wskaznik na element (przechowywany w kontenerze).
\return Wskaznik na obiekt.
*/
T* iPointer();
/*! Zwraca bezposredni wskaznik na element (przechowywany w elemencie).
\return Wskaznik na obiekt.
*/
T* dPointer();
/*! Inkrementuje element (o pozycje w przod).
Jesli natrafi za koncowy element kontenera, to przyjmuje wartosc pustego elementu.
*/
void Next();
/*! Dekrementuje element (o pozycje wstecz).
Jesli natrafi przed poczatkowy element kontenera, to przyjmuje wartosc pustego elementu.
*/
void Prev();
/*! Sprawdza czy element jest pusty.
\return true jesli element jest pusty.
*/
bool IsEmpty();
/*! Dowiazuje sie do obiektu danego klona elementu.
\param elm Referencja na klon elementu.
*/
void RefClone(XE_ELEMENT_CLONE<T>& elm);
/*! Dowiazuje sie do obiektu danego wskaznika elementu.
\param elm Referencja na wskaznik elementu.
*/
void RefPointer(XE_ELEMENT_POINTER<T>& elm);
/*! Odwiazuje sie od kontenera. */
void Unref();
};

/*! Operator porownania.
Sprawdza czy oba klony elementow sa rowne.
\param elm1 Pierwszy klon elementu.
\param elm2 Drugi klon elementu.
\return true jesli klony elementow sa rowne.
*/
template<typename T>
bool operator== (XE_ELEMENT_CLONE<T>& elm1,XE_ELEMENT_CLONE<T>& elm2)
{
int res=memcmp(&elm1,&elm2,sizeof(XE_ELEMENT_CLONE<T>));
if(res==0)
return(true);
return(false);
}

/*! Operator porownania.
Sprawdza czy oba klony elementow sa rozne.
\param elm1 Pierwszy klon elementu.
\param elm2 Drugi klon elementu.
\return true jesli klony elementow sa rozne.
*/
template<typename T>
bool operator!= (XE_ELEMENT_CLONE<T>& elm1,XE_ELEMENT_CLONE<T>& elm2)
{
int res=memcmp(&elm1,&elm2,sizeof(XE_ELEMENT_CLONE<T>));
if(res!=0)
return(true);
return(false);
}

template<typename T>
XE_ELEMENT_CLONE<T>::XE_ELEMENT_CLONE()
{
empty=true;
original=false;
parent=NULL;
}

template<typename T>
T XE_ELEMENT_CLONE<T>::iClone()
{
T temp;
if(!empty)
if(parent!=NULL)
temp=parent->data;
return(temp);
}

template<typename T>
T XE_ELEMENT_CLONE<T>::dClone()
{
T temp;
if(!empty)
temp=data;
return(temp);
}

template<typename T>
T* XE_ELEMENT_CLONE<T>::iPointer()
{
T* temp=NULL;
if(!empty)
if(parent!=NULL)
temp=&parent->data;
return(temp);
}

template<typename T>
T* XE_ELEMENT_CLONE<T>::dPointer()
{
T* temp=NULL;
if(!empty)
temp=&data;
return(temp);
}

template<typename T>
void XE_ELEMENT_CLONE<T>::Next()
{
if(parent!=NULL)
{
XE_ELEMENT_CLONE<T> temp;
if(parent->next!=NULL)
{
temp.data=parent->next->data;
temp.empty=parent->next->empty;
temp.parent=parent->next;
}
data=temp.data;
empty=temp.empty;
parent=temp.parent;
}
}

template<typename T>
void XE_ELEMENT_CLONE<T>::Prev()
{
if(parent!=NULL)
{
XE_ELEMENT_CLONE<T> temp;
if(parent->prev!=NULL)
{
temp.data=parent->prev->data;
temp.empty=parent->prev->empty;
temp.parent=parent->prev;
}
data=temp.data;
empty=temp.empty;
parent=temp.parent;
}
}

template<typename T>
bool XE_ELEMENT_CLONE<T>::IsEmpty()
{
return(empty);
}

template<typename T>
void XE_ELEMENT_CLONE<T>::RefClone(XE_ELEMENT_CLONE<T>& elm)
{
if(elm.parent!=NULL)
{
data=elm.parent->data;
empty=elm.parent->empty;
parent=elm.parent->parent;
}
}

template<typename T>
void XE_ELEMENT_CLONE<T>::RefPointer(XE_ELEMENT_POINTER<T>& elm)
{
if(elm.parent!=NULL)
{
data=elm.parent->data;
empty=elm.parent->empty;
parent=elm.parent->parent;
}
}

template<typename T>
void XE_ELEMENT_CLONE<T>::Unref()
{
ZeroMemory(&data,sizeof(T));
empty=true;
parent=NULL;
prev=NULL;
next=NULL;
}

/*-- XE_ELEMENT_POINTER --*/
/*! Klasa wskaznika elementu kontenera elementow.
Przechowuje wskaznik elementu kontenera elementow.
*/
template<typename T>
class XE_ELEMENT_POINTER
{
friend class XE_ELEMENT_BASE<T>;
friend class XE_ELEMENT_CLONE<T>;
friend class XE_ELEMENTS<T>;
private:
bool empty;
bool original;
XE_ELEMENT_BASE<T> *parent;
public:
/*! Konstruktor domyslny. */
XE_ELEMENT_POINTER();
/*! Zwraca posredni klon elementu (przechowywany w kontenerze).
\return Klon obiektu.
*/
T iClone();
/*! Zwraca posredni wskaznik na element (przechowywany w kontenerze).
\return Wskaznik na obiekt.
*/
T* iPointer();
/*! Inkrementuje element (o pozycje w przod).
Jesli natrafi za koncowy element kontenera, to przyjmuje wartosc pustego elementu.
*/
void Next();
/*! Dekrementuje element (o pozycje wstecz).
Jesli natrafi przed poczatkowy element kontenera, to przyjmuje wartosc pustego elementu. 
*/
void Prev();
/*! Sprawdza czy element jest pusty.
\return true jesli element jest pusty.
*/
bool IsEmpty();
/*! Dowiazuje sie do obiektu danego klona elementu.
\param elm Referencja na klon elementu.
*/
void RefClone(XE_ELEMENT_CLONE<T>& elm);
/*! Dowiazuje sie do obiektu danego wskaznika elementu.
\param elm Referencja na wskaznik elementu.
*/
void RefPointer(XE_ELEMENT_POINTER<T>& elm);
/*! Odwiazuje sie od kontenera. */
void Unref();
};

/*! Operator porownania.
Sprawdza czy oba wskazniki elementow sa rowne.
\param elm1 Pierwszy wskaznik elementu.
\param elm2 Drugi wskaznik elementu.
\return true jesli wskazniki elementow sa rowne.
*/
template<typename T>
bool operator== (XE_ELEMENT_POINTER<T>& elm1,XE_ELEMENT_POINTER<T>& elm2)
{
int res=memcmp(&elm1,&elm2,sizeof(XE_ELEMENT_POINTER<T>));
if(res==0)
return(true);
return(false);
}

/*! Operator porownania.
Sprawdza czy oba wskazniki elementow sa rozne.
\param elm1 Pierwszy wskaznik elementu.
\param elm2 Drugi wskaznik elementu.
\return true jesli wskazniki elementow sa rozne.
*/
template<typename T>
bool operator!= (XE_ELEMENT_POINTER<T>& elm1,XE_ELEMENT_POINTER<T>& elm2)
{
int res=memcmp(&elm1,&elm2,sizeof(XE_ELEMENT_POINTER<T>));
if(res!=0)
return(true);
return(false);
}

template<typename T>
XE_ELEMENT_POINTER<T>::XE_ELEMENT_POINTER()
{
empty=true;
original=false;
parent=NULL;
}

template<typename T>
T XE_ELEMENT_POINTER<T>::iClone()
{
T temp;
if(!empty)
if(parent!=NULL)
temp=parent->data;
return(temp);
}

template<typename T>
T* XE_ELEMENT_POINTER<T>::iPointer()
{
T* temp=NULL;
if(!empty)
if(parent!=NULL)
temp=&parent->data;
return(temp);
}

template<typename T>
void XE_ELEMENT_POINTER<T>::Next()
{
if(parent!=NULL)
{
XE_ELEMENT_POINTER<T> temp;
if(parent->next!=NULL)
{
temp.empty=parent->next->empty;
temp.parent=parent->next;
}
empty=temp.empty;
parent=temp.parent;
}
}

template<typename T>
void XE_ELEMENT_POINTER<T>::Prev()
{
if(parent!=NULL)
{
XE_ELEMENT_POINTER<T> temp;
if(parent->prev!=NULL)
{
temp.empty=parent->prev->empty;
temp.parent=parent->prev;
}
empty=temp.empty;
parent=temp.parent;
}
}

template<typename T>
bool XE_ELEMENT_POINTER<T>::IsEmpty()
{
return(empty);
}

template<typename T>
void XE_ELEMENT_POINTER<T>::RefClone(XE_ELEMENT_CLONE<T>& elm)
{
if(elm.parent!=NULL)
{
empty=elm.parent->empty;
parent=elm.parent->parent;
}
}

template<typename T>
void XE_ELEMENT_POINTER<T>::RefPointer(XE_ELEMENT_POINTER<T>& elm)
{
if(elm.parent!=NULL)
{
empty=elm.parent->empty;
parent=elm.parent->parent;
}
}

template<typename T>
void XE_ELEMENT_POINTER<T>::Unref()
{
empty=true;
parent=NULL;
}

/*-- XE_ELEMENTS --*/
/*! Klasa kontenera elementow.
Przechowuje obiekty, do ktorych dostep daja klony oraz wskazniki elementow.
*/
template<typename T>
class XE_ELEMENTS
{
public:
private:
XE_ELEMENT_BASE<T>* first;
XE_ELEMENT_BASE<T>* last;
unsigned int size;
public:
/*! Ustala czy pamiec kontenera jest statyczna (jesli true to podczas niszczenia kontenera nie zostaje zwalniana jego pamiec). */
bool Static;
/*! Ustala czy elementy maja byc dodawane poprzez klonowanie pamieci obiektu, a nie samego obiektu. (nie zaleca sie stosowac) */
bool AddingByCloning;
/*! Konstruktor domyslny. */
XE_ELEMENTS();
/*! Destruktor. */
~XE_ELEMENTS();
/*! Zwraca rozmiar kontenera.
\return Ilosc elementow w kontenerze.
*/
unsigned int Size();
/*! Czysci kontener. */
void Clear();
/*! Sortuje kontener funkcja foo.
\param foo Wskaznik na funkcje porownujaca 2 elementy.
*/
void Sort(bool(*foo)(T&,T&));
/*! Sortuje kontener funkcja foo uzywajac algorytmu QuickSort.
\param foo Wskaznik na funkcje porownujaca 2 elementy.
*/
void QuickSort(bool(*foo)(T&,T&));
/*! Zamienia elementy miejscami.
\param elm1 Pierwszy element.
\param elm2 Drugi element.
\return true jesli operacja powiodla sie.
*/
bool Swap(XE_ELEMENT_POINTER<T> elm1,XE_ELEMENT_POINTER<T> elm2);
/*! Przechwytuje element z innego kontenera (wskazniki na dane nie traca waznosci).
\param elm element do przechwycenia.
\return true jesli operacja powiodla sie.
*/
bool Capture(XE_ELEMENT_POINTER<T> elm);
/*! Zwraca klon pierwszego elementu kontenera.
\return Klon elementu.
*/
XE_ELEMENT_CLONE<T> FirstClone();
/*! Zwraca wskaznik pierwszego elementu kontenera.
\return Wskaznik elementu.
*/
XE_ELEMENT_POINTER<T> FirstPointer();
/*! Zwraca klon ostatniego elementu kontenera.
\return Klon elementu.
*/
XE_ELEMENT_CLONE<T> LastClone();
/*! Zwraca wskaznik ostatniego elementu kontenera.
\return Wskaznik elementu.
*/
XE_ELEMENT_POINTER<T> LastPointer();
/*! Dodaje obiekt do kontenera i zwraca jego klon elementu.
\param value Obiekt.
\return Klon elementu.
*/
XE_ELEMENT_CLONE<T> AddClone(const T& value);
/*! Dodaje obiekt do kontenera i zwraca jego wskaznik elementu.
\param value Obiekt.
\return Wskaznik elementu.
*/
XE_ELEMENT_POINTER<T> AddPointer(const T& value);
/*! Dodaje obiekt do kontenera, tuz za okreslonym elementem i zwraca jego wskaznik elementu.
\param value Obiekt.
\param elm Wskaznik elementu.
\return Wskaznik elementu.
*/
XE_ELEMENT_POINTER<T> InsertPointer(const T& value,XE_ELEMENT_POINTER<T> elm);
/*! Usuwa obiekt z kontenera wskazywany przez klon elementu.
\param elm Referencja na klon elementu.
*/
void EraseClone(XE_ELEMENT_CLONE<T>& elm);
/*! Usuwa obiekt z kontenera wskazywany przez wskaznik elementu.
\param elm Referencja na wskaznik elementu.
*/
void ErasePointer(XE_ELEMENT_POINTER<T>& elm);
};

template<typename T>
XE_ELEMENTS<T>::XE_ELEMENTS()
{
first=NULL;
last=NULL;
size=0;
Static=false;
AddingByCloning=false;
}

template<typename T>
XE_ELEMENTS<T>::~XE_ELEMENTS()
{
if(!Static)Clear();
}

template<typename T>
unsigned int XE_ELEMENTS<T>::Size()
{
return(size);
}

template<typename T>
void XE_ELEMENTS<T>::Clear()
{
if(size>0)
{
XE_ELEMENT_BASE<T>* temp=first;
XE_ELEMENT_BASE<T>* _temp;
while(temp!=NULL)
{
_temp=temp;
temp=temp->next;
#ifdef XE_ALLOC_C
free(_temp);
#else
delete[] _temp;
#endif /* XE_ALLOC_C */
}
first=NULL;
last=NULL;
size=0;
}
}

template<typename T>
void XE_ELEMENTS<T>::Sort(bool(*foo)(T&,T&))
{
if(foo)
{
bool change=true;
XE_ELEMENT_POINTER<T> iter;
XE_ELEMENT_POINTER<T> temp;
while(change)
{
change=false;
iter=FirstPointer();
temp=iter;
temp.Next();
while(!iter.IsEmpty())
{
if(!temp.IsEmpty())
if(!foo(iter.iClone(),temp.iClone()))
{
T toswap;
toswap=iter.parent->data;
iter.parent->data=temp.parent->data;
temp.parent->data=toswap;
change=true;
}
iter.Next();
temp.Next();
}
iter=LastPointer();
temp=iter;
temp.Prev();
if(change)
while(!iter.IsEmpty())
{
if(!temp.IsEmpty())
if(foo(iter.iClone(),temp.iClone()))
{
T toswap;
toswap=iter.parent->data;
iter.parent->data=temp.parent->data;
temp.parent->data=toswap;
change=true;
}
iter.Prev();
temp.Prev();
}
}
}
}

template<typename T>
bool XE_ELEMENTS<T>::Swap(XE_ELEMENT_POINTER<T> elm1,XE_ELEMENT_POINTER<T> elm2)
{
if(elm1.IsEmpty()||elm2.IsEmpty())return(false);
T toswap;
XE_ELEMENTS<T> *cont1=NULL;
XE_ELEMENTS<T> *cont2=NULL;
cont1=elm1.parent->container;
cont2=elm2.parent->container;
toswap=elm1.parent->data;
elm1.parent->data=elm2.parent->data;
elm2.parent->data=toswap;
elm1.parent->container=cont2;
elm2.parent->container=cont1;
return(true);
}

template<typename T>
bool XE_ELEMENTS<T>::Capture(XE_ELEMENT_POINTER<T> elm)
{
if(elm.IsEmpty())return(false);
if(elm.parent->container==this)return(false);
XE_ELEMENT_BASE<T>* _elm=elm.parent;
XE_ELEMENT_BASE<T>* _prev=_elm->prev;
XE_ELEMENT_BASE<T>* _next=_elm->next;
if(_prev)_prev->next=_next;
if(_next)_next->prev=_prev;
_elm->next=NULL;
_elm->prev=last;
if(last)last->next=_elm;
last=_elm;
size++;
if(_elm->container)_elm->container->size--;
_elm->container=this;
return(true);
}

template<typename T>
XE_ELEMENT_CLONE<T> XE_ELEMENTS<T>::FirstClone()
{
XE_ELEMENT_CLONE<T> temp;
if(first!=NULL)
{
temp.data=first->data;
temp.empty=first->empty;
temp.parent=first;
}
return(temp);
}

template<typename T>
XE_ELEMENT_POINTER<T> XE_ELEMENTS<T>::FirstPointer()
{
XE_ELEMENT_POINTER<T> temp;
if(first!=NULL)
{
temp.empty=first->empty;
temp.parent=first;
}
return(temp);
}

template<typename T>
XE_ELEMENT_CLONE<T> XE_ELEMENTS<T>::LastClone()
{
XE_ELEMENT_CLONE<T> temp;
if(last!=NULL)
{
temp.data=last->data;
temp.empty=last->empty;
temp.parent=last;
}
return(temp);
}

template<typename T>
XE_ELEMENT_POINTER<T> XE_ELEMENTS<T>::LastPointer()
{
XE_ELEMENT_POINTER<T> temp;
if(last!=NULL)
{
temp.empty=last->empty;
temp.parent=last;
}
return(temp);
}

template<typename T>
XE_ELEMENT_CLONE<T> XE_ELEMENTS<T>::AddClone(const T& value)
{
if(size==0)
{
size++;
XE_ELEMENT_BASE<T>* _first=first;
XE_ELEMENT_BASE<T>* _last=last;
XE_ELEMENT_BASE<T> temp;
temp.data=value;
temp.empty=false;
temp.original=true;
temp.parent=NULL;
temp.next=NULL;
temp.prev=NULL;
#ifdef XE_ALLOC_C
first=(XE_ELEMENT_BASE<T>*)malloc(sizeof(XE_ELEMENT_BASE<T>));
#else
first=new XE_ELEMENT_BASE<T>[1];
#endif /* XE_ALLOC_C */
last=first;
XE_ELEMENT_CLONE<T> _temp;
if(last==NULL)
{
first=_first;
last=_last;
return(_temp);
}
*last=temp;
_temp.data=value;
_temp.empty=false;
_temp.original=false;
_temp.parent=last;
_temp.parent->container=this;
return(_temp);
}
else
{
size++;
XE_ELEMENT_BASE<T>* _first=first;
XE_ELEMENT_BASE<T>* _last=last;
XE_ELEMENT_BASE<T> temp;
temp.data=value;
temp.empty=false;
temp.original=true;
temp.parent=NULL;
temp.next=NULL;
temp.prev=last;
#ifdef XE_ALLOC_C
last->next=(XE_ELEMENT_BASE<T>*)malloc(sizeof(XE_ELEMENT_BASE<T>));
#else
last->next=new XE_ELEMENT_BASE<T>[1];
#endif /* XE_ALLOC_C */
last=last->next;
XE_ELEMENT_CLONE<T> _temp;
if(last==NULL)
{
first=_first;
last=_last;
return(_temp);
}
*last=temp;
_temp.data=value;
_temp.empty=false;
_temp.original=false;
_temp.parent=last;
_temp.parent->container=this;
return(_temp);
}
}

template<typename T>
XE_ELEMENT_POINTER<T> XE_ELEMENTS<T>::AddPointer(const T& value)
{
if(size==0)
{
size++;
XE_ELEMENT_BASE<T>* _first=first;
XE_ELEMENT_BASE<T>* _last=last;
XE_ELEMENT_BASE<T> temp;
temp.data=(T)value;
temp.empty=false;
temp.original=true;
temp.parent=NULL;
temp.next=NULL;
temp.prev=NULL;
#ifdef XE_ALLOC_C
first=(XE_ELEMENT_BASE<T>*)malloc(sizeof(XE_ELEMENT_BASE<T>));
#else
first=new XE_ELEMENT_BASE<T>[1];
#endif /* XE_ALLOC_C */
last=first;
XE_ELEMENT_POINTER<T> _temp;
if(last==NULL)
{
first=_first;
last=_last;
return(_temp);
}
if(AddingByCloning)
memcpy(last,&temp,sizeof(XE_ELEMENT_BASE<T>));
else
*last=temp;
_temp.empty=false;
_temp.original=false;
_temp.parent=last;
_temp.parent->container=this;
return(_temp);
}
else
{
size++;
XE_ELEMENT_BASE<T>* _first=first;
XE_ELEMENT_BASE<T>* _last=last;
XE_ELEMENT_BASE<T> temp;
temp.data=(T)value;
temp.empty=false;
temp.original=true;
temp.parent=NULL;
temp.next=NULL;
temp.prev=last;
#ifdef XE_ALLOC_C
last->next=(XE_ELEMENT_BASE<T>*)malloc(sizeof(XE_ELEMENT_BASE<T>));
#else
last->next=new XE_ELEMENT_BASE<T>[1];
#endif /* XE_ALLOC_C */
last=last->next;
XE_ELEMENT_POINTER<T> _temp;
if(last==NULL)
{
first=_first;
last=_last;
return(_temp);
}
*last=temp;
_temp.empty=false;
_temp.original=false;
_temp.parent=last;
_temp.parent->container=this;
return(_temp);
}
}

template<typename T>
XE_ELEMENT_POINTER<T> XE_ELEMENTS<T>::InsertPointer(const T& value,XE_ELEMENT_POINTER<T> elm)
{
if(!elm.IsEmpty())
if(elm.parent->container!=this)
return(XE_ELEMENT_POINTER<T>());
if(size==0)
{
size++;
XE_ELEMENT_BASE<T>* _first=first;
XE_ELEMENT_BASE<T>* _last=last;
XE_ELEMENT_BASE<T> temp;
temp.data=value;
temp.empty=false;
temp.original=true;
temp.parent=NULL;
temp.next=NULL;
temp.prev=NULL;
#ifdef XE_ALLOC_C
first=(XE_ELEMENT_BASE<T>*)malloc(sizeof(XE_ELEMENT_BASE<T>));
#else
first=new XE_ELEMENT_BASE<T>[1];
#endif /* XE_ALLOC_C */
last=first;
XE_ELEMENT_POINTER<T> _temp;
if(last==NULL)
{
first=_first;
last=_last;
return(_temp);
}
if(AddingByCloning)
memcpy(last,&temp,sizeof(XE_ELEMENT_BASE<T>));
else
*last=temp;
_temp.empty=false;
_temp.original=false;
_temp.parent=last;
_temp.parent->container=this;
return(_temp);
}
else
{
size++;
XE_ELEMENT_BASE<T>* _first=first;
XE_ELEMENT_BASE<T>* _last=last;
XE_ELEMENT_BASE<T> temp;
temp.data=value;
temp.empty=false;
temp.original=true;
temp.parent=NULL;
XE_ELEMENT_BASE<T>* _base;
#ifdef XE_ALLOC_C
_base=(XE_ELEMENT_BASE<T>*)malloc(sizeof(XE_ELEMENT_BASE<T>));
#else
_base=new XE_ELEMENT_BASE<T>[1];
#endif /* XE_ALLOC_C */
XE_ELEMENT_POINTER<T> _temp;
if(_base==NULL)
{
first=_first;
last=_last;
return(_temp);
}
*_base=temp;
if(elm.parent==last)
{
temp.next=NULL;
temp.prev=last;
last->next=_base;
last=_base;
}
if(elm.IsEmpty())
{
temp.next=first;
temp.prev=NULL;
first->prev=_base;
first=_base;
}
if(elm.parent!=last&&!elm.IsEmpty())
{
temp.next=elm.parent->next;
temp.prev=elm.parent;
elm.parent->next=_base;
}
_temp.empty=false;
_temp.original=false;
_temp.parent=_base;
_temp.parent->container=this;
return(_temp);
}
}

template<typename T>
void XE_ELEMENTS<T>::EraseClone(XE_ELEMENT_CLONE<T>& elm)
{
if(size>0)
if(!elm.empty)
if(elm.parent!=NULL)
{
size--;
XE_ELEMENT_BASE<T>* temp=elm.parent;
XE_ELEMENT_BASE<T>* _prev=temp->prev;
XE_ELEMENT_BASE<T>* _next=temp->next;
if(_prev!=NULL)_prev->next=_next;
if(_prev==NULL)first=_next;
if(_next!=NULL)_next->prev=_prev;
if(_next==NULL)last=_prev;
#ifdef XE_ALLOC_C
free(temp);
#else
delete[] temp;
#endif /* XE_ALLOC_C */
}
}

template<typename T>
void XE_ELEMENTS<T>::ErasePointer(XE_ELEMENT_POINTER<T>& elm)
{
if(size>0)
if(!elm.empty)
if(elm.parent!=NULL)
{
size--;
XE_ELEMENT_BASE<T>* temp=elm.parent;
XE_ELEMENT_BASE<T>* _prev=temp->prev;
XE_ELEMENT_BASE<T>* _next=temp->next;
if(_prev!=NULL)_prev->next=_next;
if(_prev==NULL)first=_next;
if(_next!=NULL)_next->prev=_prev;
if(_next==NULL)last=_prev;
#ifdef XE_ALLOC_C
free(temp);
#else
delete[] temp;
#endif /* XE_ALLOC_C */
}
}

/*--- XE_NUMBER ---*/
/*! Klasa liczby.
Bufor wartości liczbowej.
*/
class XE_NUMBER
{
private:
double Value;
public:
/*! Konstruktor domyślny. */
XE_NUMBER();
/*! Konstruktor wypelniajacy. */
XE_NUMBER(double val);
/*! Zwraca double. */
double Get();
/*! Ustala wartość. */
void Set(double val);
/*! Operator przypisania.
Ustala nowa wartosc.
\param val Wartosc double.
*/
double& operator= (double& val);
/*! Operator przypisania.
Ustala nowa wartosc.
\param val Wartosc float.
*/
float& operator= (float& val);
/*! Operator przypisania.
Ustala nowa wartosc.
\param val Wartosc int.
*/
int& operator= (int& val);
/*! Operator przypisania.
Ustala nowa wartosc.
\param val Wartosc unsigned int.
*/
unsigned int& operator= (unsigned int& val);
/*! Operator przypisania.
Ustala nowa wartosc.
\param val Wartosc short.
*/
short& operator= (short& val);
/*! Operator przypisania.
Ustala nowa wartosc.
\param val Wartosc unsigned short.
*/
unsigned int& operator= (unsigned short& val);
/*! Operator przypisania.
Ustala nowa wartosc.
\param val Wartosc byte (char).
*/
char& operator= (char& val);
/*! Operator przypisania.
Ustala nowa wartosc.
\param val Wartosc unsigned byte (char).
*/
unsigned char& operator= (unsigned char& val);
/*! Operator przypisania.
Ustala nowa wartosc.
\param val Wartosc bool.
*/
bool& operator= (bool& val);
/*! Operator porownania.
Sprawdza czy wartosci sa rowne.
\param val Wartosc liczbowa.
\return true jesli wartosci sa rowne.
*/
bool operator== (XE_NUMBER& val);
/*! Operator porownania.
Sprawdza czy wartosci sa rowne.
\param val Wartosc double.
\return true jesli wartosci sa rowne.
*/
bool operator== (double& val);
/*! Operator porownania.
Sprawdza czy wartosci sa rowne.
\param val Wartosc float.
\return true jesli wartosci sa rowne.
*/
bool operator== (float& val);
/*! Operator porownania.
Sprawdza czy wartosci sa rowne.
\param val Wartosc int.
\return true jesli wartosci sa rowne.
*/
bool operator== (int& val);
/*! Operator porownania.
Sprawdza czy wartosci sa rowne.
\param val Wartosc unsigned int.
\return true jesli wartosci sa rowne.
*/
bool operator== (unsigned int& val);
/*! Operator porownania.
Sprawdza czy wartosci sa rowne.
\param val Wartosc short.
\return true jesli wartosci sa rowne.
*/
bool operator== (short& val);
/*! Operator porownania.
Sprawdza czy wartosci sa rowne.
\param val Wartosc unsigned short.
\return true jesli wartosci sa rowne.
*/
bool operator== (unsigned short& val);
/*! Operator porownania.
Sprawdza czy wartosci sa rowne.
\param val Wartosc byte (char).
\return true jesli wartosci sa rowne.
*/
bool operator== (char& val);
/*! Operator porownania.
Sprawdza czy wartosci sa rowne.
\param val Wartosc unsigned byte (char).
\return true jesli wartosci sa rowne.
*/
bool operator== (unsigned char& val);
/*! Operator porownania.
Sprawdza czy wartosci sa rowne.
\param val Wartosc bool.
\return true jesli wartosci sa rowne.
*/
bool operator== (bool& val);
/*! Operator porownania.
Sprawdza czy wartosci sa rozne.
\param val Wartosc liczbowa.
\return true jesli wartosci sa rowne.
*/
bool operator!= (XE_NUMBER& val);
/*! Operator porownania.
Sprawdza czy wartosci sa rozne.
\param val Wartosc double.
\return true jesli wartosci sa rowne.
*/
bool operator!= (double& val);
/*! Operator porownania.
Sprawdza czy wartosci sa rozne.
\param val Wartosc float.
\return true jesli wartosci sa rowne.
*/
bool operator!= (float& val);
/*! Operator porownania.
Sprawdza czy wartosci sa rozne.
\param val Wartosc int.
\return true jesli wartosci sa rowne.
*/
bool operator!= (int& val);
/*! Operator porownania.
Sprawdza czy wartosci sa rozne.
\param val Wartosc unsigned int.
\return true jesli wartosci sa rowne.
*/
bool operator!= (unsigned int& val);
/*! Operator porownania.
Sprawdza czy wartosci sa rozne.
\param val Wartosc short.
\return true jesli wartosci sa rowne.
*/
bool operator!= (short& val);
/*! Operator porownania.
Sprawdza czy wartosci sa rozne.
\param val Wartosc unsigned short.
\return true jesli wartosci sa rowne.
*/
bool operator!= (unsigned short& val);
/*! Operator porownania.
Sprawdza czy wartosci sa rozne.
\param val Wartosc byte (char).
\return true jesli wartosci sa rowne.
*/
bool operator!= (char& val);
/*! Operator porownania.
Sprawdza czy wartosci sa rozne.
\param val Wartosc unsigned byte (char).
\return true jesli wartosci sa rowne.
*/
bool operator!= (unsigned char& val);
/*! Operator porownania.
Sprawdza czy wartosci sa rozne.
\param val Wartosc bool.
\return true jesli wartosci sa rowne.
*/
bool operator!= (bool& val);
/*! Operator konwersji.
Rzutuje wartosc na double.
*/
operator double() const;
/*! Operator konwersji.
Rzutuje wartosc na float.
*/
operator float() const;
/*! Operator konwersji.
Rzutuje wartosc na int.
*/
operator int() const;
/*! Operator konwersji.
Rzutuje wartosc na unsigned int.
*/
operator unsigned int() const;
/*! Operator konwersji.
Rzutuje wartosc na short.
*/
operator short() const;
/*! Operator konwersji.
Rzutuje wartosc na unsigned short.
*/
operator unsigned short() const;
/*! Operator konwersji.
Rzutuje wartosc na byte (char).
*/
operator char() const;
/*! Operator konwersji.
Rzutuje wartosc na unsigned byte (char).
*/
operator unsigned char() const;
/*! Operator konwersji.
Rzutuje wartosc na bool.
*/
operator bool() const;
};

XE_NUMBER::XE_NUMBER()
{
Value=0;
}

XE_NUMBER::XE_NUMBER(double val)
{
Value=val;
}

double XE_NUMBER::Get()
{
return(Value);
}

void XE_NUMBER::Set(double val)
{
Value=val;
}

double& XE_NUMBER::operator= (double& val)
{
Set(val);
return(Value);
}

float& XE_NUMBER::operator= (float& val)
{
Set((double&)val);
return((float&)Value);
}

int& XE_NUMBER::operator= (int& val)
{
Set((double&)val);
return((int&)Value);
}

unsigned int& XE_NUMBER::operator= (unsigned int& val)
{
Set((double&)val);
return((unsigned int&)Value);
}

char& XE_NUMBER::operator= (char& val)
{
Set((double&)val);
return((char&)Value);
}

unsigned char& XE_NUMBER::operator= (unsigned char& val)
{
Set((double&)val);
return((unsigned char&)Value);
}

bool& XE_NUMBER::operator= (bool& val)
{
Set((double&)val);
return((bool&)Value);
}

bool XE_NUMBER::operator== (XE_NUMBER& val)
{
return((Value==val.Value)?true:false);
}

bool XE_NUMBER::operator== (double& val)
{
return((Value==val)?true:false);
}

bool XE_NUMBER::operator== (float& val)
{
return((Value==(double)val)?true:false);
}

bool XE_NUMBER::operator== (int& val)
{
return((Value==(double)val)?true:false);
}

bool XE_NUMBER::operator== (unsigned int& val)
{
return((Value==(double)val)?true:false);
}

bool XE_NUMBER::operator== (short& val)
{
return((Value==(double)val)?true:false);
}

bool XE_NUMBER::operator== (unsigned short& val)
{
return((Value==(double)val)?true:false);
}

bool XE_NUMBER::operator== (char& val)
{
return((Value==(double)val)?true:false);
}

bool XE_NUMBER::operator== (unsigned char& val)
{
return((Value==(double)val)?true:false);
}

bool XE_NUMBER::operator== (bool& val)
{
return((Value==(double)val)?true:false);
}

bool XE_NUMBER::operator!= (XE_NUMBER& val)
{
return((Value!=val.Value)?true:false);
}

bool XE_NUMBER::operator!= (double& val)
{
return((Value!=val)?true:false);
}

bool XE_NUMBER::operator!= (float& val)
{
return((Value!=(double)val)?true:false);
}

bool XE_NUMBER::operator!= (int& val)
{
return((Value!=(double)val)?true:false);
}

bool XE_NUMBER::operator!= (unsigned int& val)
{
return((Value!=(double)val)?true:false);
}

bool XE_NUMBER::operator!= (short& val)
{
return((Value!=(double)val)?true:false);
}

bool XE_NUMBER::operator!= (unsigned short& val)
{
return((Value!=(double)val)?true:false);
}

bool XE_NUMBER::operator!= (char& val)
{
return((Value!=(double)val)?true:false);
}

bool XE_NUMBER::operator!= (unsigned char& val)
{
return((Value!=(double)val)?true:false);
}

bool XE_NUMBER::operator!= (bool& val)
{
return((Value!=(double)val)?true:false);
}

XE_NUMBER::operator double() const
{
return(Value);
}

XE_NUMBER::operator float() const
{
return((float)Value);
}

XE_NUMBER::operator int() const
{
return((int)Value);
}

XE_NUMBER::operator unsigned int() const
{
return((unsigned int)Value);
}

XE_NUMBER::operator short() const
{
return((short)Value);
}

XE_NUMBER::operator unsigned short() const
{
return((unsigned short)Value);
}

XE_NUMBER::operator char() const
{
return((char)Value);
}

XE_NUMBER::operator unsigned char() const
{
return((unsigned char)Value);
}

XE_NUMBER::operator bool() const
{
return(Value?true:false);
}

/*--- XE_STRING ---*/
/*! Klasa stringa.
Bufor wartosci tekstowych.
*/
class XE_STRING
{
private:
char* Text;
public:
/*! Ustala czy pamiec bufora jest statyczna (jesli true to podczas niszczenia stringa nie zostaje zwalniana pamiec bufora). */
bool Static;
/*! Konstruktor domyslny. */
XE_STRING();
/*! Konstruktor wypelniajacy. */
XE_STRING(char*& str);
/*! Destruktor. */
~XE_STRING();
/*! Zwraca cstring.
\return Cstring zawartości bufora tekstu.
*/
char* Get();
/*! Ustala zawartosc bufora tekstu.
\param str Wskaznik na cstring.
*/
void Set(char* str);
/*! Zwraca dlugosc tekstu.
\return Rozmiar bufora tekstu.
*/
unsigned int Length();
/*! Czysci bufor tekstu. */
void Clear();
/*! Operator przypisania.
Ustala nowa zawartosc bufora tekstu.
\param str Wskaznik na string.
*/
void operator= (XE_STRING& str);
/*! Operator przypisania.
Ustala nowa zawartosc bufora tekstu.
\param str Wskaznik na cstring.
*/
void operator= (char* str);
/*! Operator konwersji.
Rzutuje zawartosc bufora tekstu na cstring.
*/
operator char*() const;
/*! Operator strumieniowania.
Konwertuje i dodaje wartosc double do bufora tekstu.
\param str Referencja na string (bufor tekstu).
\param value Wartosc do dodania.
\return Referencja na string str.
*/
friend XE_STRING& operator<< (XE_STRING& str,double value);
/*! Operator strumieniowania.
Konwertuje i dodaje wartosc int do bufora tekstu.
\param str Referencja na string (bufor tekstu).
\param value Wartosc do dodania.
\return Referencja na string str.
*/
friend XE_STRING& operator<< (XE_STRING& str,int value);
/*! Operator strumieniowania.
Konwertuje i dodaje wartosc unsigned int do bufora tekstu.
\param str Referencja na string (bufor tekstu).
\param value Wartosc do dodania.
\return Referencja na string str.
*/
friend XE_STRING& operator<< (XE_STRING& str,unsigned int value);
/*! Operator strumieniowania.
Dodaje cstring do bufora tekstu.
\param str Referencja na string (bufor tekstu).
\param value Wskaznik na cstring.
\return Referencja na string str.
*/
friend XE_STRING& operator<< (XE_STRING& str,char* value);
/*! Operator porownania.
Sprawdza czy oba stringi sa rowne.
\param str1 Pierwszy string.
\param str2 Drugi string.
\return true jesli oba stringi sa rowne.
*/
friend bool operator== (XE_STRING& str1,XE_STRING& str2);
/*! Operator porownania.
Sprawdza czy oba stringi sa rozne.
\param str1 Pierwszy string.
\param str2 Drugi string.
\return true jesli oba stringi sa rozne.
*/
friend bool operator!= (XE_STRING& str1,XE_STRING& str2);
/*! Operator porownania.
Sprawdza czy string i cstring sa rowne.
\param str1 Pierwszy string.
\param str2 Drugi cstring.
\return true jesli string i cstring sa rowne.
*/
friend bool operator== (XE_STRING& str1,char* str2);
/*! Operator porownania.
Sprawdza czy string i cstring sa rozne.
\param str1 Pierwszy string.
\param str2 Drugi cstring.
\return true jesli string i cstring sa rozne.
*/
friend bool operator!= (XE_STRING& str1,char* str2);
/*! Operator porownania.
Sprawdza czy cstring i string sa rowne.
\param str1 Pierwszy cstring.
\param str2 Drugi string.
\return true jesli cstring i string sa rowne.
*/
friend bool operator== (char* str1,XE_STRING& str2);
/*! Operator porownania.
Sprawdza czy cstring i string sa rozne.
\param str1 Pierwszy cstring.
\param str2 Drugi string.
\return true jesli cstring i string sa rozne.
*/
friend bool operator!= (char* str1,XE_STRING& str2);
};

/*! Znak nowej linii. */
#define XeEndL "\r\n"
/*! Znak konca tekstu. */
#define XeEndS "\0"

XE_STRING::XE_STRING()
{
#ifdef XE_ALLOC_C
Text=(char*)malloc(sizeof(char)*1);
#else
Text=new char[1];
#endif /* XE_ALLOC_C */
Text[0]=0;
Static=false;
}

XE_STRING::XE_STRING(char*& str)
{
#ifdef XE_ALLOC_C
Text=(char*)malloc(sizeof(char)*1);
#else
Text=new char[1];
#endif /* XE_ALLOC_C */
Text[0]=0;
Static=false;
Set(str);
}

XE_STRING::~XE_STRING()
{
if(!Static)
{
if(Text!=NULL)
#ifdef XE_ALLOC_C
free(Text);
#else
delete[] Text;
#endif /* XE_ALLOC_C */
Text=NULL;
}
}

char* XE_STRING::Get()
{
return(Text);
}

void XE_STRING::Set(char* str)
{
#ifdef XE_ALLOC_C
free(Text);
Text=(char*)malloc(sizeof(char)*(strlen(str)+1));
#else
delete[] Text;
Text=new char[strlen(str)+1];
#endif /* XE_ALLOC_C */
Text[0]=0;
strcpy_s(Text,strlen(str)+1,str);
Text[strlen(str)]=0;
}

unsigned int XE_STRING::Length()
{
return(strlen(Text));
}

void XE_STRING::Clear()
{
#ifdef XE_ALLOC_C
free(Text);
Text=(char*)malloc(sizeof(char)*1);
#else
delete[] Text;
Text=new char[1];
#endif /* XE_ALLOC_C */
Text[0]=0;
}

void XE_STRING::operator= (XE_STRING& str)
{
Set(str.Get());
}

void XE_STRING::operator= (char* str)
{
Set(str);
}

XE_STRING::operator char*() const
{
return(Text);
}

XE_STRING& operator<< (XE_STRING& str,double value)
{
#ifdef XE_ALLOC_C
char* buff=(char*)malloc(sizeof(char)*33);
#else
char* buff=new char[33];
#endif /* XE_ALLOC_C */
_gcvt_s(buff,32,value,8);
unsigned int _len=strlen(str.Text)+strlen(buff);
#ifdef XE_ALLOC_C
char* newtext=(char*)malloc(sizeof(char)*(_len+1));
#else
char* newtext=new char[_len+1];
#endif /* XE_ALLOC_C */
strcpy_s(newtext,(_len+1),str.Text);
strcat_s(newtext,(_len+1),buff);
#ifdef XE_ALLOC_C
free(buff);
free(str.Text);
#else
delete[] buff;
delete[] str.Text;
#endif /* XE_ALLOC_C */
str.Text=newtext;
str.Text[_len]=0;
return(str);
}

XE_STRING& operator<< (XE_STRING& str,int value)
{
#ifdef XE_ALLOC_C
char* buff=(char*)malloc(sizeof(char)*33);
#else
char* buff=new char[33];
#endif /* XE_ALLOC_C */
_itoa_s(value,buff,32,XE_BASE_DEC);
unsigned int _len=strlen(str.Text)+strlen(buff);
#ifdef XE_ALLOC_C
char* newtext=(char*)malloc(sizeof(char)*(_len+1));
#else
char* newtext=new char[_len+1];
#endif /* XE_ALLOC_C */
strcpy_s(newtext,_len+1,str.Text);
strcat_s(newtext,_len+1,buff);
#ifdef XE_ALLOC_C
free(buff);
free(str.Text);
#else
delete[] buff;
delete[] str.Text;
#endif /* XE_ALLOC_C */
str.Text=newtext;
str.Text[_len]=0;
return(str);
}

XE_STRING& operator<< (XE_STRING& str,unsigned int value)
{
#ifdef XE_ALLOC_C
char* buff=(char*)malloc(sizeof(char)*33);
#else
char* buff=new char[33];
#endif /* XE_ALLOC_C */
_ultoa_s((unsigned long)value,buff,32,XE_BASE_DEC);
unsigned int _len=strlen(str.Text)+strlen(buff);
#ifdef XE_ALLOC_C
char* newtext=(char*)malloc(sizeof(char)*(_len+1));
#else
char* newtext=new char[_len+1];
#endif /* XE_ALLOC_C */
strcpy_s(newtext,_len+1,str.Text);
strcat_s(newtext,_len+1,buff);
#ifdef XE_ALLOC_C
free(buff);
free(str.Text);
#else
delete[] buff;
delete[] str.Text;
#endif /* XE_ALLOC_C */
str.Text=newtext;
str.Text[_len]=0;
return(str);
}

XE_STRING& operator<< (XE_STRING& str,char* value)
{
unsigned int _len=strlen(str.Text)+strlen(value);
#ifdef XE_ALLOC_C
char* newtext=(char*)malloc(sizeof(char)*(_len+1));
#else
char* newtext=new char[_len+1];
#endif /* XE_ALLOC_C */
strcpy_s(newtext,_len+1,str.Text);
strcat_s(newtext,_len+1,value);
#ifdef XE_ALLOC_C
free(str.Text);
#else
delete[] str.Text;
#endif /* XE_ALLOC_C */
str.Text=newtext;
str.Text[_len]=0;
return(str);
}

bool operator== (XE_STRING& str1,XE_STRING& str2)
{
if(strcmp(str1.Get(),str2.Get())==0)
return(true);
return(false);
}

bool operator!= (XE_STRING& str1,XE_STRING& str2)
{
if(strcmp(str1.Get(),str2.Get())!=0)
return(true);
return(false);
}

bool operator== (XE_STRING& str1,char* str2)
{
if(strcmp(str1.Get(),str2)==0)
return(true);
return(false);
}

bool operator!= (XE_STRING& str1,char* str2)
{
if(strcmp(str1.Get(),str2)!=0)
return(true);
return(false);
}

bool operator== (char* str1,XE_STRING& str2)
{
if(strcmp(str1,str2.Get())==0)
return(true);
return(false);
}

bool operator!= (char* str1,XE_STRING& str2)
{
if(strcmp(str1,str2.Get())!=0)
return(true);
return(false);
}

/*--- XE_UNI ---*/
#ifndef $
#define $(value)	XE_UNI(value,true)
#endif /* $ */
#ifndef $$
#define $$(value)	XE_UNI(value)
#endif /* $$ */
#ifndef $N
#define $N(value)	XE_UNI((XE_NUMBER)value,true)
#endif /* $N */
#ifndef $$N
#define $$N(value)	XE_UNI((XE_NUMBER)value)
#endif /* $$N */
#ifndef $S
#define $S(value)	XE_UNI((XE_STRING)value,true)
#endif /* $S */
#ifndef $$S
#define $$S(value)	XE_UNI((XE_STRING)value)
#endif /* $$S */

/*! Klasa wartosci uniwersalnej (tekstowo-liczbowa). */
class XE_UNI
{
private:
void* Data;
unsigned char Type; // 0: undefined; 1: number; 2: string;
public:
/*! Ustala czy pamiec bufora jest statyczna (jesli true to podczas niszczenia unii nie zostaje zwalniana pamiec bufora). */
bool Static;
/*! Konstruktor domyslny. */
XE_UNI();
/*! Konstruktor wypelniajacy. */
XE_UNI(XE_NUMBER value);
/*! Konstruktor wypelniajacy. */
XE_UNI(XE_STRING& value);
/*! Konstruktor wypelniajacy. */
XE_UNI(double value);
/*! Konstruktor wypelniajacy. */
XE_UNI(char* value);
/*! Konstruktor wypelniajacy. */
XE_UNI(XE_NUMBER value,bool stat);
/*! Konstruktor wypelniajacy. */
XE_UNI(XE_STRING& value,bool stat);
/*! Konstruktor wypelniajacy. */
XE_UNI(double value,bool stat);
/*! Konstruktor wypelniajacy. */
XE_UNI(char* value,bool stat);
/*! Destruktor. */
~XE_UNI();
/*! Pobiera wartosc liczbowa.
\return Wartosc liczbowa.
*/
XE_NUMBER GetNumber();
/*! Pobiera wartosc tekstowa.
\return Wartosc tekstowa.
*/
XE_STRING GetString();
/*! Zwraca typ przechowywanej wartosci.
\return Typ wartosci.
*/
unsigned char GetType();
/*! Sprawdza czy ma typ niezdefiniowany.
\return true jesli typ niezdefiniowany.
*/
bool IsUndefined();
/*! Sprawdza czy ma typ liczbowy.
\return true jesli typ liczbowy.
*/
bool IsNumber();
/*! Sprawdza czy ma typ tekstowy.
\return true jesli typ tekstowy.
*/
bool IsString();
/*! Ustala wartosc liczbowa.
\param value Wartosc liczbowa.
*/
void SetNumber(XE_NUMBER value);
/*! Ustala wartosc tekstowa.
\param value Wartosc tekstowa.
*/
void SetString(XE_STRING value);
/*! Kasuje wartosc. */
void Unset();
/*! Operator przypisania.
\param value Wartosc liczbowa.
*/
void operator= (XE_NUMBER value);
/*! Operator przypisania.
\param value Wartosc tekstowa.
*/
void operator= (XE_STRING& value);
/*! Operator przypisania.
\param value Wartosc double.
*/
void operator= (double value);
/*! Operator przypisania.
\param value Wartosc cstring.
*/
void operator= (char* value);
/*! Operator rzutowania. */
operator XE_NUMBER() const;
/*! Operator rzutowania. */
operator XE_STRING() const;
/*! Operator rzutowania. */
operator double() const;
/*! Operator rzutowania. */
operator char*() const;
/*! Operator porownania.
\param value Wartosc uniwersalna.
*/
bool operator== (XE_UNI& value);
/*! Operator porownania.
\param value Wartosc uniwersalna.
*/
bool operator!= (XE_UNI& value);
};

XE_UNI::XE_UNI()
{
Data=NULL;
Type=0;
Static=false;
}

XE_UNI::XE_UNI(XE_NUMBER value)
{
Data=NULL;
Type=0;
Static=false;
SetNumber(value);
}

XE_UNI::XE_UNI(XE_STRING& value)
{
Data=NULL;
Type=0;
Static=false;
value.Static=true;
SetString(value);
}

XE_UNI::XE_UNI(double value)
{
Data=NULL;
Type=0;
Static=false;
XE_NUMBER temp;
temp=value;
SetNumber(temp);
}

XE_UNI::XE_UNI(char* value)
{
Data=NULL;
Type=0;
Static=false;
XE_STRING temp;
temp=value;
temp.Static=true;
SetString(temp);
}

XE_UNI::XE_UNI(XE_NUMBER value,bool stat)
{
Data=NULL;
Type=0;
Static=stat;
SetNumber(value);
}

XE_UNI::XE_UNI(XE_STRING& value,bool stat)
{
Data=NULL;
Type=0;
Static=stat;
value.Static=true;
SetString(value);
}

XE_UNI::XE_UNI(double value,bool stat)
{
Data=NULL;
Type=0;
Static=stat;
XE_NUMBER temp;
temp=value;
SetNumber(temp);
}

XE_UNI::XE_UNI(char* value,bool stat)
{
Data=NULL;
Type=0;
Static=stat;
XE_STRING temp;
temp=value;
temp.Static=true;
SetString(temp);
}

XE_UNI::~XE_UNI()
{
if(!Static)if(Data)delete Data;
Type=0;
}

XE_NUMBER XE_UNI::GetNumber()
{
if(Type==1)if(Data)return(*(XE_NUMBER*)Data);
return(XE_NUMBER());
}

XE_STRING XE_UNI::GetString()
{
if(Type==2)if(Data)return(*(XE_STRING*)Data);
return(XE_STRING());
}

unsigned char XE_UNI::GetType()
{
return(Type);
}

bool XE_UNI::IsUndefined()
{
	return((Type==0)?true:false);
}

bool XE_UNI::IsNumber()
{
	return((Type==1)?true:false);
}

bool XE_UNI::IsString()
{
	return((Type==2)?true:false);
}

void XE_UNI::SetNumber(XE_NUMBER value)
{
if(Type==2)if(Data)((XE_STRING*)Data)->Clear();
if(Data)delete Data;
Data=new XE_NUMBER;
((XE_NUMBER*)Data)->Set(value.Get());
Type=1;
}

void XE_UNI::SetString(XE_STRING value)
{
if(Data)delete Data;
Data=new XE_STRING;
((XE_STRING*)Data)->Set(value.Get());
Type=2;
}

void XE_UNI::Unset()
{
if(Type==2)if(Data)((XE_STRING*)Data)->Clear();
if(Data)delete Data;
Type=0;
}

void XE_UNI::operator= (XE_NUMBER value)
{
SetNumber(value);
}

void XE_UNI::operator= (XE_STRING& value)
{
value.Static=true;
SetString(value);
}

void XE_UNI::operator= (double value)
{
XE_NUMBER temp;
temp=value;
SetNumber(temp);
}

void XE_UNI::operator= (char* value)
{
XE_STRING temp;
temp=value;
temp.Static=true;
SetString(temp);
}

XE_UNI::operator XE_NUMBER() const
{
if(Type==1)
return((*(XE_NUMBER*)Data));
return(XE_NUMBER());
}

XE_UNI::operator XE_STRING() const
{
if(Type==2)
{
((XE_STRING*)Data)->Static=true;
return((*(XE_STRING*)Data));
}
return(XE_STRING());
}

XE_UNI::operator double() const
{
if(Type==1)
return((*(XE_NUMBER*)Data).Get());
return(0);
}

XE_UNI::operator char*() const
{
if(Type==2)
return((*(XE_STRING*)Data).Get());
return(NULL);
}

bool XE_UNI::operator== (XE_UNI& value)
{
if(Type==value.Type)
{
if(Type==0)return(true);
if(Type==1)if((*(XE_NUMBER*)Data)==(*(XE_NUMBER*)value.Data))return(true);
if(Type==2)if((*(XE_STRING*)Data)==(*(XE_STRING*)value.Data))return(true);
}
return(false);
}

bool XE_UNI::operator!= (XE_UNI& value)
{
if(Type==value.Type)
{
if(Type==0)return(false);
if(Type==1)if((*(XE_NUMBER*)Data)==(*(XE_NUMBER*)value.Data))return(false);
if(Type==2)if((*(XE_STRING*)Data)==(*(XE_STRING*)value.Data))return(false);
}
return(true);
}

/*--- XE_BYTE & XE_WORD & XE_DWORD & XE_QWORD ---*/
/*! Definicja typu: BYTE (8bit). */
typedef unsigned char XE_BYTE;
/*! Definicja typu: WORD (16bit). */
typedef unsigned short XE_WORD;
/*! Definicja typu: DWORD (32bit). */
typedef unsigned int XE_DWORD;

/*! Klasa typu QWORD (64bit). */
class XE_QWORD
{
public:
/*! Wartosc nizsza (32bit). */
unsigned int Low;
/*! Wartosc wyzsza (32bit). */
unsigned int High;
/*! Konstruktor domyslny. */
XE_QWORD();
/*! Konstruktor kopiujacy.
\param h Wartosc wyzsza (32bit).
\param l Wartosc nizsza (32bit).
*/
XE_QWORD(XE_DWORD h,XE_DWORD l);
/*! Operator porownania.
Sprawdza czy oba QWORD sa rowne.
\param value1 Pierwszy QWORD.
\param value2 Drugi QWORD.
\return true jesli oba QWORD sa rowne.
*/
friend bool operator== (XE_QWORD& value1,XE_QWORD& value2);
/*! Operator porownania.
Sprawdza czy oba QWORD sa rozne.
\param value1 Pierwszy QWORD.
\param value2 Drugi QWORD.
\return true jesli oba QWORD sa rozne.
*/
friend bool operator!= (XE_QWORD& value1,XE_QWORD& value2);
/*! Operator porownania.
Sprawdza czy pierwszy QWORD jest wiekszy badz rowny niz drugi.
\param value1 Pierwszy QWORD.
\param value2 Drugi QWORD.
\return true jesli pierwszy QWORD jest wiekszy badz rowny niz drugi.
*/
friend bool operator>= (XE_QWORD& value1,XE_QWORD& value2);
/*! Operator porownania.
Sprawdza czy pierwszy QWORD jest mniejszy badz rowny niz drugi.
\param value1 Pierwszy QWORD.
\param value2 Drugi QWORD.
\return true jesli pierwszy QWORD jest mniejszy badz rowny niz drugi.
*/
friend bool operator<= (XE_QWORD& value1,XE_QWORD& value2);
/*! Operator porownania.
Sprawdza czy pierwszy QWORD jest wiekszy niz drugi.
\param value1 Pierwszy QWORD.
\param value2 Drugi QWORD.
\return true jesli pierwszy QWORD jest wiekszy niz drugi.
*/
friend bool operator> (XE_QWORD& value1,XE_QWORD& value2);
/*! Operator porownania.
Sprawdza czy pierwszy QWORD jest mniejszy niz drugi.
\param value1 Pierwszy QWORD.
\param value2 Drugi QWORD.
\return true jesli pierwszy QWORD jest mniejszy niz drugi.
*/
friend bool operator< (XE_QWORD& value1,XE_QWORD& value2);
/*! Operator bitowy.
Wykonuje sume bitowa na obu QWORD.
\param value1 Pierwszy QWORD.
\param value2 Pierwszy QWORD.
\return QWORD wynikowy.
*/
friend XE_QWORD operator| (XE_QWORD value1,XE_QWORD value2);
/*! Operator bitowy.
Wykonuje iloczyn bitowy na obu QWORD.
\param value1 Pierwszy QWORD.
\param value2 Pierwszy QWORD.
\return QWORD wynikowy.
*/
friend XE_QWORD operator& (XE_QWORD value1,XE_QWORD value2);
/*! Operator bitowy.
Wykonuje roznice bitowa na obu QWORD.
\param value1 Pierwszy QWORD.
\param value2 Pierwszy QWORD.
\return QWORD wynikowy.
*/
friend XE_QWORD operator^ (XE_QWORD value1,XE_QWORD value2);
/*! Operator bitowy przypisania.
Wykonuje przypisanie sumy bitowej obu QWORD.
\param value1 Pierwszy QWORD.
\param value2 Pierwszy QWORD.
\return Referencja na QWORD wynikowy.
*/
friend XE_QWORD& operator|= (XE_QWORD& value1,XE_QWORD value2);
/*! Operator bitowy przypisania.
Wykonuje przypisanie iloczynu bitowego obu QWORD.
\param value1 Pierwszy QWORD.
\param value2 Pierwszy QWORD.
\return Referencja na QWORD wynikowy.
*/
friend XE_QWORD& operator&= (XE_QWORD& value1,XE_QWORD value2);
/*! Operator bitowy przypisania.
Wykonuje przypisanie roznicy bitowej obu QWORD.
\param value1 Pierwszy QWORD.
\param value2 Pierwszy QWORD.
\return Referencja na QWORD wynikowy.
*/
friend XE_QWORD& operator^= (XE_QWORD& value1,XE_QWORD value2);
};

XE_QWORD::XE_QWORD()
{
Low=0;
High=0;
}

XE_QWORD::XE_QWORD(XE_DWORD h,XE_DWORD l)
{
Low=l;
High=h;
}

bool operator== (XE_QWORD& value1,XE_QWORD& value2)
{
if(value1.High==value2.High&&value1.Low==value2.Low)
return(true);
return(false);
}

bool operator!= (XE_QWORD& value1,XE_QWORD& value2)
{
if(value1.High!=value2.High||value1.Low!=value2.Low)
return(true);
return(false);
}

bool operator>= (XE_QWORD& value1,XE_QWORD& value2)
{
if(value1.High>value2.High)return(true);
if(value1.High<value2.High)return(false);
if(value1.High==value2.High)
{
if(value1.Low>value2.Low)return(true);
if(value1.Low==value2.Low)return(true);
return(false);
}
return(false);
}

bool operator<= (XE_QWORD& value1,XE_QWORD& value2)
{
if(value1.High<value2.High)return(true);
if(value1.High>value2.High)return(false);
if(value1.High==value2.High)
{
if(value1.Low<value2.Low)return(true);
if(value1.Low==value2.Low)return(true);
return(false);
}
return(false);
}

bool operator> (XE_QWORD& value1,XE_QWORD& value2)
{
if(value1.High>value2.High)return(true);
if(value1.High<value2.High)return(false);
if(value1.High==value2.High)
{
if(value1.Low>value2.Low)return(true);
return(false);
}
return(false);
}

bool operator< (XE_QWORD& value1,XE_QWORD& value2)
{
if(value1.High<value2.High)return(true);
if(value1.High>value2.High)return(false);
if(value1.High==value2.High)
{
if(value1.Low<value2.Low)return(true);
return(false);
}
return(false);
}

XE_QWORD operator| (XE_QWORD value1,XE_QWORD value2)
{
XE_QWORD _res;
_res.High=value1.High|value2.High;
_res.Low=value1.Low|value2.Low;
return(_res);
}

XE_QWORD operator& (XE_QWORD value1,XE_QWORD value2)
{
XE_QWORD _res;
_res.High=value1.High&value2.High;
_res.Low=value1.Low&value2.Low;
return(_res);
}

XE_QWORD operator^ (XE_QWORD value1,XE_QWORD value2)
{
XE_QWORD _res;
_res.High=value1.High^value2.High;
_res.Low=value1.Low^value2.Low;
return(_res);
}

XE_QWORD& operator|= (XE_QWORD& value1,XE_QWORD value2)
{
value1.High|=value2.High;
value1.Low|=value2.Low;
return(value1);
}

XE_QWORD& operator&= (XE_QWORD& value1,XE_QWORD value2)
{
value1.High&=value2.High;
value1.Low&=value2.Low;
return(value1);
}

XE_QWORD& operator^= (XE_QWORD& value1,XE_QWORD value2)
{
value1.High^=value2.High;
value1.Low^=value2.Low;
return(value1);
}

/*--- XE_VECTOR ---*/
//! Klasa wektora
class XE_VECTOR
{
public:
//! Wspolrzedna X
double X;
//! Wspolrzedna Y
double Y;
//! Wspolrzedna Z
double Z;
//! Wspolczedna W
double W;
//! Konstruktor zerujacy wartosci
XE_VECTOR();
//! Konstruktor przypisujacy poczatkowe wartosci
XE_VECTOR(double _x,double _y,double _z,double _w);
//! Przypisanie wartosci
void Set(double _x,double _y,double _z,double _w);
//! Operator dodawania
XE_VECTOR operator+ (XE_VECTOR& v);
//! Operator odejmowania
XE_VECTOR operator- (XE_VECTOR& v);
//! Operator mnozenia
XE_VECTOR operator* (XE_VECTOR& v);
//! Operator dzielenia
XE_VECTOR operator/ (XE_VECTOR& v);
};

XE_VECTOR::XE_VECTOR()
{
X=0;
Y=0;
Z=0;
W=1;
}

XE_VECTOR::XE_VECTOR(double _x,double _y,double _z,double _w)
{
X=_x;
Y=_y;
Z=_z;
W=_w;
}

void XE_VECTOR::Set(double _x,double _y,double _z,double _w)
{
X=_x;
Y=_y;
Z=_z;
W=_w;
}

XE_VECTOR XE_VECTOR::operator+ (XE_VECTOR& v)
{
XE_VECTOR vr;
vr.X=X+v.X;
vr.Y=Y+v.Y;
vr.Z=Z+v.Z;
vr.W=W+v.W;
return(vr);
}

XE_VECTOR XE_VECTOR::operator- (XE_VECTOR& v)
{
XE_VECTOR vr;
vr.X=X-v.X;
vr.Y=Y-v.Y;
vr.Z=Z-v.Z;
vr.W=W-v.W;
return(vr);
}

XE_VECTOR XE_VECTOR::operator* (XE_VECTOR& v)
{
XE_VECTOR vr;
vr.X=X*v.X;
vr.Y=Y*v.Y;
vr.Z=Z*v.Z;
vr.W=W*v.W;
return(vr);
}

XE_VECTOR XE_VECTOR::operator/ (XE_VECTOR& v)
{
XE_VECTOR vr;
vr.X=X/v.X;
vr.Y=Y/v.Y;
vr.Z=Z/v.Z;
vr.W=W/v.W;
return(vr);
}

/*--- XE_ANGLEVECTOR ---*/
//! Klasa wektora sferycznego
class XE_ANGLEVECTOR
{
public:
//! Kat Alpha (os Z)
double Alpha;
//! Kat Beta (os Y)
double Beta;
//! Kat Gamma (os X)
double Gamma;
//! Dlugosc
double Length;
//! Konstruktor zerujacy wartosci
XE_ANGLEVECTOR();
//! Konstruktor przypisujacy poczatkowe wartosci
XE_ANGLEVECTOR(double _alpha,double _beta,double _gamma,double _length);
//! Przypisanie wartosci
void Set(double _alpha,double _beta,double _gamma,double _length);
//! Operator dodawania
XE_ANGLEVECTOR operator+ (XE_ANGLEVECTOR& v);
//! Operator odejmowania
XE_ANGLEVECTOR operator- (XE_ANGLEVECTOR& v);
//! Operator mnozenia
XE_ANGLEVECTOR operator* (XE_ANGLEVECTOR& v);
//! Operator dzielenia
XE_ANGLEVECTOR operator/ (XE_ANGLEVECTOR& v);
};

XE_ANGLEVECTOR::XE_ANGLEVECTOR()
{
Alpha=0;
Beta=0;
Gamma=0;
Length=0;
}

XE_ANGLEVECTOR::XE_ANGLEVECTOR(double _alpha,double _beta,double _gamma,double _length)
{
Alpha=_alpha;
Beta=_beta;
Gamma=_gamma;
Length=_length;
}

void XE_ANGLEVECTOR::Set(double _alpha,double _beta,double _gamma,double _length)
{
Alpha=_alpha;
Beta=_beta;
Gamma=_gamma;
Length=_length;
}

XE_ANGLEVECTOR XE_ANGLEVECTOR::operator+ (XE_ANGLEVECTOR& v)
{
XE_ANGLEVECTOR vr;
vr.Alpha=Alpha+v.Alpha;
vr.Beta=Beta+v.Beta;
vr.Gamma=Gamma+v.Gamma;
vr.Length=Length+v.Length;
return(vr);
}

XE_ANGLEVECTOR XE_ANGLEVECTOR::operator- (XE_ANGLEVECTOR& v)
{
XE_ANGLEVECTOR vr;
vr.Alpha=Alpha-v.Alpha;
vr.Beta=Beta-v.Beta;
vr.Gamma=Gamma-v.Gamma;
vr.Length=Length-v.Length;
return(vr);
}

XE_ANGLEVECTOR XE_ANGLEVECTOR::operator* (XE_ANGLEVECTOR& v)
{
XE_ANGLEVECTOR vr;
vr.Alpha=Alpha*v.Alpha;
vr.Beta=Beta*v.Beta;
vr.Gamma=Gamma*v.Gamma;
vr.Length=Length*v.Length;
return(vr);
}

XE_ANGLEVECTOR XE_ANGLEVECTOR::operator/ (XE_ANGLEVECTOR& v)
{
XE_ANGLEVECTOR vr;
vr.Alpha=Alpha/v.Alpha;
vr.Beta=Beta/v.Beta;
vr.Gamma=Gamma/v.Gamma;
vr.Length=Length/v.Length;
return(vr);
}

/*--- XE_MATRIX ---*/
//! Klasa macierzy 4x4.
class XE_MATRIX
{
public:
//! Komorki macierzy.
double Cell[16];
//! Konstruktor domyslny.
XE_MATRIX();
/*! Ustala maciez jednostkowa. */
void Identity();
/*! Dodanie macierzy.
\param mat Macierz.
\return Macierz wynikowa.
*/
XE_MATRIX Add(XE_MATRIX mat);
/*! Odejmmowanie macierzy.
\param mat Macierz.
\return Macierz wynikowa.
*/
XE_MATRIX Substract(XE_MATRIX mat);
/*! Mnozenie macierzy przez skalar.
\param value Wartosc.
\return Macierz wynikowa.
*/
XE_MATRIX ScalarMultiply(double value);
/*! Transpozycja macierzy.
\return Macierz wynikowa.
*/
XE_MATRIX Transpose();
/*! Mnozenie macierzy przez macierz.
\param mat Macierz.
\return Macierz wynikowa.
*/
XE_MATRIX Multiply(XE_MATRIX mat);
/*! Mnozenie macierzy przez vector.
\param vec Vector.
\return Vector wynikowy.
*/
XE_VECTOR Multiply(XE_VECTOR vec);
/*! Oblicza wyznacznik macierzy.
\return Wyznacznik.
*/
double Determinant();
/*! Oblicza odwrotnosc macierzy.
\return Macierz odwrotna.
*/
XE_MATRIX Inverse();
/*! Przesuniecie.
\param x Wartosc X.
\param y Wartosc Y.
\param z Wartosc Z.
\return Macierz wynikowa.
*/
XE_MATRIX Translate(double x,double y,double z);
/*! Skalowanie.
\param x Wartosc X.
\param y Wartosc Y.
\param z Wartosc Z.
\return Macierz wynikowa.
*/
XE_MATRIX Scale(double x,double y,double z);
/*! Obrot w osi X.
\param angle Wartosc obrotu.
\return Macierz wynikowa.
*/
XE_MATRIX RotateX(double angle);
/*! Obrot w osi Y.
\param angle Wartosc obrotu.
\return Macierz wynikowa.
*/
XE_MATRIX RotateY(double angle);
/*! Obrot w osi Z.
\param angle Wartosc obrotu.
\return Macierz wynikowa.
*/
XE_MATRIX RotateZ(double angle);
/*! Operator dodawania. */
XE_MATRIX operator+ (XE_MATRIX &mat);
/*! Operator odejmowania. */
XE_MATRIX operator- (XE_MATRIX &mat);
/*! Operator mnozenia przez skalar. */
XE_MATRIX operator* (double &value);
/*! Operator mnozenia. */
XE_MATRIX operator* (XE_MATRIX &mat);
};

XE_MATRIX::XE_MATRIX()
{
Identity();
}

void XE_MATRIX::Identity()
{
Cell[0]=1;	Cell[4]=0;	Cell[8]=0;	Cell[12]=0;
Cell[1]=0;	Cell[5]=1;	Cell[9]=0;	Cell[13]=0;
Cell[2]=0;	Cell[6]=0;	Cell[10]=1;	Cell[14]=0;
Cell[3]=0;	Cell[7]=0;	Cell[11]=0;	Cell[15]=1;
}

XE_MATRIX XE_MATRIX::Add(XE_MATRIX mat)
{
XE_MATRIX temp;
for(int i=0;i<16;i++)
temp.Cell[i]=Cell[i]+mat.Cell[i];
return(temp);
}

XE_MATRIX XE_MATRIX::Substract(XE_MATRIX mat)
{
XE_MATRIX temp;
for(int i=0;i<16;i++)
temp.Cell[i]=Cell[i]-mat.Cell[i];
return(temp);
}

XE_MATRIX XE_MATRIX::ScalarMultiply(double value)
{
XE_MATRIX temp;
for(int i=0;i<16;i++)
temp.Cell[i]=Cell[i]*value;
return(temp);
}

XE_MATRIX XE_MATRIX::Transpose()
{
XE_MATRIX temp;
temp.Cell[0]=Cell[0];	temp.Cell[4]=Cell[1];	temp.Cell[8]=Cell[2];	temp.Cell[12]=Cell[3];
temp.Cell[1]=Cell[4];	temp.Cell[5]=Cell[5];	temp.Cell[9]=Cell[6];	temp.Cell[13]=Cell[7];
temp.Cell[2]=Cell[8];	temp.Cell[6]=Cell[9];	temp.Cell[10]=Cell[10];	temp.Cell[14]=Cell[11];
temp.Cell[3]=Cell[12];	temp.Cell[7]=Cell[13];	temp.Cell[11]=Cell[14];	temp.Cell[15]=Cell[15];
return(temp);
}

XE_MATRIX XE_MATRIX::Multiply(XE_MATRIX mat)
{
XE_MATRIX temp;
temp.Cell[0]=Cell[0]*mat.Cell[0]+Cell[4]*mat.Cell[1]+Cell[8]*mat.Cell[2]+Cell[12]*mat.Cell[3];
temp.Cell[1]=Cell[1]*mat.Cell[0]+Cell[5]*mat.Cell[1]+Cell[9]*mat.Cell[2]+Cell[13]*mat.Cell[3];
temp.Cell[2]=Cell[2]*mat.Cell[0]+Cell[6]*mat.Cell[1]+Cell[10]*mat.Cell[2]+Cell[14]*mat.Cell[3];
temp.Cell[3]=Cell[3]*mat.Cell[0]+Cell[7]*mat.Cell[1]+Cell[11]*mat.Cell[2]+Cell[15]*mat.Cell[3];
temp.Cell[4]=Cell[0]*mat.Cell[4]+Cell[4]*mat.Cell[5]+Cell[8]*mat.Cell[6]+Cell[12]*mat.Cell[7];
temp.Cell[5]=Cell[1]*mat.Cell[4]+Cell[5]*mat.Cell[5]+Cell[9]*mat.Cell[6]+Cell[13]*mat.Cell[7];
temp.Cell[6]=Cell[2]*mat.Cell[4]+Cell[6]*mat.Cell[5]+Cell[10]*mat.Cell[6]+Cell[14]*mat.Cell[7];
temp.Cell[7]=Cell[3]*mat.Cell[4]+Cell[7]*mat.Cell[5]+Cell[11]*mat.Cell[6]+Cell[15]*mat.Cell[7];
temp.Cell[8]=Cell[0]*mat.Cell[8]+Cell[4]*mat.Cell[9]+Cell[8]*mat.Cell[10]+Cell[12]*mat.Cell[11];
temp.Cell[9]=Cell[1]*mat.Cell[8]+Cell[5]*mat.Cell[9]+Cell[9]*mat.Cell[10]+Cell[13]*mat.Cell[11];
temp.Cell[10]=Cell[2]*mat.Cell[8]+Cell[6]*mat.Cell[9]+Cell[10]*mat.Cell[10]+Cell[14]*mat.Cell[11];
temp.Cell[11]=Cell[3]*mat.Cell[8]+Cell[7]*mat.Cell[9]+Cell[11]*mat.Cell[10]+Cell[15]*mat.Cell[11];
temp.Cell[12]=Cell[0]*mat.Cell[12]+Cell[4]*mat.Cell[13]+Cell[8]*mat.Cell[14]+Cell[12]*mat.Cell[15];
temp.Cell[13]=Cell[1]*mat.Cell[12]+Cell[5]*mat.Cell[13]+Cell[9]*mat.Cell[14]+Cell[13]*mat.Cell[15];
temp.Cell[14]=Cell[2]*mat.Cell[12]+Cell[6]*mat.Cell[13]+Cell[10]*mat.Cell[14]+Cell[14]*mat.Cell[15];
temp.Cell[15]=Cell[3]*mat.Cell[12]+Cell[7]*mat.Cell[13]+Cell[11]*mat.Cell[14]+Cell[15]*mat.Cell[15];
return(temp);
}

XE_VECTOR XE_MATRIX::Multiply(XE_VECTOR vec)
{
XE_VECTOR temp;
temp.X=Cell[0]*vec.X+Cell[4]*vec.Y+Cell[8]*vec.Z+Cell[12]*vec.W;
temp.Y=Cell[1]*vec.X+Cell[5]*vec.Y+Cell[9]*vec.Z+Cell[13]*vec.W;
temp.Z=Cell[2]*vec.X+Cell[6]*vec.Y+Cell[10]*vec.Z+Cell[14]*vec.W;
temp.W=Cell[3]*vec.X+Cell[7]*vec.Y+Cell[11]*vec.Z+Cell[15]*vec.W;
temp.X/=temp.W;
temp.Y/=temp.W;
temp.Z/=temp.W;
return(temp);
}

double XE_MATRIX::Determinant()
{
double temp
=Cell[0]*Cell[5]*Cell[10]*Cell[15]+Cell[0]*Cell[9]*Cell[14]*Cell[7]+Cell[0]*Cell[13]*Cell[6]*Cell[11]
+Cell[4]*Cell[1]*Cell[14]*Cell[11]+Cell[4]*Cell[9]*Cell[2]*Cell[15]+Cell[4]*Cell[13]*Cell[10]*Cell[3]
+Cell[8]*Cell[1]*Cell[6]*Cell[15]+Cell[8]*Cell[5]*Cell[14]*Cell[3]+Cell[8]*Cell[13]*Cell[2]*Cell[7]
+Cell[12]*Cell[1]*Cell[10]*Cell[7]+Cell[12]*Cell[5]*Cell[2]*Cell[11]+Cell[12]*Cell[9]*Cell[6]*Cell[3]
-Cell[0]*Cell[5]*Cell[14]*Cell[11]-Cell[0]*Cell[9]*Cell[6]*Cell[15]-Cell[0]*Cell[13]*Cell[10]*Cell[7]
-Cell[4]*Cell[1]*Cell[10]*Cell[15]-Cell[4]*Cell[9]*Cell[14]*Cell[3]-Cell[4]*Cell[13]*Cell[2]*Cell[11]
-Cell[8]*Cell[1]*Cell[14]*Cell[7]-Cell[8]*Cell[5]*Cell[2]*Cell[15]-Cell[8]*Cell[13]*Cell[6]*Cell[3]
-Cell[12]*Cell[1]*Cell[6]*Cell[11]-Cell[12]*Cell[5]*Cell[10]*Cell[3]-Cell[12]*Cell[9]*Cell[2]*Cell[7];
return(temp);
}

XE_MATRIX XE_MATRIX::Inverse()
{
double det=Determinant();
XE_MATRIX _temp;
// calculate _temp
XE_MATRIX temp=_temp.ScalarMultiply(1/det);
return(temp);
}

XE_MATRIX XE_MATRIX::Translate(double x,double y,double z)
{
XE_MATRIX temp;
XE_MATRIX mat;
mat.Cell[12]=x;
mat.Cell[13]=y;
mat.Cell[14]=z;
temp=Multiply(mat);
return(temp);
}

XE_MATRIX XE_MATRIX::Scale(double x,double y,double z)
{
XE_MATRIX temp;
XE_MATRIX mat;
mat.Cell[0]=x;
mat.Cell[5]=y;
mat.Cell[10]=z;
temp=Multiply(mat);
return(temp);
}

XE_MATRIX XE_MATRIX::RotateX(double angle)
{
XE_MATRIX temp;
XE_MATRIX mat;
mat.Cell[5]=cos(degtorad(angle));
mat.Cell[6]=sin(degtorad(angle));
mat.Cell[9]=-sin(degtorad(angle));
mat.Cell[10]=cos(degtorad(angle));
temp=Multiply(mat);
return(temp);
}

XE_MATRIX XE_MATRIX::RotateY(double angle)
{
XE_MATRIX temp;
XE_MATRIX mat;
mat.Cell[0]=cos(degtorad(angle));
mat.Cell[2]=-sin(degtorad(angle));
mat.Cell[8]=sin(degtorad(angle));
mat.Cell[10]=cos(degtorad(angle));
temp=Multiply(mat);
return(temp);
}

XE_MATRIX XE_MATRIX::RotateZ(double angle)
{
XE_MATRIX temp;
XE_MATRIX mat;
mat.Cell[0]=cos(degtorad(angle));
mat.Cell[1]=sin(degtorad(angle));
mat.Cell[4]=-sin(degtorad(angle));
mat.Cell[5]=cos(degtorad(angle));
temp=Multiply(mat);
return(temp);
}

XE_MATRIX XE_MATRIX::operator+ (XE_MATRIX &mat)
{
return(Add(mat));
}

XE_MATRIX XE_MATRIX::operator- (XE_MATRIX &mat)
{
return(Substract(mat));
}

XE_MATRIX XE_MATRIX::operator* (double &value)
{
return(ScalarMultiply(value));
}

XE_MATRIX XE_MATRIX::operator* (XE_MATRIX &mat)
{
return(Multiply(mat));
}

/*--- XE_PAIR ---*/
//! Klasa pary klucz-wartośść.
template <typename TK,typename TV>
class XE_PAIR
{
public:
TK Key;
TV Value;
};

/*--- XE_ASSOC ---*/
//! Klasa tablicy asocjacyjnej.
template <typename TK,typename TV>
class XE_ASSOC
{
private:
XE_ELEMENTS< XE_PAIR<TK,TV> > Array;
public:
/*! Czysci zawartosc tablicy. */
void Clear();
/*! Kasuje element tablicy.
\param key Klucz.
\return true jesli operacja powiodla sie.
*/
bool Erase(TK key);
/*! Zmienia klucz danej pary.
\param oldkey Stary klucz.
\param newkey Nowy klucz.
\return true jesli operacja powiodla sie.
*/
bool Change(TK oldkey,TK newkey);
/*! Zwraca ilosc elementow w tablicy.
\return Ilosc elementow.
*/
unsigned int Size();
/*! Operator wyłuskania wartości po kluczu.
\param key Klucz.
*/
TV& operator[] (TK key);
};

template <typename TK,typename TV>
void XE_ASSOC<TK,TV>::Clear()
{
Array.Clear();
}

template <typename TK,typename TV>
bool XE_ASSOC<TK,TV>::Erase(TK key)
{
XE_ELEMENT_POINTER< XE_PAIR<TK,TV> > elm;
for(elm=Array.FirstPointer();!elm.IsEmpty();elm.Next())
if(elm.iPointer()->Key==key)
{
Array.ErasePointer(elm);
return(true);
}
return(false);
}

template <typename TK,typename TV>
bool XE_ASSOC<TK,TV>::Change(TK oldkey,TK newkey)
{
XE_ELEMENT_POINTER< XE_PAIR<TK,TV> > elm;
for(elm=Array.FirstPointer();!elm.IsEmpty();elm.Next())
if(elm.iPointer()->Key==oldkey)
{
elm.iPointer()->Key=newkey;
return(true);
}
return(false);
}

template <typename TK,typename TV>
unsigned int XE_ASSOC<TK,TV>::Size()
{
return(Array.Size());
}

template <typename TK,typename TV>
TV& XE_ASSOC<TK,TV>::operator[] (TK key)
{
XE_ELEMENT_POINTER< XE_PAIR<TK,TV> > elm;
for(elm=Array.FirstPointer();!elm.IsEmpty();elm.Next())
if(elm.iPointer()->Key==key)
return(elm.iPointer()->Value);
XE_PAIR<TK,TV> pair;
TV temp;
pair.Key=key;
pair.Value=temp;
elm=Array.AddPointer(pair);
return(elm.iPointer()->Value);
}

// XenoN Core Uniform Element
typedef XE_ELEMENT_POINTER<void> XE_ELM_UNIFORM;
#define UniformConversion(IN,OUT) OUT& operator^ (OUT& out,IN& in){return(out=*(OUT*)&in);}
#define UniformConversionRev(OUT,IN) OUT& operator^ (OUT& out,IN& in){return(out=*(OUT*)&in);}
UniformConversion(XE_ELM_UNIFORM,double)
UniformConversionRev(XE_ELM_UNIFORM,double)
UniformConversion(XE_ELM_UNIFORM,XE_UNI)
UniformConversionRev(XE_ELM_UNIFORM,XE_UNI)

#ifndef XE_NOT_USE_LIBRARY

/*--- XenoN Core Elements declaration ---*/
/*! Definicja elementu: Psyche. */
class XENON_CORE_PSYCHE;
typedef XE_ELEMENT_POINTER<XENON_CORE_PSYCHE> XE_ELM_PSYCHE;
/*! Definicja elementu: RenderTarget. */
class XENON_CORE_RENDERTARGET;
typedef XE_ELEMENT_POINTER<XENON_CORE_RENDERTARGET> XE_ELM_RENDERTARGET;
/*! Definicja elementu: FrameBuffer. */
class XENON_CORE_FRAMEBUFFER;
typedef XE_ELEMENT_POINTER<XENON_CORE_FRAMEBUFFER> XE_ELM_FRAMEBUFFER;
/*! Definicja elementu: Shader. */
class XENON_CORE_SHADER;
typedef XE_ELEMENT_POINTER<XENON_CORE_SHADER> XE_ELM_SHADER;
/*! Definicja elementu: RenderQueue. */
class XENON_CORE_RENDERQUEUE;
typedef XE_ELEMENT_POINTER<XENON_CORE_RENDERQUEUE> XE_ELM_RENDERQUEUE;
/*! Definicja elementu: Texture. */
class XENON_CORE_TEXTURE;
typedef XE_ELEMENT_POINTER<XENON_CORE_TEXTURE> XE_ELM_TEXTURE;
/*! Definicja elementu: Displaylist. */
class XENON_CORE_DISPLAYLIST;
typedef XE_ELEMENT_POINTER<XENON_CORE_DISPLAYLIST> XE_ELM_DISPLAYLIST;
/*! Definicja elementu: Substance. */
class XENON_CORE_SUBSTANCE;
typedef XE_ELEMENT_POINTER<XENON_CORE_SUBSTANCE> XE_ELM_SUBSTANCE;
/*! Definicja elementu: Ray. */
class XENON_CORE_RAY;
typedef XE_ELEMENT_POINTER<XENON_CORE_RAY> XE_ELM_RAY;
/*! Definicja elementu: Covalence Force. */
class XENON_CORE_COVALENCE_FORCE;
typedef XE_ELEMENT_POINTER<XENON_CORE_COVALENCE_FORCE> XE_ELM_COVALENCE_FORCE;
/*! Definicja elementu: Force. */
class XENON_CORE_FORCE;
typedef XE_ELEMENT_POINTER<XENON_CORE_FORCE> XE_ELM_FORCE;
/*! Definicja elementu: Surface. */
class XENON_CORE_SURFACE;
typedef XE_ELEMENT_POINTER<XENON_CORE_SURFACE> XE_ELM_SURFACE;
/*! Definicja elementu: Particle. */
class XENON_CORE_PARTICLE;
typedef XE_ELEMENT_POINTER<XENON_CORE_PARTICLE> XE_ELM_PARTICLE;
/*! Definicja elementu: Global Force. */
class XENON_CORE_GLOBAL_FORCE;
typedef XE_ELEMENT_POINTER<XENON_CORE_GLOBAL_FORCE> XE_ELM_GLOBAL_FORCE;
/*! Definicja elementu: Black Hole. */
class XENON_CORE_BLACK_HOLE;
typedef XE_ELEMENT_POINTER<XENON_CORE_BLACK_HOLE> XE_ELM_BLACK_HOLE;
/*! Definicja elementu: Wave Force. */
class XENON_CORE_WAVE_FORCE;
typedef XE_ELEMENT_POINTER<XENON_CORE_WAVE_FORCE> XE_ELM_WAVE_FORCE;

// Uniform translation operators
UniformConversion(XE_ELM_UNIFORM,XE_ELM_BLACK_HOLE)
UniformConversion(XE_ELM_UNIFORM,XE_ELM_COVALENCE_FORCE)
UniformConversion(XE_ELM_UNIFORM,XE_ELM_DISPLAYLIST)
UniformConversion(XE_ELM_UNIFORM,XE_ELM_FORCE)
UniformConversion(XE_ELM_UNIFORM,XE_ELM_FRAMEBUFFER)
UniformConversion(XE_ELM_UNIFORM,XE_ELM_GLOBAL_FORCE)
UniformConversion(XE_ELM_UNIFORM,XE_ELM_PARTICLE)
UniformConversion(XE_ELM_UNIFORM,XE_ELM_PSYCHE)
UniformConversion(XE_ELM_UNIFORM,XE_ELM_RAY)
UniformConversion(XE_ELM_UNIFORM,XE_ELM_RENDERQUEUE)
UniformConversion(XE_ELM_UNIFORM,XE_ELM_RENDERTARGET)
UniformConversion(XE_ELM_UNIFORM,XE_ELM_SHADER)
UniformConversion(XE_ELM_UNIFORM,XE_ELM_SUBSTANCE)
UniformConversion(XE_ELM_UNIFORM,XE_ELM_SURFACE)
UniformConversion(XE_ELM_UNIFORM,XE_ELM_TEXTURE)
UniformConversion(XE_ELM_UNIFORM,XE_ELM_WAVE_FORCE)
UniformConversionRev(XE_ELM_UNIFORM,XE_ELM_BLACK_HOLE)
UniformConversionRev(XE_ELM_UNIFORM,XE_ELM_COVALENCE_FORCE)
UniformConversionRev(XE_ELM_UNIFORM,XE_ELM_DISPLAYLIST)
UniformConversionRev(XE_ELM_UNIFORM,XE_ELM_FORCE)
UniformConversionRev(XE_ELM_UNIFORM,XE_ELM_FRAMEBUFFER)
UniformConversionRev(XE_ELM_UNIFORM,XE_ELM_GLOBAL_FORCE)
UniformConversionRev(XE_ELM_UNIFORM,XE_ELM_PARTICLE)
UniformConversionRev(XE_ELM_UNIFORM,XE_ELM_PSYCHE)
UniformConversionRev(XE_ELM_UNIFORM,XE_ELM_RAY)
UniformConversionRev(XE_ELM_UNIFORM,XE_ELM_RENDERQUEUE)
UniformConversionRev(XE_ELM_UNIFORM,XE_ELM_RENDERTARGET)
UniformConversionRev(XE_ELM_UNIFORM,XE_ELM_SHADER)
UniformConversionRev(XE_ELM_UNIFORM,XE_ELM_SUBSTANCE)
UniformConversionRev(XE_ELM_UNIFORM,XE_ELM_SURFACE)
UniformConversionRev(XE_ELM_UNIFORM,XE_ELM_TEXTURE)
UniformConversionRev(XE_ELM_UNIFORM,XE_ELM_WAVE_FORCE)

/*--- XenoN Core Psyche ---*/
class XENON_CORE_PSYCHE
{
#ifdef XE_COMPILE_EXPORT
public:
#else
private:
#endif /* XE_COMPILE_EXPORT */
void* Neurons;
void* Detectors;
void* Effectors;
void* SignalTrackers;
bool Evolution;
#ifdef XE_COMPILE_EXPORT
XENON_CORE_PSYCHE();
~XENON_CORE_PSYCHE();
void Progress(float factor);
#endif /* XE_COMPILE_EXPORT */
};

/*--- XenoN Core Rendertarget ---*/
class XENON_CORE_RENDERTARGET
{
#ifdef XE_COMPILE_EXPORT
public:
#else
private:
#endif /* XE_COMPILE_EXPORT */
double id;
HWND hwnd;
HDC hdc;
HGLRC hrc;
#ifdef XE_COMPILE_EXPORT
XENON_CORE_RENDERTARGET();
bool Create(HANDLE handle,XE_ESTATE mode,int bits,bool shared);
void Destroy();
bool Activate();
#endif /* XE_COMPILE_EXPORT */
};

/*--- XenoN Core Texture ---*/
class XENON_CORE_TEXTURE
{
#ifdef XE_COMPILE_EXPORT
public:
#else
private:
#endif /* XE_COMPILE_EXPORT */
double id;
unsigned int texture;
unsigned int width;
unsigned int height;
#ifdef XE_COMPILE_EXPORT
XENON_CORE_TEXTURE();
void Create();
bool LoadEmpty(int w,int h,bool achan);
bool LoadFromXET(char* fname,bool achan);
bool LoadFromXETmem(void* pbuff,bool achan);
bool LoadFromRGBA(char* fname,int _width,int _height);
bool LoadFromRGBAmem(void* pbuff,int _width,int _height);
bool LoadFromRender(int px,int py,int width,int height,bool asnew);
bool Activate();
void Destroy();
#endif /* XE_COMPILE_EXPORT */
};

/*--- XenoN Core FrameBuffer ---*/
class XENON_CORE_FRAMEBUFFER
{
#ifdef XE_COMPILE_EXPORT
public:
#else
private:
#endif /* XE_COMPILE_EXPORT */
double id;
XE_ELM_TEXTURE texture;
unsigned int hbuffer;
unsigned int hdepth;
int Width;
int Height;
#ifdef XE_COMPILE_EXPORT
XENON_CORE_FRAMEBUFFER();
bool Create(int width,int height);
bool CreateFromTexture(XE_ELM_TEXTURE tex);
void Destroy();
void Activate();
void ActivateTexture();
bool SwapTexture(XE_ELM_TEXTURE tex);
#endif /* XE_COMPILE_EXPORT */
};

/*--- XenoN Core VertexBuffer ---*/
class XENON_CORE_VERTEXBUFFER
{
#ifdef XE_COMPILE_EXPORT
public:
#else
private:
#endif /* XE_COMPILE_EXPORT */
double id;
unsigned int MaxSize;
unsigned int MaxSizeIndices;
unsigned int Vbuffer;
unsigned int Ibuffer;
#ifdef XE_COMPILE_EXPORT
XENON_CORE_VERTEXBUFFER();
bool Create(unsigned int maxsize,void* data,unsigned int maxsizeindices,void* dataindices);
void Destroy();
bool SendData(XE_ESTATE mode,unsigned int size,unsigned int offset,void* data);
void* Map(XE_ESTATE mode,XE_ESTATE type);
bool Unmap(XE_ESTATE mode);
#endif /* XE_COMPILE_EXPORT */
};

/*--- XenoN Core Shader ---*/
class XENON_CORE_SHADER
{
#ifdef XE_COMPILE_EXPORT
public:
#else
private:
#endif /* XE_COMPILE_EXPORT */
double id;
unsigned int shader;
XE_ESTATE type;
#ifdef XE_COMPILE_EXPORT
XENON_CORE_SHADER();
bool LoadFromFile(XE_ESTATE mode,char* filename);
bool LoadFromMemory(XE_ESTATE mode,void* buffer);
bool Activate();
void Destroy();
#endif /* XE_COMPILE_EXPORT */
};

/*--- XenoN Core Renderqueue ---*/
class XENON_CORE_RENDERQUEUE
{
#ifdef XE_COMPILE_EXPORT
public:
#else
private:
#endif /* XE_COMPILE_EXPORT */
double object;
double order;
#ifdef XE_COMPILE_EXPORT
XENON_CORE_RENDERQUEUE();
XENON_CORE_RENDERQUEUE(double _obj,double _ord);
#endif /* XE_COMPILE_EXPORT */
};

/*--- XenoN Core DisplayList ---*/
class XENON_CORE_DISPLAYLIST
{
#ifdef XE_COMPILE_EXPORT
public:
#else
private:
#endif /* XE_COMPILE_EXPORT */
double id;
unsigned int dlist;
#ifdef XE_COMPILE_EXPORT
XENON_CORE_DISPLAYLIST();
~XENON_CORE_DISPLAYLIST();
void begin();
void end();
void call();
#endif /* XE_COMPILE_EXPORT */
};

/*--- XenoN Core Substance ---*/
class XENON_CORE_SUBSTANCE
{
#ifdef XE_COMPILE_EXPORT
public:
#else
private:
#endif /* XE_COMPILE_EXPORT */
double id;
double object;
double x;
double y;
double z;
double range;
double mass;
double energy;
bool solid;
double inert_vx;
double inert_vy;
double inert_vz;
double xprev;
double yprev;
double zprev;
double xtemp;
double ytemp;
double ztemp;
double density;
#ifdef XE_COMPILE_EXPORT
XENON_CORE_SUBSTANCE();
#endif /* XE_COMPILE_EXPORT */
};

/*--- XenoN Core Ray ---*/
class XENON_CORE_RAY
{
#ifdef XE_COMPILE_EXPORT
public:
#else
private:
#endif /* XE_COMPILE_EXPORT */
double id;
double x;
double y;
double z;
double alpha;
double beta;
double angle;
double energy;
double frequency;
#ifdef XE_COMPILE_EXPORT
XENON_CORE_RAY();
#endif /* XE_COMPILE_EXPORT */
};

/*--- XenoN Core Covalence Force ---*/
class XENON_CORE_COVALENCE_FORCE
{
#ifdef XE_COMPILE_EXPORT
public:
#else
private:
#endif /* XE_COMPILE_EXPORT */
double id;
double force;
double resilience;
double range;
XE_ESTATE solid;
XE_ELM_SUBSTANCE subst1;
XE_ELM_SUBSTANCE subst2;
#ifdef XE_COMPILE_EXPORT
XENON_CORE_COVALENCE_FORCE();
#endif /* XE_COMPILE_EXPORT */
};

/*--- XenoN Core Force ---*/
class XENON_CORE_FORCE
{
#ifdef XE_COMPILE_EXPORT
public:
#else
private:
#endif /* XE_COMPILE_EXPORT */
double id;
double x;
double y;
double z;
double alpha;
double beta;
double force;
double speed;
XE_ELM_SUBSTANCE subst;
#ifdef XE_COMPILE_EXPORT
XENON_CORE_FORCE();
#endif /* XE_COMPILE_EXPORT */
};

/*--- XenoN Core Surface ---*/
class XENON_CORE_SURFACE
{
#ifdef XE_COMPILE_EXPORT
public:
#else
private:
#endif /* XE_COMPILE_EXPORT */
double id;
XE_ELM_TEXTURE texture_id;
XE_ESTATE texture_type;
double order;
double tex1_x;
double tex1_y;
double tex1_z;
double tex2_x;
double tex2_y;
double tex2_z;
double tex3_x;
double tex3_y;
double tex3_z;
double diffusion;
double mirror;
int orientation;
XE_ELM_SUBSTANCE subst1;
XE_ELM_SUBSTANCE subst2;
XE_ELM_SUBSTANCE subst3;
#ifdef XE_COMPILE_EXPORT
XENON_CORE_SURFACE();
#endif /* XE_COMPILE_EXPORT */
};

/*--- XenoN Core Particle ---*/
class XENON_CORE_PARTICLE
{
#ifdef XE_COMPILE_EXPORT
public:
#else
private:
#endif /* XE_COMPILE_EXPORT */
double id;
double width;
double height;
XE_ELM_TEXTURE texture_id;
XE_ESTATE texture_type;
XE_ELM_SUBSTANCE subst;
#ifdef XE_COMPILE_EXPORT
XENON_CORE_PARTICLE();
#endif /* XE_COMPILE_EXPORT */
};

/*--- XenoN Core Global Force ---*/
class XENON_CORE_GLOBAL_FORCE
{
#ifdef XE_COMPILE_EXPORT
public:
#else
private:
#endif /* XE_COMPILE_EXPORT */
double id;
double alpha;
double beta;
double force;
#ifdef XE_COMPILE_EXPORT
XENON_CORE_GLOBAL_FORCE();
#endif /* XE_COMPILE_EXPORT */
};

/*--- XenoN Core Black Hole ---*/
class XENON_CORE_BLACK_HOLE
{
#ifdef XE_COMPILE_EXPORT
public:
#else
private:
#endif /* XE_COMPILE_EXPORT */
double id;
double x;
double y;
double z;
double force;
double range;
XE_ELM_BLACK_HOLE bhole;
#ifdef XE_COMPILE_EXPORT
XENON_CORE_BLACK_HOLE();
#endif /* XE_COMPILE_EXPORT */
};

/*--- XenoN Core Wave Force ---*/
class XENON_CORE_WAVE_FORCE
{
#ifdef XE_COMPILE_EXPORT
public:
#else
private:
#endif /* XE_COMPILE_EXPORT */
double id;
double x;
double y;
double z;
double force;
double range;
double length;
double speed;
XE_ESTATE orientation;
#ifdef XE_COMPILE_EXPORT
XENON_CORE_WAVE_FORCE();
#endif /* XE_COMPILE_EXPORT */
};

/*--- XeGetState() ---*/
/*! Pobiera wartosc zmiennej stanu silnika.
\param index Zmienna stanu.
\return Wskaznik nieokreslony, ktory nalezy rzutowac na wlasciwy danej zmiennej typ. Przyklad: XE_ESTATE texturing=*(XE_ESTATE*)XeGetState(XE_RENDER_TEXTURING_2D);
*/
exportdll void* XeGetState(XE_ESTATE index);

/*--- XeSetState() ---*/
/*! Ustala wartosc zmiennej stanu silnika.
\param index Zmienna stanu.
\param ... Kolejne wartosci.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeSetState(XE_ESTATE index,...);

/*--- XeIsSupported() ---*/
/*! Sprawdza czy rozszerzenie jest obslugiwane.
\param extension Rozszerzenie.
\return true jesli rozszerzenie jest dostepne.
*/
exportdll bool XeIsSupported(XE_ESTATE extension);

/*--- XeLengthdirX() ---*/
/*! Przesuniecie w osi X o wektor opisany kierunkiem i dlugoscia.
\param alpha Kat alpha (poziomy).
\param beta Kat beta (pionowy).
\param length Dlugosc.
\return Przesuniecie w osi X.
*/
exportdll double XeLengthdirX(double alpha,double beta,double length);

/*--- XeLengthdirY() ---*/
/*! Przesuniecie w osi Y o wektor opisany kierunkiem i dlugoscia.
\param alpha Kat alpha (poziomy).
\param beta Kat beta (pionowy).
\param length Dlugosc.
\return Przesuniecie w osi Y.
*/
exportdll double XeLengthdirY(double alpha,double beta,double length);

/*--- XeLengthdirZ() ---*/
/*! Przesuniecie w osi Z o wektor opisany kierunkiem i dlugoscia.
\param alpha Kat alpha (poziomy).
\param beta Kat beta (pionowy).
\param length Dlugosc.
\return Przesuniecie w osi Z.
*/
exportdll double XeLengthdirZ(double alpha,double beta,double length);

/*--- XeDirectionAlpha() ---*/
/*! Oblicza kierunek alpha (poziomy) pomiedzy dwoma punktami.
\param x1 Pozycja X pierwszego punktu.
\param y1 Pozycja Y pierwszego punktu.
\param z1 Pozycja Z pierwszego punktu.
\param x2 Pozycja X drugiego punktu.
\param y2 Pozycja Y drugiego punktu.
\param z2 Pozycja Z drugiego punktu.
\return Kierunek alpha.
*/
exportdll double XeDirectionAlpha(double x1,double y1,double z1,double x2,double y2,double z2);

/*--- XeDirectionBeta() ---*/
/*! Oblicza kierunek beta (poziomy) pomiedzy dwoma punktami.
\param x1 Pozycja X pierwszego punktu.
\param y1 Pozycja Y pierwszego punktu.
\param z1 Pozycja Z pierwszego punktu.
\param x2 Pozycja X drugiego punktu.
\param y2 Pozycja Y drugiego punktu.
\param z2 Pozycja Z drugiego punktu.
\return Kierunek beta.
*/
exportdll double XeDirectionBeta(double x1,double y1,double z1,double x2,double y2,double z2);

/*--- XeDistance() ---*/
/*! Oblicza odleglosc pomiedzy dwoma punktami.
\param x1 Pozycja X pierwszego punktu.
\param y1 Pozycja Y pierwszego punktu.
\param z1 Pozycja Z pierwszego punktu.
\param x2 Pozycja X drugiego punktu.
\param y2 Pozycja Y drugiego punktu.
\param z2 Pozycja Z drugiego punktu.
\return Dystans.
*/
exportdll double XeDistance(double x1,double y1,double z1,double x2,double y2,double z2);

/*--- XeCameraOrtho() ---*/
/*! Uaktywnia projekcje prostopadla.
\param px Pozycja X na scenie.
\param py Pozycja Y na scenie.
\param width Szerokosc na scenie.
\param height Wysokosc na scenie.
\param angle Kat obrotu widoku.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeCameraOrtho(double px,double py,double width,double height,double angle);

/*--- XeCameraToOrtho() ---*/
/*! Rzutuje punkt 3D na plaszczyzne ekranu. (Funkcja nie w pelni poprawnie dzialajaca; zostanie zastapiona)
\param x Pozycja X.
\param y Pozycja Y.
\param z Pozycja Z.
\return 1 jesli punkt znajduje sie przed kamera; 0 jesli lezy na plaszczyznie kamery; -1 jesli za kamera.
*/
exportdll int XeCameraToOrtho(double x,double y,double z);

/*--- XeCameraOrthoSign() ---*/
/*! Pobiera strone kamery po ktorej znajduje sie ostatnio rzutowany punkt.
\return 1 jesli punkt znajduje sie przed kamera; 0 jesli lezy na plaszczyznie kamery; -1 jesli za kamera.
*/
exportdll int XeCameraOrthoSign();

/*--- XeCameraOrthoX() ---*/
/*! Pobiera pozycje X ostatnio rzutowanego punktu.
\return Pozycja X.
*/
exportdll double XeCameraOrthoX();

/*--- XeCameraOrthoY() ---*/
/*! Pobiera pozycje Y ostatnio rzutowanego punktu.
\return Pozycja Y.
*/
exportdll double XeCameraOrthoY();

/*--- XeCameraSet() ---*/
/*! Uaktywnia projekcje perspektywistyczna.
\param alpha Kierunek alpha oka kamery.
\param beta Kierunek beta oka kamery.
\param gamma Kierunek gamma oka kamery.
\param x Pozycja X kamery.
\param y Pozycja Y kamery.
\param z Pozycja Z kamery.
\param length Odleglosc oka od kamery.
\param projection Typ projekcji: false jesli oko patrzy od punktu kamery; true jesli oko patrzy na punkt kamery;
\return true jesli operacja powiodla sie.
*/
exportdll bool XeCameraSet(double alpha,double beta,double gamma,double x,double y,double z,double length,bool projection);

/*--- XeCameraUpdate() ---*/
/*! Aktualizuje ostatnio ustawiona projekcje perspektywistyczna.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeCameraUpdate();

/*--- XeCameraFromX() ---*/
/*! Zwraca pozycje X z ktorej patrzy kamera.
\return Pozycja X.
*/
exportdll double XeCameraFromX();

/*--- XeCameraFromY() ---*/
/*! Zwraca pozycje Y z ktorej patrzy kamera.
\return Pozycja Y.
*/
exportdll double XeCameraFromY();

/*--- XeCameraFromZ() ---*/
/*! Zwraca pozycje Z z ktorej patrzy kamera.
\return Pozycja Z.
*/
exportdll double XeCameraFromZ();

/*--- XeCameraToX() ---*/
/*! Zwraca pozycje X na ktora patrzy kamera.
\return Pozycja X.
*/
exportdll double XeCameraToX();

/*--- XeCameraToY() ---*/
/*! Zwraca pozycje Y na ktora patrzy kamera.
\return Pozycja Y.
*/
exportdll double XeCameraToY();

/*--- XeCameraToZ() ---*/
/*! Zwraca pozycje Z na ktora patrzy kamera.
\return Pozycja Z.
*/
exportdll double XeCameraToZ();

/*--- XeCameraUpX() ---*/
/*! Zwraca wartosc X znormalizowanego wektora gory kamery.
\return Znormalizowana wartosc X.
*/
exportdll double XeCameraUpX();

/*--- XeCameraUpY() ---*/
/*! Zwraca wartosc Y znormalizowanego wektora gory kamery.
\return Znormalizowana wartosc Y.
*/
exportdll double XeCameraUpY();

/*--- XeCameraUpZ() ---*/
/*! Zwraca wartosc Z znormalizowanego wektora gory kamery.
\return Znormalizowana wartosc Z.
*/
exportdll double XeCameraUpZ();

/*--- XeInitiation() ---*/
/*! Inicjacja silnika.
\return true jesli operacja poziodla sie.
*/
exportdll bool XeInitiation();

namespace Photon
{

/*--- XeRenderViewport() ---*/
/*! Aktualizuje prostokat renderowania, uzywajac wartosci zmiennych stanu: XE_RENDER_X, XE_RENDER_Y, XE_RENDER_WIDTH, XE_RENDER_HEIGHT. */
exportdll void XeRenderViewport();

/*--- XeRenderScene() ---*/
/*! Renderuje scene.
\param flag Flagi operacji. Dostepne wartosci:<br>
XE_FLAG_SWAP: Zamienia bufory ramki.<br>
XE_FLAG_COLORBUFF: Czysci bufor koloru.<br>
XE_FLAG_DEPTHBUFF: Czysci bufor glebi.<br>
XE_FLAG_ACCUMBUFF: Czysci bufor akumulacyjny.<br>
XE_FLAG_STENCILBUFF: Czysci bufor szablonowy.<br>
XE_FLAG_TRANSFORMIDENT: Resetuje transformacje (laduje macierz jednostkowa).
\return true jesli operacja powiodla sie
*/
exportdll bool XeRenderScene(unsigned int flag);

/*--- XeRenderGetData() ---*/
/*! Pobiera dane pixeli z bufora ramki.
\param x Pozycja X (zaczynajac od lewego dolnego rogu).
\param y Pozycja Y (zaczynajac od lewego dolnego rogu).
\param w Szerokosc.
\param h Wysokosc.
\param format Typ bufora. Dostepne wartosci:<br>
XE_RENDER_FORMAT_RGBA: Bufor koloru.<br>
XE_RENDER_FORMAT_STENCIL: Bufor szablonowy.<br>
XE_RENDER_FORMAT_DEPTH: Bufor glebi.
\param buffer Wskaznik na bufor do ktorego zapisze dane.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeRenderGetData(int x,int y,int w,int h,XE_ESTATE format,void* buffer);

/*--- XeRenderDrawData() ---*/
/*! Wpisuje dane pixeli do bufora ramki.
\param x Pozycja X (zaczynajac od lewego dolnego rogu).
\param y Pozycja Y (zaczynajac od lewego dolnego rogu).
\param w Szerokosc.
\param h Wysokosc.
\param format Typ bufora. Dostepne wartosci:<br>
XE_RENDER_FORMAT_RGBA: Bufor koloru.<br>
XE_RENDER_FORMAT_STENCIL: Bufor szablonowy.<br>
XE_RENDER_FORMAT_DEPTH: Bufor glebi.
\param buffer Wskaznik na bufor z ktorego wczyta dane.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeRenderDrawData(int x,int y,int w,int h,XE_ESTATE format,void* buffer);

/*--- XeRenderVsync() ---*/
/*! Wlacza badz wylacza pionowa synchronizacje obrazu.
\param mode XE_TRUE jesli ma wlaczyc, XE_FALSE jesli ma wylaczyc synchronizacje.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeRenderVsync(XE_ESTATE mode);

/*--- XePrimitiveBegin() ---*/
/*! Rozpoczyna proces rysowania prymitywow.
\param type Typ figur. Dostepne wartosci:<br>
XE_POINTS: Punkty.<br>
XE_LINES: Linie.<br>
XE_LINELOOP: Petla linii.<br>
XE_LINESTRIP: Tasma linii.<br>
XE_TRIANGLES: Trojkaty.<br>
XE_TRIANGLESTRIP: Tasma trojkatow.<br>
XE_TRIANGLEFAN: Wachlaz trojkatow.<br>
XE_QUADS: Czworokaty.<br>
XE_QUADSTRIP: Tasma czworokatow.<br>
XE_POLYGON: Wielokat.
*/
exportdll void XePrimitiveBegin(XE_ESTATE type);

/*--- XePrimitiveEnd() ---*/
/*! Konczy proces rysowania prymitywow. */
exportdll void XePrimitiveEnd();

/*--- XePutVertex() ---*/
/*! Dodaje wierzcholek do sceny.
\param x Pozycja X.
\param y Pozycja Y.
\param z Pozycja Z.
\param w Pozycja W.
*/
exportdll void XePutVertex(double x,double y,double z,double w);

/*--- XePutNormal() ---*/
/*! Ustala wektor normalny.
\param nx Wartosc X.
\param ny Wartosc Y.
\param nz Wartosc Z.
*/
exportdll void XePutNormal(double nx,double ny,double nz);

/*--- XePutFogvertex() ---*/
/*! Ustala gestosc mgly.
\param d Gestosc (zakres: <0;1>).
*/
exportdll void XePutFogvertex(double d);

/*--- XePutTexvertex() ---*/
/*! Ustala koordynaty tekstury.
\param tx Pozycja X tekstury.
\param ty Pozycja Y tekstury.
\param tz Pozycja Z tekstury.
\param tw Pozycja W tekstury.
*/
exportdll void XePutTexvertex(double tx,double ty,double tz,double tw);

/*--- XePutMultitexvertex() ---*/
/*! Ustala koordynaty multitekstury.
\param un Numer jednostki teksturujacej.
\param tx Pozycja X multitekstury.
\param ty Pozycja Y multitekstury.
\param tz Pozycja Z multitekstury.
\param tw Pozycja W multitekstury.
\return true jesli operacja powiodla sie.
*/
exportdll bool XePutMultitexvertex(double un,double tx,double ty,double tz,double tw);

/*--- XeRenderTargetCreate() ---*/
/*! Tworzy kontekst renderowania.
\param elm Wskaznik na element RenderTarget. Zwraca do niego utworzony element jesli operacja powiodla sie.
\param handle Uchwyt okna do ktorego podepniemy kontekst.
\param mode Tryb rysowania. Musi byc: XE_DRAW_TO_WINDOW.
\param bits Ilosc bitow glebi koloru (8,16,24,32).
\param share Wspoldzielenie zasobow.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeRenderTargetCreate(XE_ELM_RENDERTARGET* elm,HANDLE handle,XE_ESTATE mode,int bits,bool share);

/*--- XeRenderTargetDestroy() ---*/
/*! Niszczy kontekst renderowania.
\param index Element RenderTarget.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeRenderTargetDestroy(XE_ELM_RENDERTARGET index);

/*--- XeRenderTargetActivate) ---*/
/*! Aktywuje kontekst renderowania.
\param index Element RenderTarget.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeRenderTargetActivate(XE_ELM_RENDERTARGET index);

/*--- XeRenderTargetRelease() ---*/
/*! Zwalnia aktywny kontekst renderowania. */
exportdll void XeRenderTargetRelease();

/*--- XeFrameBufferInit() ---*/
/*! Inicjuje obsluge buforow ramki.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeFrameBufferInit();

/*--- XeFrameBufferCreate() ---*/
/*! Tworzy bufor ramki.
\param elm Wskaznik na element bufora ramki.
\param width Szerokosc.
\param height Wysokosc.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeFrameBufferCreate(XE_ELM_FRAMEBUFFER* elm,int width,int height);

/*--- XeFrameBufferCreateFromTexture() ---*/
/*! Tworzy bufor ramki.
\param elm Wskaznik na element bufora ramki.
\param tex Element tekstury.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeFrameBufferCreateFromTexture(XE_ELM_FRAMEBUFFER* elm,XE_ELM_TEXTURE tex);

/*--- XeFrameBufferDestroy() ---*/
/*! Niszczy bufor ramki.
\param index Element bufora ramki.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeFrameBufferDestroy(XE_ELM_FRAMEBUFFER index);

/*--- XeFrameBufferActivate() ---*/
/*! Aktywuje bufor ramki.
\param index Element bufora ramki.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeFrameBufferActivate(XE_ELM_FRAMEBUFFER index);

/*--- XeFrameBufferActivateTexture() ---*/
/*! Aktywuje bufor ramki jako teksture.
\param index Element bufora ramki.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeFrameBufferActivateTexture(XE_ELM_FRAMEBUFFER index);

/*--- XeFrameBufferUnactivate() ---*/
/*! Dezaktywuje bufor ramki. */
exportdll void XeFrameBufferUnactivate();

/*--- XeFrameBufferMipmap() ---*/
/*! Generuje mipmapy dla tekstury bufora ramki.
\param index Element bufora ramki.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeFrameBufferMipmap(XE_ELM_FRAMEBUFFER index);

/*--- XeFrameBufferSwapTexture() ---*/
/*! Podmienia zawartosc tekstury, tylko wtedy gdy maja te same wymiary.
\param tex Element tekstury.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeFrameBufferSwapTexture(XE_ELM_FRAMEBUFFER index,XE_ELM_TEXTURE tex);

/*--- XeFrameBufferGet() ---*/
/*! Pobiera wartosc zmiennej stanu bufora ramki.
\param index Element bufora ramki.
\param type Zmienna stanu. Dostepne wartosci:<br>
XE_PIXELBUFFER_WIDTH: Szerokosc.<br>
XE_PIXELBUFFER_HEIGHT: Wysokosc.<br>
\return Wskaznik nieokreslony na zmienna stanu bufora ramki. Nalezy rzutowac na konkretny typ danej zmiennej stanu.
*/
exportdll void* XeFrameBufferGet(XE_ELM_FRAMEBUFFER index,XE_ESTATE type);

/*--- XeShaderInit() ---*/
/*! Inicjuje obsluge shaderow.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeShaderInit();

/*--- XeShaderModel() ---*/
/*! Zwraca Shader Model obslugiwany przez silnik i karte graficzna.
\return obsługiwany Shader Model.
*/
exportdll int XeShaderModel();

/*--- XeShaderCreate() ---*/
/*! Tworzy shader.
\param elm Wskaznik na element shadera.
*/
exportdll void XeShaderCreate(XE_ELM_SHADER* elm);

/*--- XeShaderDestroy() ---*/
/*! Niszczy shader.
\param index Element shadera.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeShaderDestroy(XE_ELM_SHADER index);

/*--- XeShaderLoad() ---*/
/*! Laduje shader z pliku.
\param index Element shadera.
\param mode Profil shadera.
\param filename Nazwa pliku.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeShaderLoad(XE_ELM_SHADER index,XE_ESTATE mode,char* filename);

/*--- XeShaderLoadFromMemory() ---*/
/*! Laduje shader z pamieci.
\param index Element shadera.
\param mode Profil shadera.
\param buff Bufor z zawartoscia shadera.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeShaderLoadFromMemory(XE_ELM_SHADER index,XE_ESTATE mode,void* buff);

/*--- XeShaderActivate() ---*/
/*! Aktywuje shader.
\param index Element shadera.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeShaderActivate(XE_ELM_SHADER index);

/*--- XeShaderLocalParam() ---*/
/*! Ustala wartosc zmiennej lokalnej.
\param mode Profil shadera.
\param index Indeks zmiennej lokalnej.
\param x Wartosc X.
\param y Wartosc Y.
\param z Wartosc Z.
\param w Wartosc W.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeShaderLocalParam(XE_ESTATE mode,int index,double x,double y,double z,double w);

/*--- XeShaderEnvParam() ---*/
/*! Ustala wartosc zmiennej srodowiskowej.
\param mode Profil shadera.
\param index Indeks zmiennej srodowiskowej.
\param x Wartosc X.
\param y Wartosc Y.
\param z Wartosc Z.
\param w Wartosc W.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeShaderEnvParam(XE_ESTATE mode,int index,double x,double y,double z,double w);

/*--- XeTextureCreate() ---*/
/*! Tworzy teksture i automatycznie ja aktywuje.
\param elm Wskaznik na element tekstury. Zwraca do niego utworzony element jesli operacja powiodla sie.
*/
exportdll void XeTextureCreate(XE_ELM_TEXTURE* elm);

/*--- XeTextureDestroy() ---*/
/*! Niszczy teksture.
\param index Element tekstury.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeTextureDestroy(XE_ELM_TEXTURE index);

/*--- XeTextureLoad() ---*/
/*! Laduje teksture z pliku.
\param index Element tekstury.
\param filename Cstring z nazwa pliku.
\param achannel Jesli true to laduje z kanalem przezroczystosci.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeTextureLoad(XE_ELM_TEXTURE index,char* filename,bool achannel);

/*--- XeTextureLoadEmpty() ---*/
/*! Laduje pusta teksture
\param index Element tekstury.
\param width Szerokosc tekstury.
\param height Wysokosc tekstury.
\param achannel Jesli true to laduje z kanalem przezroczystosci.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeTextureLoadEmpty(XE_ELM_TEXTURE index,int width,int height,bool achannel);

/*--- XeTextureLoadFromMemory() ---*/
/*! Laduje teksture z pamieci.
\param index Element tekstury.
\param buff Wskaznik na bufor w ktorym sa dane tekstury.
\param achannel Jesli true to laduje z kanalem przezroczystosci.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeTextureLoadFromMemory(XE_ELM_TEXTURE index,void* buff,bool achannel);

/*--- XeTextureLoadRGBA() ---*/
/*! Laduje teksture RAW RGBA z pliku.
\param index Element tekstury.
\param filename Cstring z nazwa pliku.
\param width Szerokosc tekstury RAW RGBA.
\param height Wysokosc tekstury RAW RGBA.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeTextureLoadRGBA(XE_ELM_TEXTURE index,char* filename,int width,int height);

/*--- XeTextureLoadRGBAFromMemory() ---*/
/*! Laduje teksture RAW RGBA z pamieci.
\param index Element tekstury.
\param buff Wskaznik na bufor w ktorym sa dane tekstury RAW RGBA.
\param width Szerokosc tekstury RAW RGBA.
\param height Wysokosc tekstury RAW RGBA.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeTextureLoadRGBAFromMemory(XE_ELM_TEXTURE index,void* buff,int width,int height);

/*--- XeTextureFromRender() ---*/
/*! Laduje teksture z bufora kolorow aktywnego kontekstu renderujacego.
\param index  Element tekstury.
\param x Pozycja X w buforze.
\param y Pozycja Y w buforze.
\param width Szerokosc w buforze.
\param height Wysokosc w buforze.
\param asnew Tworzy nowa teksture jesli wartosc jest true, jesli zas false to zastepuje istniejace dane.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeTextureFromRender(XE_ELM_TEXTURE index,int x,int y,int width,int height,bool asnew);

/*--- XeTextureActivate() ---*/
/*! Aktywuje teksture.
\param index Element tekstury.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeTextureActivate(XE_ELM_TEXTURE index);

/*--- XeTextureUnactivate() ---*/
/*! Dezaktywuje teksture. */
exportdll void XeTextureUnactivate();

/*--- XeTextureParamater() ---*/
/*! Ustala parametr aktywnej tekstury.
\param type Typ parametru. Dostepne wartosci:<br>
XE_TEXTURE_PARAM_PRIORITY: Priorytet tekstury (zakres: <0;1>).<br>
XE_TEXTURE_PARAM_MIN_FILTER: Filtr pomniejszajacy.<br>
XE_TEXTURE_PARAM_MAG_FILTER: Filtr powiekszajacy.<br>
XE_TEXTURE_PARAM_WRAP_S: Poziome powtarzanie tekstury.<br>
XE_TEXTURE_PARAM_WRAP_T: Pionowe powtarzanie tekstury.
\param value Wartosc parametru.
*/
exportdll void XeTextureParameter(XE_ESTATE type,float value);

/*--- XeTextureMode() ---*/
/*! Ustala tryb mieszania kolorow tekstur z kolorami wierzcholkow.
\param mode Tryb mieszania. Dostepne wartosci:<br>
XE_TEXTURE_MODE_ADD: Dodaje kolor tekstury do koloru wierzcholka.<br>
XE_TEXTURE_MODE_MODULATE: Moduluje kolor tekstury przez kolor wierzcholka.<br>
XE_TEXTURE_MODE_DECAL: Zastepuje kolor wierzcholka kolorem tekstury.<br>
XE_TEXTURE_MODE_BLEND: Miesza kolor wierzcholka z kolorem tekstury.<br>
XE_TEXTURE_MODE_REPLACE: Zastepuje kolor wierzcholka kolorem tekstury.<br>
XE_TEXTURE_MODE_COMBINE: Opcje mieszania dla multitekstur .[Obecnie wartosc nie uzywana]
*/
exportdll void XeTextureMode(XE_ESTATE mode);

/*--- XeTextureMulti() ---*/
/*! Wlacza tryb multiteksturingu.
Pozwala to na mieszanie tekstur poprzez uzywanie wiecej niz jednej jednostki teksturujacej.
Dodatkowo umieszcza w zmiennej stanu XE_TEXTURE_UNIT_COUNT wartosc maxymalnej liczby jednostek teksturowania jaka dysponuje karta.
\return true jesli operacja powiodla sie; false jesli karta nie obsluguje wieloteksturowania.
*/
exportdll bool XeTextureMulti();

/*--- XeTextureUnit() ---*/
/*! Aktywuje jednostke teksturowania.
\param unit Numer jednostki teksturowania.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeTextureUnit(int unit);

/*--- XeTextureGet() ---*/
/*! Pobiera wartosc zmiennej stanu tekstury.
\param index Element tekstury.
\param type Zmienna stanu. Dostepne wartosci:<br>
XE_TEXTURE_WIDTH: Szerokosc tekstury.<br>
XE_TEXTURE_HEIGHT: Wysokosc tekstury.
\return Wskaznik nieokreslony na zmienna stanu tekstury. Nalezy rzutowac na konkretny typ danej zmiennej stanu.
*/
exportdll void* XeTextureGet(XE_ELM_TEXTURE index,XE_ESTATE type);

/*--- XeDisplaylistCreate() ---*/
/*! Tworzy liste wyswietlania.
Czyli liste polecen graficznych rysujacych scene. Polecenia sa kompilowane i zapamietywane, a podczas wywolywania listy polecenia sa przetwarzane szybko.
\param elm Wskaznik na element listy wyswietlania. Zwraca do niego utworzony element jesli operacja powiodla sie.
*/
exportdll void XeDisplaylistCreate(XE_ELM_DISPLAYLIST* elm);

/*--- XeDisplaylistDestroy() ---*/
/*! Niszczy liste wyswietlania.
\param index Element listy wyswietlania.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeDisplaylistDestroy(XE_ELM_DISPLAYLIST index);

/*--- XeDisplaylistBegin() ---*/
/*! Rozpoczyna zapamietywanie polecen rysowania.
\param index Element listy wyswietlania.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeDisplaylistBegin(XE_ELM_DISPLAYLIST index);

/*--- XeDisplaylistEnd() ---*/
/*! Konczy zapamietywanie polecen rysowania. */
exportdll void XeDisplaylistEnd();

/*--- XeDisplaylistCall() ---*/
/*! Wywoluje zapamietane polecenia z listy wyswietlania.
\param index Element listy wyswietlania.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeDisplaylistCall(XE_ELM_DISPLAYLIST index);

} // namespace Photon

namespace Chaos
{

/*--- XePhysicsSetFactor() ---*/
/*! Ustala wspolczynnik fizyczny.
\param factor Wspolczynnik.
*/
exportdll void XePhysicsSetFactor(double factor);

/*--- XePhysicsGetFactor() ---*/
/*! Zwraca wspolczynnik fizyczny.
\return Wspolczynnik.
*/
exportdll double XePhysicsGetFactor();

/*--- XePhysicsUpdate() ---*/
/*! Aktualizuje fizyke. */
exportdll void XePhysicsUpdate();

/*--- XeSubstanceCreate() ---*/
/*! Tworzy substancje.
\param elm Wskaznik na element substancji. Zwraca do niego utworzony element jesli operacja powiodla sie.
*/
exportdll void XeSubstanceCreate(XE_ELM_SUBSTANCE* elm);

/*--- XeSubstanceSetObject() ---*/
/*! Ustala obiekt.
\param index Element substancji.
\param object Obiekt.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeSubstanceSetObject(XE_ELM_SUBSTANCE index,double object);

/*--- XeSubstanceSetPosition() ---*/
/*! Ustala pozycje.
\param index Element substancji.
\param x Pozycja X.
\param y Pozycja Y.
\param z Pozycja Z.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeSubstanceSetPosition(XE_ELM_SUBSTANCE index,double x,double y,double z);

/*--- XeSubstanceSetRange() ---*/
/*! Ustala promien.
\param index Element substancji.
\param range Promien.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeSubstanceSetRange(XE_ELM_SUBSTANCE index,double range);

/*--- XeSubstanceSetMass() ---*/
/*! Ustala mase.
\param index Element substancji.
\param mass Masa.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeSubstanceSetMass(XE_ELM_SUBSTANCE index,double mass);

/*--- XeSubstanceSetEnergy() ---*/
/*! Ustala energie.
\param index Element substancji.
\param energy Energia.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeSubstanceSetEnergy(XE_ELM_SUBSTANCE index,double energy);

/*--- XeSubstanceSetSolid() ---*/
/*! Ustala czy obiekt jest trwaly.
\param index Element substancji.
\param solid Trwalosc.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeSubstanceSetSolid(XE_ELM_SUBSTANCE index,bool solid);

/*--- XeSubstanceAddInertion() ---*/
/*! Dodaje bezwladnosc.
\param index Element substancji.
\param x Wartosc X.
\param y Wartosc Y.
\param z Wartosc Z.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeSubstanceAddInertion(XE_ELM_SUBSTANCE index,double x,double y,double z);

/*--- XeSubstanceResetInertion() ---*/
/*! Zeruje bezwladnosc.
\param index Element substancji.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeSubstanceResetInertion(XE_ELM_SUBSTANCE index);

/*--- XeSubstanceGet() ---*/
/*! Pobiera wartosc zmiennej stanu substancji.
\param index Element substancji.
\param type Zmienna stanu substancji. Dostepne wartosci:<br>
XE_SUBSTANCE_ID: Indeks substancji.<br>
XE_SUBSTANCE_OBJECT: Obiekt.<br>
XE_SUBSTANCE_X: Pozycja X.<br>
XE_SUBSTANCE_Y: Pozycja Y.<br>
XE_SUBSTANCE_Z: Pozycja Z.<br>
XE_SUBSTANCE_RANGE: Promien.<br>
XE_SUBSTANCE_MASS: Masa.<br>
XE_SUBSTANCE_ENERGY: Energia.<br>
XE_SUBSTANCE_SOLID: Trwalosc.<br>
XE_SUBSTANCE_INERT_X: Wartosc X bezwladnosci.<br>
XE_SUBSTANCE_INERT_Y: Wartosc Y bezwladnosci.<br>
XE_SUBSTANCE_INERT_Z: Wartosc Z bezwladnosci.
\return Wskaznik nieokreslony na zmienna stanu substancji. Nalezy rzutowac na konkretny typ danej zmiennej stanu. 
*/
exportdll void* XeSubstanceGet(XE_ELM_SUBSTANCE index,XE_ESTATE type);

/*--- XeSubstanceDestroy() ---*/
/*! Niszczy substancje.
\param index Element substancji.
*/
exportdll bool XeSubstanceDestroy(XE_ELM_SUBSTANCE index);

/*--- XeRayCreate() ---*/
/*! Tworzy wiazke (promien).
\param elm Wskaznik na element wiazki. Zwraca do niego utworzony obiekt jesli operacja powiodla sie.
*/
exportdll void XeRayCreate(XE_ELM_RAY* elm);

/*--- XeRaySetPosition() ---*/
/*! Ustala pozycje.
\param index Element wiazki.
\param x Pozycja X.
\param y Pozycja Y.
\param z Pozycja Z.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeRaySetPosition(XE_ELM_RAY index,double x,double y,double z);

/*--- XeRaySetAlpha() ---*/
/*! Ustala kierunek alpha.
\param index Element wiazki.
\param alpha Kat alpha.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeRaySetAlpha(XE_ELM_RAY index,double alpha);

/*--- XeRaySetBeta() ---*/
/*! Ustala kierunek beta.
\param index Element wiazki.
\param beta Kat beta.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeRaySetBeta(XE_ELM_RAY index,double beta);

/*--- XeRaySetAngle() ---*/
/*! Ustala kat zasiegu (rozproszenia).
\param index Element wiazki.
\param angle Kat angle.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeRaySetAngle(XE_ELM_RAY index,double angle);

/*--- XeRaySetEnergy() ---*/
/*! Ustala energie.
\param index Element wiazki.
\param energy Energia.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeRaySetEnergy(XE_ELM_RAY index,double energy);

/*--- XeRaySetFrequency() ---*/
/*! Ustala czestotliwosc.
\param index Element wiazki.
\param frequency Czestotliwosc.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeRaySetFrequency(XE_ELM_RAY index,double frequency);

/*--- XeRayGet() ---*/
/*! Pobiera wartosc zmiennej stanu wiazki.
\param index Element wiazki.
\param type Zmienna stanu wiazki. Dostepne wartosci:<br>
XE_RAY_ID: Indeks wiazki.<br>
XE_RAY_X: Pozycja X.<br>
XE_RAY_Y: Pozycja Y.<br>
XE_RAY_Z: Pozycja Z.<br>
XE_RAY_ALPHA: Kierunek alpha.<br>
XE_RAY_BETA: Kierunek beta.<br>
XE_RAY_ANGLE: Zasieg (rozproszenie).<br>
XE_RAY_ENERGY: Energia.<br>
XE_RAY_FREQUENCY: Czestotliwosc.
\return Wskaznik nieokreslony na zmienna stanu wiazki. Nalezy rzutowac na konkretny typ danej zmiennej stanu.
*/
exportdll void* XeRayGet(XE_ELM_RAY index,XE_ESTATE type);

/*--- XeRayDestroy() ---*/
/*! Niszczy wiazke.
\param index Element wiazki.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeRayDestroy(XE_ELM_RAY index);

/*--- XeCovalenceForceCreate() ---*/
/*! Tworzy sile kowalencyjna (miedzyczasteczkowa).
\param elm Wskaznik na element sily kowalencyjnej. Zwraca do niego utworzony element jesli operacja powiodla sie.
*/
exportdll void XeCovalenceForceCreate(XE_ELM_COVALENCE_FORCE* elm);

/*--- XeCovalenceForceSetSubstance() ---*/
/*! Ustala substancje.
\param index Element sily kowalencyjnej.
\param type Typ substancji. Dostepne wartosci:<br>
XE_COVALENCE_FORCE_SUBSTANCE_1: Pierwsza substancja.<br>
XE_COVALENCE_FORCE_SUBSTANCE_2: Druga substancja.
\param substance Element substancji.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeCovalenceForceSetSubstance(XE_ELM_COVALENCE_FORCE index,XE_ESTATE type,XE_ELM_SUBSTANCE substance);

/*--- XeCovalenceForceSetForce() ---*/
/*! Ustala sile.
\param index Element sily kowalencyjnej.
\param force Sila.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeCovalenceForceSetForce(XE_ELM_COVALENCE_FORCE index,double force);

/*--- XeCovalenceSetForceResilience() ---*/
/*! Ustala sprezystosc.
\param index Element sily kowalencyjnej.
\param resilience Sprezystosc.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeCovalenceForceSetResilience(XE_ELM_COVALENCE_FORCE index,double resilience);

/*--- XeCovalenceForceSetRange() ---*/
/*! Ustala zasieg sily, po ktorego przekroczeniu nastepuje rozerwanie sily.
\param index Element sily kowalencyjnej.
\param range Zasieg.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeCovalenceForceSetRange(XE_ELM_COVALENCE_FORCE index,double range);

/*--- XeCovalenceForceSetSolid() ---*/
/*! Ustala zasieg sily, po ktorego przekroczeniu nastepuje rozerwanie sily.
\param index Element sily kowalencyjnej.
\param range Solidnosc; zachowanie sily po przekroczeniu zasiegu. Dostepne wartosci:<br>
XE_COVALENCE_FORCE_SOLID_ALWAYS: Utrzymuje sile zawsze.<br>
XE_COVALENCE_FORCE_SOLID_TEMPORARY: Tymczasowo nie utrzymuje sily, dopoki dystans pomiedzy substancjami nie bedzie w zasiegu sily.<br>
XE_COVALENCE_FORCE_SOLID_NEVER: Po przekroczeniu zasiegu zrywa polaczenie.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeCovalenceForceSetSolid(XE_ELM_COVALENCE_FORCE index,XE_ESTATE solid);

/*--- XeCovalenceForceGet() ---*/
/*! Pobiera wartosc zmiennej stanu sily kowalencyjnej.
\param index Element sily kowalencyjnej.
\param type Zmienna stanu sily kowalencyjnej. Dostepne wartosci:<br>
XE_COVALENCE_FORCE_ID: Indeks sily kowalencyjnej.<br>
XE_COVALENCE_FORCE_SUBSTANCE_1: Pierwsza substancja.<br>
XE_COVALENCE_FORCE_SUBSTANCE_2: Druga substancja.<br>
XE_COVALENCE_FORCE_FORCE: Sila.<br>
XE_COVALENCE_FORCE_RESILIENCE: Sprezystosc.<br>
XE_COVALENCE_FORCE_RANGE: Zasieg. Jesli odleglosc substancji bedzie wieksza niz zasieg i sila nie jest solidna to nastapi rozerwanie na stale badz na chwile.<br>
XE_COVALENCE_FORCE_SOLID: Solidnosc, utrzymywanie polaczenia po przekroczeniu zasiegu: zawsze, tymczasowo lub nigdy.
\return Wskaznik nieokreslony na zmienna stanu sily kowalencyjnej. Nalezy rzutowac na konkretny typ danej zmiennej stanu.
*/
exportdll void* XeCovalenceForceGet(XE_ELM_COVALENCE_FORCE index,XE_ESTATE type);

/*--- XeCovalenceForceDestroy() ---*/
/*! Niszczy sile kowalencyjna.
\param index Element sily kowalencyjnej.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeCovalenceForceDestroy(XE_ELM_COVALENCE_FORCE index);

/*--- XeForceCreate() ---*/
/*! Tworzy sile.
\param elm Wskaznik na element sily. Zwraca do niego utworzony element jesli operacja powiodla sie.
*/
exportdll void XeForceCreate(XE_ELM_FORCE* elm);

/*--- XeForceSetPosition() ---*/
/*! Ustala pozycje.
\param index Element sily.
\param x Pozycja X.
\param y Pozycja Y.
\param z Pozycja Z.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeForceSetPosition(XE_ELM_FORCE index,double x,double y,double z);

/*--- XeForceSetAlpha() ---*/
/*! Ustala kierunek alpha.
\param index Element sily.
\param alpha Kat alpha.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeForceSetAlpha(XE_ELM_FORCE index,double alpha);

/*--- XeForceSetBeta() ---*/
/*! Ustala kierunek beta.
\param index Element sily.
\param beta Kat beta.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeForceSetBeta(XE_ELM_FORCE index,double beta);

/*--- XeForceSetForce() ---*/
/*! Ustala sile.
\param index Element sily.
\param force Sila.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeForceSetForce(XE_ELM_FORCE index,double force);

/*--- XeForceSetSpeed() ---*/
/*! Ustala predkosc.
\param index Element sily.
\param speed Predkosc.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeForceSetSpeed(XE_ELM_FORCE index,double speed);

/*--- XeForceSetAnchor() ---*/
/*! Ustala kotwice (substancje z ktora jest polaczony).
\param index Element sily.
\param anchor Substancja.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeForceSetAnchor(XE_ELM_FORCE index,XE_ELM_SUBSTANCE anchor);

/*--- XeForceGet() ---*/
/*! Pobiera wartosc zmiennej stanu sily.
\param index Element sily.
\param type Zmienna stanu. Dostepne wartosci:<br>
XE_FORCE_ID: Indeks sily.<br>
XE_FORCE_X: Pozycja X.<br>
XE_FORCE_Y: Pozycja Y.<br>
XE_FORCE_Z: Pozycja Z.<br>
XE_FORCE_ALPHA: Kierunek alpha.<br>
XE_FORCE_BETA: Kierunek beta.<br>
XE_FORCE_FORCE: Sila.<br>
XE_FORCE_SPEED: Predkosc.<br>
XE_FORCE_ANCHOR: Kotwica (substancja z ktora jest polaczony).
\return Wskaznik nieokreslony na zmienna stanu sily. Nalezy rzutowac na konkretny typ danej zmiennej stanu.
*/
exportdll void* XeForceGet(XE_ELM_FORCE index,XE_ESTATE type);

/*--- XeForceDestroy() ---*/
/*! Niszczy sile.
\param index Element sily.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeForceDestroy(XE_ELM_FORCE index);

/*--- XeSurfaceCreate() ---*/
/*! Tworzy powierzchnie.
\param elm Wskaznik na element powierzchni. Zwraca do niego utworzony element jesli operacja powiodla sie.
*/
exportdll void XeSurfaceCreate(XE_ELM_SURFACE* elm);

/*--- XeSurfaceSetSubstance() ---*/
/*! Ustala substancje.
\param index Element powierzchni.
\param type Typ substancji. Dostepne wartosci:<br>
XE_SURFACE_SUBSTANCE_1: Pierwsza substancja.<br>
XE_SURFACE_SUBSTANCE_2: Druga substancja.<br>
XE_SURFACE_SUBSTANCE_3: Trzecia substancja.
\param substance Element substancji.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeSurfaceSetSubstance(XE_ELM_SURFACE index,XE_ESTATE type,XE_ELM_SUBSTANCE substance);

/*--- XeSurfaceSetTexture() ---*/
/*! Ustala teksture.
\param index Element powierzchni.
\param texture Element tekstury.
\param type Typ tekstury. Musi byc: XE_2D.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeSurfaceSetTexture(XE_ELM_SURFACE index,XE_ELM_TEXTURE texture,XE_ESTATE type);

/*--- XeSurfaceSetCoord() ---*/
/*! Ustala koordynaty tekstury.
\param index Element powierzchni.
\param type Typ substancji tekstury. Dostepne wartosci:<br>
XE_SURFACE_TEX_SUBSTANCE_1: Pierwsza substancja tekstury.<br>
XE_SURFACE_TEX_SUBSTANCE_2: Druga substancja tekstury.<br>
XE_SURFACE_TEX_SUBSTANCE_3: Trzecia substancja tekstury.
\param x Koordynat X.
\param y Koordynat Y.
\param z Koordynat Z.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeSurfaceSetCoord(XE_ELM_SURFACE index,XE_ESTATE type,double x,double y,double z);

/*--- XeSurfaceSetDiffusion() ---*/
/*! Ustala rozproszenie swiatla.
\param index Element powierzchni.
\param diffusion Rozproszenie (zakres: <0;1>).
\return true jesli operacja powiodla sie.
*/
exportdll bool XeSurfaceSetDiffusion(XE_ELM_SURFACE index,double diffusion);

/*--- XeSurfaceSetMirror() ---*/
/*! Ustala poziom odbicia swiatla.
\param index Element powierzchni.
\param mirror Poziom odbicia.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeSurfaceSetMirror(XE_ELM_SURFACE index,double mirror);

/*--- XeSurfaceSetOrientation() ---*/
/*! Ustala orientacje.
\param index Element powierzchni.
\param orientation Orientacja (wartosci: -1;0;1).
\return true jesli operacja powiodla sie.
*/
exportdll bool XeSurfaceSetOrientation(XE_ELM_SURFACE index,int orientation);

/*--- XeSurfaceSetOrder() ---*/
/*! Ustala kolejnosc dla kolejki rysowania.
\param index Element powierzchni.
\param order Kolejnosc.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeSurfaceSetOrder(XE_ELM_SURFACE index,double order);

/*--- XeSurfaceGet() ---*/
/*! Pobiera wartosc zmiennej stanu powierzchni.
\param index Element powierzchni.
\param type Zmienna stanu. Dostepne wartosci:<br>
XE_SURFACE_ID: Indeks powierzchni.<br>
XE_SURFACE_SUBSTANCE_1: Pierwsza substancja.<br>
XE_SURFACE_SUBSTANCE_2: Druga substancja.<br>
XE_SURFACE_SUBSTANCE_3: Trzecia substancja.<br>
XE_SURFACE_TEXTURE: Element tekstury.<br>
XE_SURFACE_TEXTURE_TYPE: Typ tekstury.<br>
XE_SURFACE_ORDER: Kolejnosc dla kolejki rysowania.<br>
XE_SURFACE_TEX_SUBSTANCE_1_X: Koordynat X pierwszej tekstury.<br>
XE_SURFACE_TEX_SUBSTANCE_2_X: Koordynat X drugiej tekstury.<br>
XE_SURFACE_TEX_SUBSTANCE_3_X: Koordynat X trzeciej tekstury.<br>
XE_SURFACE_TEX_SUBSTANCE_1_Y: Koordynat Y pierwszej tekstury.<br>
XE_SURFACE_TEX_SUBSTANCE_2_Y: Koordynat Y drugiej tekstury.<br>
XE_SURFACE_TEX_SUBSTANCE_3_Y: Koordynat Y trzeciej tekstury.<br>
XE_SURFACE_TEX_SUBSTANCE_1_Z: Koordynat Z pierwszej tekstury.<br>
XE_SURFACE_TEX_SUBSTANCE_2_Z: Koordynat Z drugiej tekstury.<br>
XE_SURFACE_TEX_SUBSTANCE_3_Z: Koordynat Z trzeciej tekstury.<br>
XE_SURFACE_DIFFUSION: Rozproszenie swiatla.<br>
XE_SURFACE_MIRROR: Poziom odbicia swiatla.<br>
XE_SURFACE_ORIENTATION: Orientacja.
\return Wskaznik nieokreslony na zmienna stanu powierzchni. Nalezy rzutowac na konkretny typ danej zmiennej stanu.
*/
exportdll void* XeSurfaceGet(XE_ELM_SURFACE index,XE_ESTATE type);

/*--- XeSurfaceDestroy() ---*/
/*! Niszczy powierzchnie.
\param index Element powierzchni.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeSurfaceDestroy(XE_ELM_SURFACE index);

/*--- XeSurfaceSort() ---*/
/*! Sortuje powierzchnie.
\param invert Sortowanie odwrotne.
*/
exportdll void XeSurfaceSort(bool invert);

/*--- XeParticleCreate() ---*/
/*! Tworzy czasteczke.
\param elm Wskaznik na element czasteczki. Zwraca do niego utworzony element jesli operacja powiodla sie.
*/
exportdll void XeParticleCreate(XE_ELM_PARTICLE* elm);

/*--- XeParticleSetSubstance() ---*/
/*! Ustala substancje.
\param index Element czasteczki.
\param substance Element substancji.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeParticleSetSubstance(XE_ELM_PARTICLE index,XE_ELM_SUBSTANCE substance);

/*--- XeParticleSetSize() ---*/
/*! Ustala rozmiar.
\param index Element czasteczki.
\param width Szerokosc.
\param height Wysokosc.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeParticleSetSize(XE_ELM_PARTICLE index,double width,double height);

/*--- XeParticleSetTexture() ---*/
/*! Ustala teksture.
\param index Element czasteczki.
\param texture Element tekstury.
\param type Typ tekstury. Musi byc: XE_2D.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeParticleSetTexture(XE_ELM_PARTICLE index,XE_ELM_TEXTURE texture,XE_ESTATE type);

/*--- XeParticleGet() ---*/
/*! Pobiera wartosc zmiennej stanu czasteczki.
\param index Element czasteczki.
\param type Zmienna stanu czasteczki. Dostepne wartosci:<br>
XE_PARTICLE_ID: Indeks czasteczki.<br>
XE_PARTICLE_WIDTH: Szerokosc.<br>
XE_PARTICLE_HEIGHT: Wysokosc.<br>
XE_PARTICLE_TEXTURE: Element tekstury.<br>
XE_PARTICLE_TEXTURE_TYPE: Typ tekstury.<br>
XE_PARTICLE_SUBSTANCE: Element substancji.
\return Wskaznik nieokreslony na zmienna stanu czasteczki. Nalezy rzutowac na konkretny typ danej zmiennej stanu.
*/
exportdll void* XeParticleGet(XE_ELM_PARTICLE index,XE_ESTATE type);

/*--- XeParticleDestroy() ---*/
/*! Niszczy czasteczke.
\param index Element czasteczki.
*/
exportdll bool XeParticleDestroy(XE_ELM_PARTICLE index);

/*--- XeGlobalForceCreate() ---*/
/*! Tworzy sile globalna.
\param elm Wskaznik na element sily globalnej. Zwraca do niego utworzony element jesli operacja powiodla sie.
*/
exportdll void XeGlobalForceCreate(XE_ELM_GLOBAL_FORCE* elm);

/*--- XeGlobalForceSetAlpha() ---*/
/*! Ustala kierunek alpha.
\param index Element sily globalnej.
\param alpha Kat alpha.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeGlobalForceSetAlpha(XE_ELM_GLOBAL_FORCE index,double alpha);

/*--- XeGlobalForceSetBeta() ---*/
/*! Ustala kierunek beta.
\param index Element sily globalnej.
\param beta Kat beta.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeGlobalForceSetBeta(XE_ELM_GLOBAL_FORCE index,double beta);

/*--- XeGlobalForceSetForce() ---*/
/*! Ustala sile.
\param index Element sily globalnej.
\param force Sila.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeGlobalForceSetForce(XE_ELM_GLOBAL_FORCE index,double force);

/*--- XeGlobalForceGet() ---*/
/*! Pobiera wartosc zmiennej stanu sily globalnej.
\param index Element sily globalnej.
\param type Zmienna stanu sily globalnej. Dostepne wartosci:<br>
XE_GLOBAL_FORCE_ID: Indeks sily globalnej.<br>
XE_GLOBAL_FORCE_ALPHA: Kierunek alpha.<br>
XE_GLOBAL_FORCE_BETA: Kierunek beta.<br>
XE_GLOBAL_FORCE_FORCE: Sila.
\return Wskaznik nieokreslony na zmienna stanu sily globalnej. Nalezy rzutowac na konkretny typ danej zmiennej stanu.
*/
exportdll void* XeGlobalForceGet(XE_ELM_GLOBAL_FORCE index,XE_ESTATE type);

/*--- XeGlobalForceDestroy() ---*/
/*! Niszczy sile globalna.
\param index Element sily globalnej.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeGlobalForceDestroy(XE_ELM_GLOBAL_FORCE index);

/*--- XeBlackHoleCreate() ---*/
/*! Tworzy czarna dziure.
\param elm Wskaznik na element czarnej dziury. Zwraca do niego utworzony element jesli operacja powiodla sie.
*/
exportdll void XeBlackHoleCreate(XE_ELM_BLACK_HOLE* elm);

/*--- XeBlackHoleSetPosition() ---*/
/*! Ustala pozycje.
\param index Element czarnej dziury.
\param x Pozycja X.
\param y Pozycja Y.
\param z Pozycja Z.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeBlackHoleSetPosition(XE_ELM_BLACK_HOLE index,double x,double y,double z);

/*--- XeBlackHoleSetForce() ---*/
/*! Ustala sile.
\param index Element czarnej dziury.
\param force Sila.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeBlackHoleSetForce(XE_ELM_BLACK_HOLE index,double force);

/*--- XeBlackHoleSetRange() ---*/
/*! Ustala zasieg.
\param index Element czarnej dziury.
\param range Zasieg.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeBlackHoleSetRange(XE_ELM_BLACK_HOLE index,double range);

/*--- XeBlackHoleSetAnchor() ---*/
/*! Ustala kotwice (czarna dziure z ktora jest polaczona).
\param index Element czarnej dziury.
\param anchor Element czarnej dziury.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeBlackHoleSetAnchor(XE_ELM_BLACK_HOLE index,XE_ELM_BLACK_HOLE anchor);

/*--- XeBlackHoleGet() ---*/
/*! Pobiera wartosc zmiennej stanu czarnej dziury.
\param index Element czarnej dziury.
\param type Zmienna stanu czarnej dziury. Dostepne wartosci:<br>
XE_BLACK_HOLE_ID: Indeks czarnej dziury.<br>
XE_BLACK_HOLE_X: Pozycja X.<br>
XE_BLACK_HOLE_Y: Pozycja Y.<br>
XE_BLACK_HOLE_Z: Pozycja Z.<br>
XE_BLACK_HOLE_FORCE: Sila.<br>
XE_BLACK_HOLE_RANGE: Zasieg.<br>
XE_BLACK_HOLE_ANCHOR: Element czarnej dziury.
\return Wskaznik nieokreslony na zmienna stanu czarnej dziury. Nalezy rzutowac na konkretny typ danej zmiennej stanu.
*/
exportdll void* XeBlackHoleGet(XE_ELM_BLACK_HOLE index,XE_ESTATE type);

/*--- XeBlackHoleDestroy() ---*/
/*! Niszczy czarna dziure.
\param index Element czarnej dziury.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeBlackHoleDestroy(XE_ELM_BLACK_HOLE index);

/*--- XeWaveForceCreate() ---*/
/*! Tworzy sile falowa.
\param elm Wskaznik na element sily falowej. Zwraca do niego utworzony element jesli operacja powiodla sie.
*/
exportdll void XeWaveForceCreate(XE_ELM_WAVE_FORCE* elm);

/*--- XeWaveForceSetPosition() ---*/
/*! Ustala pozycje.
\param index Element sily falowej.
\param x Pozycja X.
\param y Pozycja Y.
\param z Pozycja Z.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeWaveForceSetPosition(XE_ELM_WAVE_FORCE index,double x,double y,double z);

/*--- XeWaveForceSetForce() ---*/
/*! Ustala sile.
\param index Element sily falowej.
\param force Sila.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeWaveForceSetForce(XE_ELM_WAVE_FORCE index,double force);

/*--- XeWaveForceSetRange() ---*/
/*! Ustala zasieg.
\param index Element sily falowej.
\param range Zasieg.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeWaveForceSetRange(XE_ELM_WAVE_FORCE index,double range);

/*--- XeWaveForceSetLength() ---*/
/*! Ustala dlugosc.
\param index Element sily falowej.
\param length Dlugosc.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeWaveForceSetLength(XE_ELM_WAVE_FORCE index,double length);

/*--- XeWaveForceSetSpeed() ---*/
/*! Ustala predkosc.
\param index Element sily falowej.
\param speed Predkosc.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeWaveForceSetSpeed(XE_ELM_WAVE_FORCE index,double speed);

/*--- XeWaveForceSetOrientation() ---*/
/*! Ustala sile.
\param index Element sily falowej.
\param orientation Orientacja. Dostepne wartosci:<br>
XE_WAVE_FORCE_ORIENTATION_IN: Sila dosrodkowa.<br>
XE_WAVE_FORCE_ORIENTATION_OUT: Sila odsrodkowa.<br>
XE_WAVE_FORCE_ORIENTATION_INNER: Sila dosrodkowa dla grzbietu fali.<br>
XE_WAVE_FORCE_ORIENTATION_OUTER: Sila odsrodkowa dla grzbietu fali.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeWaveForceSetOrientation(XE_ELM_WAVE_FORCE index,XE_ESTATE orientation);

/*--- XeWaveForceGet() ---*/
/*! Pobiera wartosc zmiennej stanu sily falowej.
\param index Element sily falowej.
\param type Zmienna stanu sily falowej. Dostepne wartosci:<br>
XE_WAVE_FORCE_ID: Indeks sily falowej.<br>
XE_WAVE_FORCE_X: Pozycja X.<br>
XE_WAVE_FORCE_Y:Pozycja Y.<br>
XE_WAVE_FORCE_Z: Pozycja Z.<br>
XE_WAVE_FORCE_FORCE: Sila.<br>
XE_WAVE_FORCE_RANGE: Zasieg.<br>
XE_WAVE_FORCE_LENGTH: Dlugosc.<br>
XE_WAVE_FORCE_SPEED: Predkosc.<br>
XE_WAVE_FORCE_ORIENTATION: Orientacja.
\return Wskaznik nieokreslony na zmienna stanu sily falowej. Nalezy rzutowac na konkretny typ danej zmiennej stanu.
*/
exportdll void* XeWaveForceGet(XE_ELM_WAVE_FORCE index,XE_ESTATE type);

/*--- XeWaveForceDestroy() ---*/
/*! Niszczy sile falowa.
\param index Element sily falowej.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeWaveForceDestroy(XE_ELM_WAVE_FORCE index);

} // namespace Chaos

/*--- XeInAngleField() ---*/
/*! Sprawdza czy punkt znajduje sie w polu widzenia kamery perspektywistycznej o danym kacie patrzenia.
\param _x Pozycja X.
\param _y Pozycja Y.
\param _z Pozycja Z.
\param _angle Kat pola widzenia.
\return true jesli punkt znajduje sie w polu widzenia.
*/
exportdll bool XeInAngleField(double _x,double _y,double _z,double _angle);

/*--- XeAngleDifference() ---*/
/*! oblicza roznice pomiedzy dwoma katami.
\param angle1 Pierwszy kat.
\param angle2 Drugi kat.
\return Roznica katow.
*/
exportdll double XeAngleDifference(double angle1,double angle2);

/*--- XeCrossProductX() ---*/
/*! Oblicza wartosc X iloczynu wektorowego.
\param x1 Wartosc X pierwszego wektora.
\param y1 Wartosc Y pierwszego wektora.
\param z1 Wartosc Z pierwszego wektora.
\param x2 Wartosc X drugiego wektora.
\param y2 Wartosc Y drugiego wektora.
\param z2 Wartosc Z drugiego wektora.
\return Wartosc X iloczynu wektorowego.
*/
exportdll double XeCrossProductX(double x1,double y1,double z1,double x2,double y2,double z2);

/*--- XeCrossProductY() ---*/
/*! Oblicza wartosc Y iloczynu wektorowego.
\param x1 Wartosc X pierwszego wektora.
\param y1 Wartosc Y pierwszego wektora.
\param z1 Wartosc Z pierwszego wektora.
\param x2 Wartosc X drugiego wektora.
\param y2 Wartosc Y drugiego wektora.
\param z2 Wartosc Z drugiego wektora.
\return Wartosc Y iloczynu wektorowego.
*/
exportdll double XeCrossProductY(double x1,double y1,double z1,double x2,double y2,double z2);

/*--- XeCrossProductZ() ---*/
/*! Oblicza wartosc Z iloczynu wektorowego.
\param x1 Wartosc X pierwszego wektora.
\param y1 Wartosc Y pierwszego wektora.
\param z1 Wartosc Z pierwszego wektora.
\param x2 Wartosc X drugiego wektora.
\param y2 Wartosc Y drugiego wektora.
\param z2 Wartosc Z drugiego wektora.
\return Wartosc Z iloczynu wektorowego.
*/
exportdll double XeCrossProductZ(double x1,double y1,double z1,double x2,double y2,double z2);

/*--- XeDotProduct() ---*/
/*! Oblicza iloczyn skalarny dwoch wektorow.
\param x1 Wartosc X pierwszego wektora.
\param y1 Wartosc Y pierwszego wektora.
\param z1 Wartosc Z pierwszego wektora.
\param x2 Wartosc X drugiego wektora.
\param y2 Wartosc Y drugiego wektora.
\param z2 Wartosc Z drugiego wektora.
\return Iloczyn skalarny.
*/
exportdll double XeDotProduct(double x1,double y1,double z1,double x2,double y2,double z2);

namespace Photon
{

/*--- XeMatrixMode() ---*/
/*! Ustala aktywny tryb macierzy.
\param mode Tryb macierzy. Dostepne wartosci:<br>
XE_MATRIX_PROJECTION: Macierz projekcji.<br>
XE_MATRIX_MODELVIEW: Macierz widoku (domyslna).<br>
XE_MATRIX_TEXTURE: Macierz tekstury.
*/
exportdll void XeMatrixMode(XE_ESTATE mode);

/*--- XeMatrixIdentity() ---*/
/*! Laduje macierz jednostkowa do aktywnej macierzy (zerujemy transformacje). */
exportdll void XeMatrixIdentity();

/*--- XeMatrixPush() ---*/
/*! Odklada bierzaca macierz na stos. */
exportdll void XeMatrixPush();

/*--- XeMatrixPop() ---*/
/*! Pobiera macierz ze stosu i oznaczamy jako bierzaca. */
exportdll void XeMatrixPop();

/*--- XeMatrixRotate() ---*/
/*! Obraca uklad wspolrzednych.
\param angle kat obrotu.
\param x Ilosc pelnych obrotow angle dla osi X.
\param y Ilosc pelnych obrotow angle dla osi Y.
\param z Ilosc pelnych obrotow angle dla osi Z.
*/
exportdll void XeMatrixRotate(double angle,double x,double y,double z);

/*--- XeMatrixScale() ---*/
/*! Skaluje uklad wspolrzednych.
\param x Skala osi X.
\param y Skala osi Y.
\param z Skala osi Z.
*/
exportdll void XeMatrixScale(double x,double y,double z);

/*--- XeMatrixTranslate() ---*/
/*! Przesuwamy uklad wspolrzednych.
\param x Wartosc przesuniecia osi X.
\param y Wartosc przesuniecia osi Y.
\param z Wartosc przesuniecia osi Z.
*/
exportdll void XeMatrixTranslate(double x,double y,double z);

/*--- XeMatrixLoad() ---*/
/*! Laduje macierz.
\param mat Wskaznik na macierz.
*/
exportdll void XeMatrixLoad(double* mat);

/*--- XeMatrixSave() ---*/
/*! Zapisuje macierz do tablicy.
\param type Typ macierzy.
\param mat Wskaznik na macierz.
*/
exportdll void XeMatrixSave(XE_ESTATE type,double* mat);

/*--- XeRenderText() ---*/
/*! Wypisuje tekst czcionka bitmapowa (aktywna tekstura).
\param string Cstring z tekstem do wypisania.
\param x Pozycja X.
\param y Pozycja Y.
*/
exportdll void XeRenderText(char* string,double x,double y);

/*--- XeRenderqueueClear() ---*/
/*! Czysci kolejke wyswietlania. */
exportdll void XeRenderqueueClear();

/*--- XeRenderqueueAdd() ---*/
/*! Dodaje obiekt do kolejki wyswietlania.
\param _object Obiekt.
\param _order Kolejnosc.
*/
exportdll void XeRenderqueueAdd(double _object,double _order);

/*--- XeRenderqueueRemove() ---*/
/*! Usuwa pierwszy obiekt z kolejki i zwraca jego ID.
\return Identyfikator obiektu.
*/
exportdll double XeRenderqueueRemove();

/*--- XeRenderqueueChange() ---*/
/*! Zmienia kolejnosc obiektu.
\param _object Obiekt.
\param _order Nowa kolejnosc.
*/
exportdll void XeRenderqueueChange(double _object,double _order);

/*--- XeRenderqueueSort() ---*/
/*! Sortuje obiekty w kolejce. */
exportdll void XeRenderqueueSort();

} // namespace Photon

/*--- XeClosure() ---*/
/*! Konczy prace silnika. */
exportdll void XeClosure();

/*--- XeLoggerMode() ---*/
/*! Ustala tryb logowania pracy silnika.
\param _m Tryb logowania. Dostepne wartosci:<br>
'o': Logowanie wylaczone.<br>
'm': Zapis do pamieci.<br>
'c': Zapis do konsoli (jesli program wykorzystuje konsole).<br>
'f': Zapis do pliku tekstowego.<br>
'b': W razie logu typu ERROR wyrzucany jest message box wraz z logiem.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeLoggerMode(int _m);

/*--- XeLoggerClear() ---*/
/*! Czysci log. */
exportdll void XeLoggerClear();

/*--- XeLoggerFlush() ---*/
/*! Wysyla log zapisany w pamieci do konsoli, pliku tekstowego lub pliku RichText HTML.
\param _m Tryb. Dostepne wartosci:<br>
'c': Zapis do konsoli (jesli program wykorzystuje konsole).<br>
'f': Zapis do pliku tekstowego.<br>
'r': Zapis do pliku RichText HTML.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeLoggerFlush(int _m);

/*--- XeLoggerFile() ---*/
/*! Zmienia nazwe pliku logu.
\param _fn Cstring z nazwa pliku.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeLoggerFile(char* _fn);

/*--- XeCryptEncode() ---*/
/*! Szyfruje bufor.
\param buff_src Wskaznik na bufor zrodlowy.
\param buff_dst Wskaznik na bufor docelowy.
\param buff_size Rozmiar (w bajtach) bufora docelowego.
\param key_string Cstring klucza.
\param key_size Dlugosc cstringa klucza.
*/
exportdll void XeCryptEncode(void* buff_src,void* buff_dst,unsigned int buff_size,char* key_string,unsigned int key_size);

/*--- XeCryptDecode() ---*/
/*! Deszyfruje bufor.
\param buff_src Wskaznik na bufor zrodlowy.
\param buff_dst Wskaznik na bufor docelowy.
\param buff_size Rozmiar (w bajtach) bufora docelowego.
\param key_string Cstring klucza.
\param key_size Dlugosc cstringa klucza.
*/
exportdll void XeCryptDecode(void* buff_src,void* buff_dst,unsigned int buff_size,char* key_string,unsigned int key_size);

#ifdef XE_COMPILE_PSYCHE

namespace Psyche
{

/*--- XePsycheCreate() ---*/
/*! Tworzy umysl.
\param elm Wskaznik na element umyslu.
*/
exportdll void XePsycheCreate(XE_ELM_PSYCHE* elm);

/*--- XePsycheCompile() ---*/
/*! Kompiluje skrypt umyslu z pliku.
\param index Element umyslu.
\param fname Nazwa pliku.
\return true jesli operacja powiodla sie.
*/
exportdll bool XePsycheCompile(XE_ELM_PSYCHE index,char* fname);

/*--- XePsycheProgress() ---*/
/*! Przetwarza sygnaly w umysle.
\param index Element umyslu.
\param factor Wspolczynnik predkosci sygnalow
*/
exportdll bool XePsycheProgress(XE_ELM_PSYCHE index,float factor);

/*--- XePsycheClear() ---*/
/*! Czysci umysl.
\param index Element umyslu.
*/
exportdll bool XePsycheClear(XE_ELM_PSYCHE index);

/*--- XePsycheGenNeuron() ---*/
/*! Generuje neuron.
\param index Element umyslu.
\param gate Wartosc bramki.
\param signalspeed Predkosc sygnalu.
\return Identyfikator neuronu.
*/
exportdll unsigned int XePsycheGenNeuron(XE_ELM_PSYCHE index,unsigned int gate,float signalspeed);

/*--- XePsycheGenDetector() ---*/
/*! Generuje detektor.
\param index Element umyslu.
\param gate Wartosc bramki.
\param signalspeed Predkosc sygnalu.
\return Identyfikator detektora.
*/
exportdll unsigned int XePsycheGenDetector(XE_ELM_PSYCHE index,unsigned int gate,float signalspeed);

/*--- XePsycheGenEffector() ---*/
/*! Generuje efektor.
\param index Element umyslu.
\param gate Wartosc bramki.
\return Identyfikator efektora.
*/
exportdll unsigned int XePsycheGenEffector(XE_ELM_PSYCHE index,unsigned int gate);

/*--- XePsycheSendToDetector() ---*/
/*! Wysyla sygnal do detektora.
\param index Element umyslu.
\param detector Identyfikator detektora.
\param signal Wartosc sygnalu do przetworzenia.
\return true jesli operacja powiodla sie.
*/
exportdll bool XePsycheSendToDetector(XE_ELM_PSYCHE index,unsigned int detector,unsigned int signal);

/*--- XePsycheReceiveFromEffector() ---*/
/*! Pobiera sygnal z efektora.
\param index Element umyslu.
\param detector Identyfikator detektora.
\return Wartosc przetworzonego sygnalu.
*/
exportdll unsigned int XePsycheReceiveFromEffector(XE_ELM_PSYCHE index,unsigned int effector);

/*--- XePsycheDetectorLinkToNeuron() ---*/
/*! Laczy detektor z neuronem.
\param index Element umyslu.
\param src Identyfikator detektoru zrodlowego.
\param dest Identyfikator neuronu docelowego.
\return true jesli operacja powiodla sie.
*/
exportdll bool XePsycheDetectorLinkToNeuron(XE_ELM_PSYCHE index,unsigned int src,unsigned int dest);

/*--- XePsycheNeuronLinkToNeuron() ---*/
/*! Laczy neuron z neuronem.
\param index Element umyslu.
\param src Identyfikator neuronu zrodlowego.
\param dest Identyfikator neuronu docelowego.
\return true jesli operacja powiodla sie.
*/
exportdll bool XePsycheNeuronLinkToNeuron(XE_ELM_PSYCHE index,unsigned int src,unsigned int dest);

/*--- XePsycheNeuronLinkToEffector() ---*/
/*! Laczy neuron z efektorem.
\param index Element umyslu.
\param src Identyfikator neuronu zrodlowego.
\param dest Identyfikator efektoru docelowego.
\return true jesli operacja powiodla sie.
*/
exportdll bool XePsycheNeuronLinkToEffector(XE_ELM_PSYCHE index,unsigned int src,unsigned int dest);

#ifdef XE_COMPILE_PSYCHE_ALLOW_TO_EVOLVE

/*--- XePsycheSetEvolution() ---*/
/*! Ustala czy umysl ma ewoluowac samoistnie (uczyc sie).
\param index Element umyslu.
\param evo Ewoluowanie.
\return true jesli operacja powiodla sie.
*/
exportdll bool XePsycheSetEvolution(XE_ELM_PSYCHE index,bool evo);

/*--- XePsycheGetEvolution() ---*/
/*! Sprawdza czy umysl ewoluuje samoistnie (uczy sie).
\param index Element umyslu.
\return true jesli umysl ewoluuje.
*/
exportdll bool XePsycheGetEvolution(XE_ELM_PSYCHE index);

#endif /* XE_COMPILE_PSYCHE_ALLOW_TO_EVOLVE */

/*--- XePsycheDestroy() ---*/
/*! Niszczy umysl.
\param index Element umyslu.
\return true jesli operacja powiodla sie.
*/
exportdll bool XePsycheDestroy(XE_ELM_PSYCHE index);

} // namespace Psyche

#endif /* XE_COMPILE_PSYCHE */

/*--- XeCurve() ---*/
/*! Oblicza wartosc krzywej na danej pozycji.
\param pos Pozycja na krzywej (zakres: <0;1>).
\param count Liczba punktow kontrolnych.
\param points Wskaznik na tablice punktow kontrolnych.
\param format Format punktow (ilosc punktow w strukturze).
\param offset Odstep od pierwszego punktu w strukturze.
\return Wartosc krzywej.
*/
exportdll double XeCurve(double pos,unsigned int count,double* points,unsigned int format,unsigned int offset);

/*--- XeDataPack() ---*/
/*! Pakuje (kompresuje) dane do nowego buforu i zwraca wskaznik na niego.
\param data Wskaznik na bufor danych surowych.
\param sizeraw Wielkosc danych surowych.
\param sizecomp Referencja na wielkosc danych spakowanych.
\param password Cstring z haslem.
\param passsize Dlugosc hasla.
\return Wskaznik na utworzony bufor.
*/
exportdll void* XeDataPack(void* data,unsigned int sizeraw,unsigned int& sizecomp,char* password,unsigned int passsize);

/*--- XeDataUnpack() ---*/
/*! Pakuje (kompresuje) dane do nowego buforu i zwraca wskaznik na niego.
\param data Wskaznik na bufor danych surowych.
\param sizeraw Referencja na wielkosc danych surowych.
\param sizecomp Wielkosc danych spakowanych.
\param password Cstring z haslem.
\param passsize Dlugosc hasla.
\return Wskaznik na utworzony bufor.
*/
exportdll void* XeDataUnpack(void* data,unsigned int& sizeraw,unsigned int sizecomp,char* password,unsigned int passsize);

/*--- XeElementsCount() ---*/
/*! Zwraca ilosc elementow danego typu.
\param element Typ elementu.
\return Ilosc elementow.
*/
exportdll unsigned int XeElementsCount(XE_ESTATE element);

/*--- XeElementsFirst() ---*/
/*! Pobiera pierwszy element danego typu.
\param type Typ elementu.
\param elm Wskaznik na element.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeElementsFirst(XE_ESTATE type,void* elm);

/*--- XeElementsLast() ---*/
/*! Pobiera ostatni element danego typu.
\param type Typ elementu.
\param elm Wskaznik na element.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeElementsLast(XE_ESTATE type,void* elm);

namespace Photon
{

/*--- XeDrawData() ---*/
/*! Rysuje prymitywy pobierajac ich dane z bufora. Upakowanie danych: texture4;color4;normal3;vertex4.
\param mode Typ prymitywow.
\param first Indeks pierwszego elementu.
\param count Liczba elementow do narysowania.
\param vertex Wskaznik na bufor wierzcholkow (4).
\param color Wskaznik na bufor koloru (4).
\param coord Wskaznik na bufor koordynatow tekstury (4).
\param normal Wskaznik na bufor normalnych (3).
*/
exportdll void XeDrawData(XE_ESTATE mode,int first,int count,void* vertex,void* color,void* coord,void* normal);

/*--- XeDrawSurfaces() ---*/
/*! Rysuje powierzchnie, wedle podanych elementow granicznych.
\param first Pierwszy element. Jesli element jest pusty, to zaczyna od pierwszego elementu powierzchni.
\param last Ostatni element. Jesli element jest pusty, to konczy na ostatnim elemencie powierzchni.
*/
exportdll void XeDrawSurfaces(XE_ELM_SURFACE first,XE_ELM_SURFACE last);

/*--- XeGenerateFont() ---*/
/*! Generuje czcionke systemowa do podanej tablicy 256 istniejacych list wyswietlania.
\param fontlists Wskaznik na element list wyswietlania.
\param fontname Nazwa czcionki systemowej.
\param size Rozmiar czcionki.
\param direction Kierunek.
\param weight Grubosc czcionki (<0;1>, gdzie 0 oznacza domyslna grubosc).
\param flags Flagi rodzaju dekoracji czcionki. Dostepne wartosci:
XE_FLAG_ITALIC: pochylona;
XE_FLAG_UNDERLINE: podkreslona;
XE_FLAG_STRIKEOUT: przekreslona.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeGenerateFont(XE_ELM_DISPLAYLIST* fontlists,char* fontname,int size,int direction,float weight,unsigned int flags);

/*--- XeDrawFont() ---*/
/*! Rysuje tekst wybrana czcionka.
\param fontlists Wskaznik na tablice 256 list wyswietlania.
\param text Tekst do wyrenderowania.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeDrawFont(XE_ELM_DISPLAYLIST* fontlists,char* text);

} // namespace Photon

#ifdef XE_COMPILE_TIMEBRIDGE

namespace TimeBridge
{

/*--- XeRandomize() ---*/
/*! Losuje wartosc liczbowa z przedzialu <0;1>.
\param time Czas.
\param seed Ziarno.
\return Wylosowana wartosc liczbowa.
*/
exportdll double XeRandomize(double time, double seed);

/*--- XeAccelerateSubstance() ---*/
/*! Nadaje substancji przyspieszenie czasowe.
\param elm Element substancji.
\param timerel Przyspieszenie czasowe.
\return true jesli operacja powiodla sie.
*/
exportdll bool XeAccelerateSubstance(XE_ELM_SUBSTANCE elm,double timerel);

} // namespace TimeBridge

#endif /* XE_COMPILE_TIMEBRIDGE */

#endif /* XE_NOT_USE_LIBRARY */

} // namespace XeCore

#endif /* _XENON_CORE_H_ */
